// 
// LowEvents.c
// 
// 
// This program:
//
// 	Contains a main event loop function, together with subsidiary functions which
//		perform nominal handling only of low-level and Operating System events.
//
// 	Opens a window in which the types of all received low-level and Operating System
//		events are displayed.
//
// 	Terminates when the user clicks the window's close box.
//
// Event handling is only nominal in this program because its main purpose is to
// demonstrate the basics of an application's main event loop.
//
// Programs in later chapters demonstrate the full gamut of individual event handling. 
//
// The program utilises the following resources:
//
// 	A 'WIND' resource (purgeable).
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible flags set.
//  
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <DiskInit.h>	

//  defines

#define	rWindowResource	128

#define topLeft(r)			(((Point *) &(r))[0])
#define botRight(r)			(((Point *) &(r))[1])

//  global variables

Boolean	gDone;
Boolean	gInBackground;
Boolean gMouseMoved = false;

//  function prototypes

void	main						(void);
void	doInitManagers	(void);
void	doNewWindow			(void);
void	eventLoop				(void);
void	doEvents				(EventRecord *);
void	doMouseDown			(EventRecord *);
void	doUpdate				(EventRecord *);
void	doDisk					(EventRecord *);
void	doOSEvent				(EventRecord *);
void	drawEventString	(Str255);
void	doAdjustCursor	(WindowPtr,RgnHandle);

//  main

void  main(void)
{
	doInitManagers();
	doNewWindow();
	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
	FlushEvents(everyEvent,0);
}

//  doNewWindow

void  doNewWindow(void)
{
	WindowPtr 	windowPtr;

	if(!(windowPtr = GetNewWindow(rWindowResource,NULL,(WindowPtr) -1)))
	{
		SysBeep(10);
		ExitToShell();
	}

	SetPort(windowPtr);
	TextSize(10);
}

//  eventLoop

void  eventLoop(void)
{
	RgnHandle		cursorRegion;
	EventRecord	eventRec;
	Boolean			gotEvent;
	
	gDone = false;
	cursorRegion = NewRgn();
	
	while(!gDone)
	{
		if(!gInBackground && gMouseMoved)
		{
			doAdjustCursor(FrontWindow(),cursorRegion);
			gMouseMoved = false;
		}
			
		gotEvent = WaitNextEvent(everyEvent,&eventRec,180,cursorRegion);
		if(gotEvent)
			doEvents(&eventRec);
	}
}

//  doEvent

void	doEvents(EventRecord *eventRecPtr)
{
	switch(eventRecPtr->what)
	{
		case mouseDown:
			drawEventString("\p    mouseDown");
			doMouseDown(eventRecPtr);
			break;

		case mouseUp:
			drawEventString("\p    mouseUp");
			break;

		case keyDown:
			drawEventString("\p    keyDown");
			break;

		case autoKey:
			drawEventString("\p    autoKey");
			break;

		case updateEvt:
			drawEventString("\p    updateEvt");
			doUpdate(eventRecPtr);
			break;

		case diskEvt:
			drawEventString("\p    diskEvt");
			doDisk(eventRecPtr);
			break;

		case activateEvt:
			drawEventString("\p    activateEvt");
			break;

		case osEvt:
			drawEventString("\p    osEvt - ");
			doOSEvent(eventRecPtr);
			break;
	}
}
	
//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	SInt16		partCode;
	WindowPtr	windowPtr;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where))
				gDone = true;
			break;
	}
}

//  doUpdate

void	doUpdate(EventRecord *eventRecPtr)
{
	BeginUpdate((WindowPtr)eventRecPtr->message);
	EndUpdate((WindowPtr)eventRecPtr->message);
}

//  doDisk

void	doDisk(EventRecord *eventRecPtr)
{
	Point	thePoint;
	OSErr	osErr;

	if(HiWord(eventRecPtr->message) != noErr)
	{
		SetPt(&thePoint,120,120);
		osErr = DIBadMount(thePoint,eventRecPtr->message);
	}
	else
	{
		// Attempt to mount was successful.  Record drive number for accessing the disk, etc.
	}
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			if((eventRecPtr->message & resumeFlag) == 1)
			{				
				gInBackground = false;
				DrawString("\pResume event");
			}
			else
			{
				gInBackground = true;
				DrawString("\pSuspend event");
			}
			break;
				
		case mouseMovedMessage:
			gMouseMoved = true;
			DrawString("\pMouse-moved event");
			break;
	}
}

//  drawEventString

void	drawEventString(Str255 eventString)
{
	RgnHandle	tempRegion;
	WindowPtr	windowPtr;
	
	windowPtr = FrontWindow();
	tempRegion = NewRgn();

	ScrollRect(&windowPtr->portRect,0,-15,tempRegion);
	DisposeRgn(tempRegion);
	
	MoveTo(8,291);
	DrawString(eventString);
}

//  doAdjustCursor

void  doAdjustCursor(WindowPtr frontWindow,RgnHandle mouseRegion)
{
	RgnHandle	myArrowRegion;
	RgnHandle	myIBeamRegion;
	Rect			cursorRect;
	Point			mousePt;

	myArrowRegion = NewRgn();
	myIBeamRegion = NewRgn();	
	SetRectRgn(myArrowRegion,-32768,-32768,32766,32766);
	
	cursorRect = frontWindow->portRect;
	LocalToGlobal(&topLeft(cursorRect));
	LocalToGlobal(&botRight(cursorRect));	

	RectRgn(myIBeamRegion,&cursorRect);
	DiffRgn(myArrowRegion,myIBeamRegion,myArrowRegion);
	
	GetMouse(&mousePt);
	LocalToGlobal(&mousePt);
	if(PtInRgn(mousePt,myIBeamRegion))
	{
		SetCursor(*(GetCursor(iBeamCursor)));
		CopyRgn(myIBeamRegion,mouseRegion);
	}
	else
	{
		SetCursor(&qd.arrow);
		CopyRgn(myArrowRegion,mouseRegion);
	}
	
	DisposeRgn(myArrowRegion);
	DisposeRgn(myIBeamRegion);
}

// 
