// 
// Windows.c
// 
// 
// This program:
//
// 	Allows the user to open any number of zoomDocProc windows, up to the maximum
//		specified in the global variable kMaxWindows, using the File menu Open Command or
//		its keyboard equivalent.
//
// 	Allows the user to close opened windows using the close box, the File menu Close
// 		command or the Close command's keyboard equivalent.
//
// 	Adds menu items representing each window to a Windows	menu as each window is
//		opened  (A keyboard equivalent is included in each menu item for windows 1 to 9.)
//
// 	Deletes menu items from the Windows menu as each window is closed.
//
// 	Fills each window with one of the system patterns as a means of proving, for 
//		demonstration purposes, the window update process.
//
// 	Facilitates activation of a window by mouse selection.
//
// 	Facilitates activation of a window by Windows menu selection.
//
// 	Correctly performs all dragging, zooming and sizing operations.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Windows menus 
//		(preload, non-purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially not visible).  
//
// 	An 'ALRT' resource and 'DITL' resource for use by Stop Alerts (purgeable).  
//
// 	A 'STR#' resource containing strings for the Stop Alerts (purgeable).  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch,
//		and is32BitCompatible flags set.	  
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>

//  defines

#define mApple				128
#define  iAbout				1
#define mFile					129
#define mEdit					130
#define  iNew					1
#define  iClose				4
#define  iQuit				11
#define mWindows			131
#define rNewWindow		128
#define rMenubar			128
#define rAlertBox			128	
#define rStringList		128
#define  sUntitled		1
#define  eMaxWindows	2
#define  eFailWindow	3
#define  eFailMenus		4
#define  eFailMemory	5
#define kMaxWindows		10
#define MAXLONG				0x7FFFFFFF

#define MIN(a,b) 			((a) < (b) ? (a) : (b))

//  global variables

Boolean		gDone;
Boolean		gInBackground;
Ptr				gPreAllocatedBlockPtr;
SInt32		gUntitledWindowNumber = 0;
SInt32		gCurrentNumberOfWindows = 0;
WindowPtr	gWindowPtrArray[kMaxWindows + 2];

//  function prototypes

void	main										(void);
void	doInitManagers					(void);
void	eventLoop								(void);
void	doEvents								(EventRecord *);
void	doMouseDown							(EventRecord *);
void	doUpdate								(EventRecord *);
void	doUpdateWindow					(EventRecord *);
void	doActivate							(EventRecord *);
void	doActivateWindow				(WindowPtr,Boolean);
void	doOSEvent								(EventRecord *);
void	doMenuChoice						(SInt32);
void	doFileMenu							(SInt16);
void	doWindowsMenu						(SInt16);
void	doNewWindow							(void);
void	doCloseWindow						(void);
void	invalidateScrollBarArea	(WindowPtr);
void	setStandardState				(WindowPtr);
void	concatPStrings					(Str255,Str255);
void	doError									(SInt16);

//  main

void  main(void)
{
	Handle			menubarHdl;
	MenuHandle	menuHdl;
	SInt16 			a;

	//  get nonrelocatable block low in heap for first window record

	if(!(gPreAllocatedBlockPtr = NewPtr(sizeof(WindowRecord))))
		doError(eFailMemory);

	//  initialise managers

	doInitManagers();

	//  set up menu bar and menus
	
	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		doError(eFailMenus);
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		doError(eFailMenus);
	else
		AppendResMenu(menuHdl,'DRVR');

	//  initialize window pointer array

	for(a=0;a<kMaxWindows+2;a++)
		gWindowPtrArray[a] = NULL;

	//  enter eventLoop

	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();
	MoreMasters();
	
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord	eventRec;

	gDone = false;
	
	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL))
			doEvents(&eventRec);

		if(gPreAllocatedBlockPtr == NULL)
			if(!(gPreAllocatedBlockPtr = NewPtr(sizeof(WindowRecord))))
				doError(eFailMemory);
	}
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	SInt8	charCode;

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
				doMenuChoice(MenuKey(charCode));
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case activateEvt:
			doActivate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			HiliteMenu(0);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;
	Rect			growRect;
	SInt32		newSize;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inMenuBar:
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;
			
		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			break;
			
		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				doCloseWindow();
			break;

		case inGrow:
			growRect = qd.screenBits.bounds;
			growRect.top 	= 80; 
			growRect.left = 160;
			newSize = GrowWindow(windowPtr,eventRecPtr->where,&growRect);
			if(newSize != 0)
			{
				invalidateScrollBarArea(windowPtr);
				SizeWindow(windowPtr,LoWord(newSize),HiWord(newSize),true);
				invalidateScrollBarArea(windowPtr);
			}
			break;

		case inZoomIn:
		case inZoomOut:
			if(TrackBox(windowPtr,eventRecPtr->where,partCode))
			{
				SetPort(windowPtr);
				EraseRect(&windowPtr->portRect);
				ZoomWindow(windowPtr,partCode,false);
				InvalRect(&windowPtr->portRect);
			}
			break;
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;

	windowPtr = (WindowPtr)eventRecPtr->message;

	BeginUpdate(windowPtr);

	if(!EmptyRgn(windowPtr->visRgn))
	{
		SetPort(windowPtr);
		EraseRgn(windowPtr->visRgn);
		doUpdateWindow(eventRecPtr);
		DrawGrowIcon(windowPtr);
	}

	EndUpdate(windowPtr);
}

//  doUpDateWindow

void  doUpdateWindow(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Rect 			paintRect;
	SInt32		windowRefCon;
	Pattern		fillPattern;

	windowPtr = (WindowPtr)eventRecPtr->message;
	SetPort(windowPtr);	

	paintRect=windowPtr->portRect;
	paintRect.right -= 15;
	paintRect.bottom -= 15;

	windowRefCon = GetWRefCon(windowPtr);

	GetIndPattern(&fillPattern,0,windowRefCon+9);
	FillRect(&paintRect,&fillPattern);
}

//  doActivate

void  doActivate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventRecPtr->message;

	becomingActive = ((eventRecPtr->modifiers & activeFlag) == activeFlag);

	doActivateWindow(windowPtr,becomingActive);
}

//  doActivateWindow

void  doActivateWindow(WindowPtr windowPtr,Boolean becomingActive)
{
	MenuHandle	windowsMenu;
	SInt16			menuItem, a = 1;

	windowsMenu = GetMenuHandle(mWindows);

	while(gWindowPtrArray[a] != windowPtr)
		a++;		
	menuItem = a;
	
	if(becomingActive)
		CheckItem(windowsMenu,menuItem,true);
	else
		CheckItem(windowsMenu,menuItem,false);

	DrawGrowIcon(FrontWindow());
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			if(gCurrentNumberOfWindows > 0)
			{
				DrawGrowIcon(FrontWindow());
				gInBackground = (eventRecPtr->message & resumeFlag) == 0;
				doActivateWindow(FrontWindow(),!gInBackground);
			}
			break;
	}
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;
	 		
	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetItem(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;
			
		case mFile:
			doFileMenu(menuItem);
			break;
			
		case mWindows:
			doWindowsMenu(menuItem);
			break;		
	}

	HiliteMenu(0);
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	switch(menuItem)
	{
		case iNew:
			doNewWindow();
			break;
		
		case iClose:
			doCloseWindow();
			break;			
			
		case iQuit:
			gDone = true;
			break;
	}
}

//  doWindowsMenu

void	doWindowsMenu(SInt16 menuItem)
{
	WindowPtr	windowPtr;

	windowPtr = gWindowPtrArray[menuItem];
	SelectWindow(windowPtr);	
}

//  doNewWindow

void  doNewWindow(void)
{
	WindowPtr		windowPtr;
	Str255			untitledString;
	Str255			numberAsString;
	MenuHandle	windowsMenu;

	if(gCurrentNumberOfWindows == kMaxWindows)
	{
		doError(eMaxWindows);
		return;
	}

	if(!(windowPtr = GetNewWindow(rNewWindow,gPreAllocatedBlockPtr,(WindowPtr)-1)))
		doError(eFailWindow);
	
	gPreAllocatedBlockPtr = NULL;

	GetIndString(untitledString,rStringList,sUntitled);
	NumToString(++gUntitledWindowNumber,numberAsString);
	concatPStrings(untitledString,numberAsString);
	SetWTitle(windowPtr,untitledString);

	setStandardState(windowPtr);

	ShowWindow(windowPtr);

	if(gUntitledWindowNumber <10)
	{
		concatPStrings(untitledString,"\p/");
		NumToString(gUntitledWindowNumber,numberAsString);
		concatPStrings(untitledString,numberAsString);
	}
	windowsMenu = GetMenu(mWindows);
	InsertMenuItem(windowsMenu,untitledString,CountMItems(windowsMenu));		

	SetWRefCon(windowPtr,gCurrentNumberOfWindows);

	gCurrentNumberOfWindows ++;
	gWindowPtrArray[gCurrentNumberOfWindows] = windowPtr;	

	if(gCurrentNumberOfWindows == 1)
	{
		EnableItem(GetMenu(mFile),iClose);
		EnableItem(GetMenu(mWindows),0);
		DrawMenuBar();
	}
}

//  doCloseWindow

void  doCloseWindow(void)
{
	WindowPtr		windowPtr;
	MenuHandle	windowsMenu;
	SInt16			a = 1;

	windowPtr = FrontWindow();
	CloseWindow(windowPtr);
	DisposePtr((Ptr) (WindowPeek) windowPtr);
	gCurrentNumberOfWindows --;

	windowsMenu = GetMenu(mWindows);
	while(gWindowPtrArray[a] != windowPtr)
		a++;
	gWindowPtrArray[a] = NULL;
	DeleteMenuItem(windowsMenu,a);

	for(a=1;a<kMaxWindows+1;a++)
	{
		if(gWindowPtrArray[a] == NULL)
		{
			gWindowPtrArray[a] = gWindowPtrArray[a+1];
			gWindowPtrArray[a+1] = NULL;
		}
	}	

	if(gCurrentNumberOfWindows == 0)
	{
		DisableItem(GetMenu(mFile),iClose);
		DisableItem(GetMenu(mWindows),0);
		DrawMenuBar();
	}
}

//  invalidateScrollBarArea

void  invalidateScrollBarArea(WindowPtr windowPtr)
{
	Rect	tempRect;
	
	SetPort(windowPtr);

	tempRect = windowPtr->portRect;
	tempRect.left = tempRect.right - 15;
	InvalRect(&tempRect);

	tempRect = windowPtr->portRect;
	tempRect.top = tempRect.bottom - 15;
	InvalRect(&tempRect);
}	

//  setStandardState

void  setStandardState(WindowPtr windowPtr)
{
	WindowPeek	windowRecPtr;
	WStateData	*winStateDataPtr;
	Rect				tempRect;

	tempRect = qd.screenBits.bounds;	
	windowRecPtr = (WindowPeek) windowPtr;
	winStateDataPtr = (WStateData *) *(windowRecPtr->dataHandle);

	SetRect(&(winStateDataPtr->stdState),tempRect.left+40,tempRect.top+60,
					tempRect.right-40,tempRect.bottom-40);
}

//  concatPStrings

void  concatPStrings(Str255 targetString,Str255 appendString)
{
	SInt16	appendLength;

	appendLength = MIN(appendString[0],255 - targetString[0]);

	if(appendLength > 0)
	{
		BlockMoveData(appendString+1,targetString+targetString[0]+1,(SInt32) appendLength);
		targetString[0] += appendLength;
	}
}

//  doError

void  doError(SInt16 errorType)
{
	Str255	errorMessage;

	GetIndString(errorMessage,rStringList,errorType);
	ParamText(errorMessage,NULL,NULL,NULL);

	if(errorType == eMaxWindows)
		CautionAlert(rAlertBox,NULL);
	else
	{
		StopAlert(rAlertBox,NULL);
		ExitToShell();	
	}
}

// 

