// 
// BasicQuickDraw.c
// 
// 
// This program:
//
// 	Opens a window in which the results of various basic QuickDraw drawing operations
//		are displayed.
//
//		Individual drawing operations (eg, draw lines, draw rectangles, draw polygons, etc)
//		are selected from a pull-down menu titled "Demonstration".
//
// 	Quits when the user selects Quit from the File menu or clicks the window's close
//		box.
//
// The program utilises the following resources:
//
// 	'WIND' resources for the main window, and a small window used for the CopyBits
//		demonstration (purgeable) (initially visible).
//
// 	An 'MBAR' resource and associated 'MENU' resources (preload, non-purgeable).
//
// 	Two 'ICON' resources (purgeable) used for the transfer modes demonstration. 
//
// 	A 'PICT' resource (purgeable) used for the CopyBits demonstration.
//
// 	'STR#' resources (purgeable) containing strings used by the CopyBits and text
//		demonstrations.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>

//  defines

#define mApple					128
#define mFile						129
#define  iQuit					11
#define mDemonstration	131
#define  iLine					1
#define  iRectAndOval		2
#define  iArcAndWedge		3
#define  iPolygon				4
#define  iRegion				5
#define  iTransferMode	6
#define  iCopyBits			7
#define  iText					8
#define  iBasicColour		9
#define  iDrawWithMouse	10
#define rMenubar				128
#define rWindow					128
#define rSmallWindow		129
#define rCrossIcon			128
#define rSquareIcon			129
#define rModeStringList	128
#define rTextStringList	129
#define rPicture				128

#define MAXLONG					0x7FFFFFFF

//  global variables

Boolean		gDone;
WindowPtr	gWindowPtr;
Boolean		gDrawWithMouseActivated;

//  function prototypes

void		main								(void);
void		doInitManagers			(void);
void		doMouseDown					(EventRecord *);
void		doEvents						(EventRecord *);
void		doMenuChoice				(SInt32);
void		doDemonstrationMenu	(SInt16);
void		doLines							(void);
SInt16	doRandomNumber			(SInt16);
void		doRectOval					(void);
void		doArcWedge					(void);
void		doPolygon						(void);
void		doRegion						(void);
void		doTransferMode			(void);
void		doCopyBits					(void);
void		doText							(void);
void		doBasicColours			(void);
void		doDrawWithMouse			(void);

//  main

void  main(void)
{
	Handle			menubarHdl;
	MenuHandle	menuHdl;
	EventRecord	eventRec;
	Boolean			gotEvent;		

	//  initialise managers

	doInitManagers();
	
	//  see random number generator
	
	GetDateTime((UInt32 *) (&qd.randSeed));
	
	//  set up menu bar and menus
	
	if(!(menubarHdl = GetNewMBar(rMenubar)))
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	if(!(menuHdl = GetMenuHandle(mApple)))
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open window

	if(!(gWindowPtr = GetNewWindow(rWindow,NULL,(WindowPtr)-1)))
		ExitToShell();

	SetPort(gWindowPtr);

	TextSize(10);

	//  eventLoop

	gDone = false;

	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL);
		if(gotEvent)
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{		
	WindowPtr	windowPtr;
	SInt8			charCode;

	windowPtr = (WindowPtr) eventRecPtr->message;
	
	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
				doMenuChoice(MenuKey(charCode));
			break;

		case updateEvt:
			BeginUpdate(windowPtr);
			EndUpdate(windowPtr);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inMenuBar:
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			else
				if(gDrawWithMouseActivated)
					doDrawWithMouse();
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				gDone = true;
			break;
	}
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
			daDriverRefNum = OpenDeskAcc(itemName);
			break;

		case mFile:
			if(menuItem == iQuit)
				gDone = true;
			break;

		case mDemonstration:
			doDemonstrationMenu(menuItem);
			break;
	}
	
	HiliteMenu(0);
}

//  doDemonstrationMenu

void	doDemonstrationMenu(SInt16 menuItem)
{
	gDrawWithMouseActivated = false;

	switch(menuItem)
	{
		case iLine:
			doLines();
			break;

		case iRectAndOval:
			doRectOval();
			break;			

		case iArcAndWedge:
			doArcWedge();
			break;

		case iPolygon:
			doPolygon();
			break;

		case iRegion:
			doRegion();
			break;

		case iTransferMode:
			doTransferMode();
			break;

		case iCopyBits:
			doCopyBits();
			break;

		case iText:
			doText();
			break;

		case iBasicColour:
			doBasicColours();
			break;

		case iDrawWithMouse:
			FillRect(&(gWindowPtr->portRect),&qd.white);
			MoveTo(10,25);
			DrawString("\pClick in the window and drag the mouse to the right and down");  
			gDrawWithMouseActivated = true;
			break;
	}
}

//  doLines

void  doLines(void)
{
	SInt16		top, left, bottom, right, a, b, c;
	RgnHandle	oldClipRgn;
	Rect			newClipRect;
	Pattern		systemPattern;
	SInt32		finalTicks;

	FillRect(&(gWindowPtr->portRect),&qd.white);

	PenMode(patCopy);
		
	left		= gWindowPtr->portRect.left + 10;
	top			= gWindowPtr->portRect.top + 10;
	right		= gWindowPtr->portRect.right - 10;
	bottom	= gWindowPtr->portRect.bottom - 10;

	oldClipRgn = NewRgn();
	GetClip(oldClipRgn);
	SetRect(&newClipRect,left,top,right,bottom);
	ClipRect(&newClipRect);

	for(a=1;a<39;a++)
	{
		b = doRandomNumber(gWindowPtr->portRect.right - gWindowPtr->portRect.left);
		c = doRandomNumber(gWindowPtr->portRect.right - gWindowPtr->portRect.left);

		GetIndPattern(&systemPattern,sysPatListID,a);
		PenPat(&systemPattern);
		PenSize(a * 2,1);		

		MoveTo(b,gWindowPtr->portRect.top);
		LineTo(c,gWindowPtr->portRect.bottom);

		Delay(15,&finalTicks);
	}		

	SetClip(oldClipRgn);
	DisposeRgn(oldClipRgn);

	SetWTitle(gWindowPtr,"\pClick Mouse for More Lines");
	while(!Button()) ;	
	SetWTitle(gWindowPtr,"\pBasic QuickDraw");

	FillRect(&(gWindowPtr->portRect),&qd.white);
	PenSize(1,1);		
	PenPat(&qd.black);
	PenMode(patXor);

	for(a=left,b=right;a<right+1;a++,b--)
	{
		MoveTo(a,top);
		LineTo(b,bottom);
	}

	for(a=bottom,b=top;b<bottom+1;a--,b++)
	{
		MoveTo(left,a);
		LineTo(right,b);
	}
}

//  doRandomNumber

SInt16  doRandomNumber(SInt16 range)
{
	SInt32 randomNumber;

	randomNumber = Random();
	if(randomNumber < 0)
		randomNumber *= -1;

	return((randomNumber * range) / 32767);
}

//  doRectOval

void  doRectOval(void)
{
	Rect		theRect;
	SInt32	finalTicks;
	Pattern	systemPattern;

	FillRect(&(gWindowPtr->portRect),&qd.white);

	PenPat(&qd.black);
	PenSize(10,20);
	PenMode(patCopy);

	SetRect(&theRect,10,20,245,130);

	MoveTo(10,15);
	DrawString("\pFrameRect");
	FrameRect(&theRect);
	Delay(30,&finalTicks);

	MoveTo(255,15);
	DrawString("\pPaintRect");
	OffsetRect(&theRect,245,0);
	PenPat(&qd.ltGray);
	PaintRect(&theRect);
	Delay(30,&finalTicks);

	MoveTo(10,154);
	DrawString("\pFillRoundRect");
	OffsetRect(&theRect,-245,140);
	GetIndPattern(&systemPattern,sysPatListID,12);
	FillRoundRect(&theRect,120,60,&systemPattern);
	Delay(30,&finalTicks);

	MoveTo(255,154);
	DrawString("\pFrameOval");
	OffsetRect(&theRect,245,0);
	PenSize(40,20);
	PenPat(&qd.dkGray);
	FrameOval(&theRect);

	SetWTitle(gWindowPtr,"\pClick Mouse For Invert and Erase");
	while(!Button()) ;
	SetWTitle(gWindowPtr,"\pBasic QuickDraw");
	SetRect(&theRect,10,145,490,154);
	EraseRect(&theRect);
	SetRect(&theRect,255,160,490,270);
	Delay(30,&finalTicks);

	MoveTo(10,154);
	DrawString("\pInvertRoundRect");
	OffsetRect(&theRect,-245,0);
	InvertRoundRect(&theRect,120,60);
	Delay(30,&finalTicks);

	MoveTo(255,154);
	DrawString("\pEraseOval");
	OffsetRect(&theRect,245,0);
	EraseOval(&theRect);
	Delay(30,&finalTicks);
}

//  doArcWedge

void  doArcWedge(void)
{
	Rect		theRect;
	SInt16	a;
	SInt32	finalTicks;
	Pattern	systemPattern;

	FillRect(&(gWindowPtr->portRect),&qd.white);

	PenSize(60,10);
	PenPat(&qd.dkGray);
	PenMode(patCopy);

	SetRect(&theRect,10,20,245,278);

	MoveTo(10,15);
	DrawString("\pFrameArc");
	for(a=0;a<270;a++)
		FrameArc(&theRect,135,a);
	Delay(30,&finalTicks);

	MoveTo(255,15);
	DrawString("\pFillArc");
	OffsetRect(&theRect,245,0);
	GetIndPattern(&systemPattern,sysPatListID,16);
	FillArc(&theRect,315,270,&systemPattern);
	Delay(30,&finalTicks);
	OffsetRect(&theRect,-30,0);
	FillArc(&theRect,225,90,&systemPattern);
}

//  doPolygon

void  doPolygon(void)
{
	PolyHandle	polygonHdl;
	SInt32			finalTicks;
	Pattern			systemPattern;

	FillRect(&(gWindowPtr->portRect),&qd.white);

	PenSize(10,30);
	PenPat(&qd.gray);
	PenMode(patCopy);

	polygonHdl = OpenPoly();
	MoveTo(10,20);
	LineTo(225,40);
	LineTo(100,120);
	LineTo(215,248);	
	LineTo(10,248);
	LineTo(50,200);
	ClosePoly();

	MoveTo(10,15);
	DrawString("\pFramePoly");
	FramePoly(polygonHdl);
	Delay(30,&finalTicks);

	MoveTo(265,15);
	DrawString("\pFillPoly");
	OffsetPoly(polygonHdl,255,0);
	GetIndPattern(&systemPattern,sysPatListID,9);
	FillPoly(polygonHdl,&systemPattern);

	KillPoly(polygonHdl);
}

//  doRegion

void  doRegion(void)
{
	RgnHandle		regionHdl;
	Rect				theRect;
	SInt32			finalTicks;

	FillRect(&(gWindowPtr->portRect),&qd.white);
	PenPat(&qd.gray);
	PenMode(patCopy);

	regionHdl = NewRgn();

	OpenRgn();
	SetRect(&theRect,10,20,100,130);
	FrameRect(&theRect);
	SetRect(&theRect,155,20,245,130);
	FrameRect(&theRect);
	SetRect(&theRect,55,30,200,120);
	FrameOval(&theRect);
	CloseRgn(regionHdl);

	MoveTo(10,15);
	DrawString("\pFrameRgn");
	PenPat(&qd.black);
	PenSize(10,20);
	FrameRgn(regionHdl);
	Delay(30,&finalTicks);

	MoveTo(255,15);
	DrawString("\p1. FillRgn");
	OffsetRgn(regionHdl,245,0);
	FillRgn(regionHdl,&qd.dkGray);
	Delay(30,&finalTicks);

	MoveTo(10,154);
	DrawString("\p2. InsetRgn (10 horizontal, 10 vertical)");
	OffsetRgn(regionHdl,-245,140);
	InsetRgn(regionHdl,10,10);
	PenPat(&qd.dkGray);
	PaintRgn(regionHdl);
	Delay(30,&finalTicks);

	MoveTo(255,154);
	DrawString("\p3. InsetRgn (-10 horizontal, -10 vertical)");
	OffsetRgn(regionHdl,245,0);
	InsetRgn(regionHdl,-10,-10);
	PenPat(&qd.dkGray);
	PaintRgn(regionHdl);

	DisposeRgn(regionHdl);		
}

//  doTransferMode

void  doTransferMode(void)
{
	Handle	crossIconHdl, squareIconHdl;
	Rect		destRect;
	SInt16	a, b;
	BitMap	squareIconMap;
	SInt32	finalTicks;
	SInt16	sourceMode = 0;
	Str255	sourceString;

	FillRect(&(gWindowPtr->portRect),&qd.white);

	PenSize(1,1);
	PenPat(&qd.gray);
	PenMode(patOr);

	if(!(crossIconHdl = GetIcon(rCrossIcon)))
	{
		SysBeep(10);
		return;
	}
	
	if(!(squareIconHdl = GetIcon(rSquareIcon)))
	{
		SysBeep(10);
		return;
	}

	SetRect(&destRect,120,8,190,78);
	PlotIcon(&destRect,crossIconHdl);
	FrameRect(&destRect);
	MoveTo(200,48);
	DrawString("\pDestination");

	SetRect(&destRect,270,8,340,78);
	PlotIcon(&destRect,squareIconHdl);
	FrameRect(&destRect);
	MoveTo(350,48);
	DrawString("\pSource");

	for(a=91;a<192;a+=100)
		for(b=30;b<391;b+=120)
		{
			SetRect(&destRect,b,a,b+70,a+70);
			PlotIcon(&destRect,crossIconHdl);
		}

	HLock(squareIconHdl);

	squareIconMap.baseAddr = *squareIconHdl;
	squareIconMap.rowBytes = 4;
	SetRect(&squareIconMap.bounds,0,0,31,31);

	for(a=91;a<192;a+=100)
		for(b=30;b<391;b+=120)
		{
			Delay(30,&finalTicks);
			SetRect(&destRect,b,a,b+70,a+70);
			CopyBits(&squareIconMap,&qd.thePort->portBits,&squareIconMap.bounds,&destRect,
							 sourceMode++,NULL);
			GetIndString(sourceString,rModeStringList,sourceMode);
			MoveTo(b,a+82);
			DrawString(sourceString);
		}

	HUnlock(squareIconHdl);
}

//  doCopyBits

void  doCopyBits(void)
{
	WindowPtr	windowPtr;
	GrafPtr		oldPort;
	PicHandle	pictureHdl;
	Rect			sourceRect, destRect;
	SInt32		finalTicks;

	FillRect(&(gWindowPtr->portRect),&qd.white);

	if(!(windowPtr = GetNewWindow(rSmallWindow,NULL,(WindowPtr)-1)))
		ExitToShell();

	GetPort(&oldPort);
	SetPort(windowPtr);

	if(!(pictureHdl = GetPicture(rPicture)))
	{
		DisposeWindow(windowPtr);
		SysBeep(10);
		return;
	}

	HNoPurge((Handle) pictureHdl);	
	SetRect(&sourceRect,65,40,165,182);
	DrawPicture(pictureHdl,&sourceRect);
	HPurge((Handle) pictureHdl);	

	SetWTitle(windowPtr,"\pClick Mouse for CopyBits");
	while(!Button()) ;
	
	SetRect(&destRect,20,21,210,272);

	CopyBits(&windowPtr->portBits,&oldPort->portBits,&sourceRect,&destRect,
					 srcCopy,NULL);

	SetWTitle(windowPtr,"\pClick Mouse to Close");
	Delay(60,&finalTicks);
	while(!Button()) ;
	
	DisposeWindow(windowPtr);
	SetPort(oldPort);
}

//  doText

void  doText(void)
{
	SInt16	windowCentre, a, fontNum, stringWidth;
	Str255	textString;

	FillRect(&(gWindowPtr->portRect),&qd.white);

	windowCentre = ((FrontWindow())->portRect.right - (FrontWindow())->portRect.left) / 2;

	for(a=1;a<9;a++)
	{
		if(a == 1)
		{
			GetFNum("\pGeneva",&fontNum);
			TextFont(fontNum);
			TextFace(normal);
		}
		else if(a == 2)
			TextFace(bold);
		else if(a == 3)
		{
			GetFNum("\pTimes",&fontNum);
			TextFont(fontNum);
			TextFace(italic);
		}
		else if(a == 4)
			TextFace(underline);
		else if(a == 5)
		{
			GetFNum("\pHelvetica",&fontNum);
			TextFont(fontNum);
			TextFace(outline);
		}
		else if(a == 6)
			TextFace(shadow);
		else if(a == 7)
		{
			GetFNum("\pChicago",&fontNum);
			TextFont(fontNum);
			TextFace(condense);
		}
		else if(a == 8)
		{
			TextFace(extend);
			TextMode(grayishTextOr);
		}

		if(a < 7)
			TextSize(a * 2 + 10);
		else
			TextSize(12);

		GetIndString(textString,rTextStringList, a);
		stringWidth = StringWidth(textString);
		MoveTo(windowCentre - (stringWidth / 2), a * 35 - 10);
		DrawString(textString);
	}

	GetFNum("\pGeneva",&fontNum);
	TextFont(fontNum);
	TextSize(10);
	TextMode(srcOr);
	TextFace(normal);
}

//  doBasicColours

void  doBasicColours(void)
{
	SInt16	a;
	Rect		theRect;
	SInt32	finalTicks;

	FillRect(&(gWindowPtr->portRect),&qd.dkGray);
	PenPat(&qd.black);
	PenMode(patCopy);

	for(a=1;a<9;a++)
	{
		Delay(30,&finalTicks);
		if(a == 1)  ForeColor(blackColor);
		if(a == 2)  ForeColor(whiteColor);
		if(a == 3)  ForeColor(redColor);
		if(a == 4)  ForeColor(greenColor);
		if(a == 5)  ForeColor(blueColor);
		if(a == 6)  ForeColor(cyanColor);
		if(a == 7)  ForeColor(magentaColor);
		if(a == 8)  ForeColor(yellowColor);

		SetRect(&theRect,35,a*28,465,a*28+23);
		PaintRect(&theRect);			
	}

	ForeColor(blackColor);
}

//  doDrawWithMouse

void  doDrawWithMouse(void)
{
	Point		mouseDownMouse, previousMouse, currentMouse;
	Rect		drawRect;
	Pattern	thePattern;
        
	PenSize(1,1);
	PenPat(&qd.gray);
	PenMode(patXor);

	GetMouse(&mouseDownMouse);                     
	drawRect.left = drawRect.right  = mouseDownMouse.h;
	drawRect.top  = drawRect.bottom = mouseDownMouse.v;

	GetMouse(&previousMouse);

	while(StillDown())
	{
		GetMouse(&currentMouse);

		if(currentMouse.v != previousMouse.v || currentMouse.h != previousMouse.h)
		{
			FrameRect(&drawRect);

			drawRect.right = currentMouse.h;
			drawRect.bottom = currentMouse.v;

			FrameRect(&drawRect);
		}

		previousMouse.v = currentMouse.v;
		previousMouse.h = currentMouse.h;
	}

	FrameRect(&drawRect);

	PenMode(patCopy);

	PenSize(2,2);
	PenPat(&qd.black);
	ForeColor(redColor);
	FrameRect(&drawRect);

	InsetRect(&drawRect,10,10);
	PenSize(8,8);
	GetIndPattern(&thePattern,0,5);
	PenPat(&thePattern);
	ForeColor(blueColor);
	FrameRoundRect(&drawRect,40,40);

	InsetRect(&drawRect,16,16);
	PenSize(14,14);
	GetIndPattern(&thePattern,0,6);
	PenPat(&thePattern);
	ForeColor(greenColor);
	PaintOval(&drawRect);

	PenMode(patCopy);
	ForeColor(blackColor);
}      

// 