// 
// ColorQuickDraw.c
// 
// 
// This program:
//
// 	Opens a window in which the results of various basic Color QuickDraw drawing
//		operations are displayed.
//
//		Individual drawing operations are selected from a pull-down menu titled
//		"Demonstration".)
//
// 	Quits when the user selects Quit from the File menu or clicks the window's close
//		box.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource and associated 'MENU' resources (preload, non-purgeable).
//
// 	'WIND' resources (purgeable) (initially visible) for the main window, and for small
//		windows used for the CopyDeepMask and Transfer Modes demonstrations.
//
// 	An 'ALRT' resource and associated 'DITL' resource (purgeable).
//
// 	Three 'PICT' resources (purgeable).
//
// 	Two 'pltt' resources (purgeable).
//
// 	Two 'ppat' resources (purgeable).
//
// 	A 'STR#' resource (purgeable).
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Palettes.h>
#include <QDOffscreen.h>
#include <LowMem.h>
#include <Gestalt.h>
#include <Resources.h>

//  defines

#define mApple						128
#define mFile							129
#define  iQuit						11
#define mDemonstration		131
#define  iBitPattern			1
#define  iPixelPattern		2
#define  iCopyDeepMask		3
#define  iTransferModes		4
#define  iHighlighting		5
#define  iColorTable			6
#define rWindow						128
#define rImageWindow			129
#define rMenubar					128
#define rAlert						128
#define rIndexedStrings		128
#define rPaletteBaseID		128
#define rPixelPattern1		128
#define rPixelPattern2		129
#define rPicture					128
#define sColorQuickdraw		1
#define sSettingMonitor		2
#define sNeedMonitor			3
#define sRestoringMonitor	4

#define MAXLONG						0x7FFFFFFF

//  global variables

Boolean		gDone;
WindowPtr	gWindowPtr;
RGBColor	gWhiteColour;
RGBColor	gBlackColour;
RGBColor	gOchreColour;
RGBColor	gGreenColour;

//  function prototypes

void		main								(void);
void		doInitManagers			(void);
void		doEvents						(EventRecord *);
void		doMouseDown					(EventRecord *);
void		doMenuChoice				(SInt32);
void		doDemonstrationMenu	(SInt16);
void		doRGBColours				(void);
void		doBitPattern				(void);
void		doPixelPattern			(void);
void		doCopyDeepMask			(void);
void		doTransferModes			(void);
void		doHighlighting			(void);
void		doColourTable				(void);
SInt16	doCheckMonitor			(void);
void		doRestoreMonitor		(SInt16);

//  main

void  main(void)
{
	OSErr				osErr;
	SInt32			response;
	Str255			alertString;
	Handle			menubarHdl;
	MenuHandle	menuHdl;
	EventRecord	eventRec;
	Boolean			gotEvent;

	//  initialise managers

	doInitManagers();

	//  check for Color QuickDraw

	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response < gestalt8BitQD)
	{
		GetIndString(alertString,rIndexedStrings,sColorQuickdraw);
		ParamText(alertString,NULL,NULL,NULL);
		StopAlert(rAlert,NULL);
		ExitToShell();
	}

	//  set up menu bar and menus
	
	if(!(menubarHdl = GetNewMBar(rMenubar)))
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	if(!(menuHdl = GetMenuHandle(mApple)))
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open window

	if(!(gWindowPtr = GetNewCWindow(rWindow,NULL,(WindowPtr)-1)))
		ExitToShell();

	SetPort(gWindowPtr);

	TextSize(10);

	//  create some RGB colours

	doRGBColours();

	//  eventLoop

	gDone = false;

	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL);
		if(gotEvent)
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{		
	WindowPtr	windowPtr;
	SInt8			charCode;

	windowPtr = (WindowPtr) eventRecPtr->message;

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
				doMenuChoice(MenuKey(charCode));
			break;

		case updateEvt:
			BeginUpdate(windowPtr);
			EndUpdate(windowPtr);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				gDone = true;
			break;
	}
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
			daDriverRefNum = OpenDeskAcc(itemName);
			break;

		case mFile:
			if(menuItem == iQuit)
				gDone = true;
			break;

		case mDemonstration:
			doDemonstrationMenu(menuItem);
			break;
	}
	
	HiliteMenu(0);
}

//  doDemonstrationMenu

void	doDemonstrationMenu(SInt16 menuItem)
{
	switch(menuItem)
	{
		case iBitPattern:
			doBitPattern();
			break;

		case iPixelPattern:
			doPixelPattern();
			break;

		case iCopyDeepMask:
			doCopyDeepMask();
			break;

		case iTransferModes:
			doTransferModes();
			break;

		case iHighlighting:
			doHighlighting();
			break;

		case iColorTable:
			doColourTable();
			break;
	}
}

//  doRGBColours

void  doRGBColours(void)
{
	gWhiteColour.red		= 0xFFFF;
	gWhiteColour.green	= 0xFFFF;
	gWhiteColour.blue		= 0xFFFF;

	gBlackColour.red		= 0x0000;
	gBlackColour.green	= 0x0000;
	gBlackColour.blue		= 0x0000;

	gOchreColour.red		= 0xCCCC;
	gOchreColour.green	= 0x71FC;
	gOchreColour.blue		= 0x6A28;

	gGreenColour.red		= 0x460D;
	gGreenColour.green	= 0xCCCC;
	gGreenColour.blue		= 0x6BE2;
}

//  doBitPattern

void  doBitPattern(void)
{
	SInt16				a;
	PaletteHandle paletteHdl;
	Rect					theRect;
	Pattern				sysPattern;
	Str255				string;

	for(a=0;a<2;a++)
	{
		paletteHdl = GetNewPalette(rPaletteBaseID + a);
		SetPalette(gWindowPtr,paletteHdl,true);

		PmBackColor(2);
		FillRect(&(gWindowPtr->portRect),&qd.white);

		SetRect(&theRect,10,30,245,150);
		PenSize(10,20);
		GetIndPattern(&sysPattern,sysPatListID,16);
		PenPat(&sysPattern);
		PmForeColor(35);
		PmBackColor(229);
		FrameRect(&theRect);	
	
		OffsetRect(&theRect,245,0);
		GetIndPattern(&sysPattern,sysPatListID,37);
		PenPat(&sysPattern);
		PmForeColor(229);
		PmBackColor(210);
		PaintRect(&theRect);	

		OffsetRect(&theRect,-245,130);
		GetIndPattern(&sysPattern,sysPatListID,18);
		PmForeColor(210);
		PmBackColor(11);
		FillRoundRect(&theRect,50,50,&sysPattern);	

		OffsetRect(&theRect,245,0);
		GetIndPattern(&sysPattern,sysPatListID,19);
		PmForeColor(11);
		PmBackColor(35);
		FillOval(&theRect,&sysPattern);	

		MoveTo(10,20);
		PmForeColor(a);
		DrawString("\pForeground & background colours set with PmForeColor & PmBackColor");
		NumToString((SInt32) a+1,string);
		DrawString("\p       Palette No ");
		DrawString(string);   

		if(a == 0)
		{
			SetWTitle(gWindowPtr,"\pClick mouse for another palette");
			while(!Button()) ;
			DisposePalette(paletteHdl);			
		}
	}

	SetWTitle(gWindowPtr,"\pColor QuickDraw");
	DisposePalette(paletteHdl);
	PenPat(&qd.black);
}

//  doPixelPattern

void  doPixelPattern(void)
{
	PixPatHandle	pixpat1Hdl, pixpat2Hdl;
	Rect					theRect;
	RgnHandle			oldClipHdl, regionAHdl, regionBHdl, regionCHdl, scrollRegionHdl;
	SInt16				a;

	RGBBackColor(&gWhiteColour);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	if(!(pixpat1Hdl = GetPixPat(rPixelPattern1)))
		ExitToShell();
	PenPixPat(pixpat1Hdl);
	PenSize(50,0);
	SetRect(&theRect,15,15,240,280);
	FrameRect(&theRect);
	SetRect(&theRect,260,15,485,280);
	FillCRect(&theRect,pixpat1Hdl);

	if(!(pixpat2Hdl = GetPixPat(rPixelPattern2)))
		ExitToShell();
	BackPixPat(pixpat2Hdl);

	regionAHdl = NewRgn();
	regionBHdl = NewRgn();
	regionCHdl = NewRgn();
	SetRect(&theRect,65,15,190,280);
	RectRgn(regionAHdl,&theRect);
	SetRect(&theRect,260,15,485,280);
	RectRgn(regionBHdl,&theRect);
	UnionRgn(regionAHdl,regionBHdl,regionCHdl);

	oldClipHdl = NewRgn();
	GetClip(oldClipHdl);
	SetClip(regionCHdl);

	SetRect(&theRect,65,15,485,280);

	scrollRegionHdl = NewRgn();

	for(a=0;a<280;a++)
	{
		ScrollRect(&theRect,0,1,scrollRegionHdl);
		theRect.top ++;
	}

	SetRect(&theRect,65,15,485,280);
	BackPixPat(pixpat1Hdl);

	for(a=0;a<280;a++)
	{
		ScrollRect(&theRect,0,-1,scrollRegionHdl);
		theRect.bottom --;
	}

	SetClip(oldClipHdl);

	DisposePixPat(pixpat1Hdl);
	DisposePixPat(pixpat2Hdl);
	DisposeRgn(oldClipHdl);
	DisposeRgn(regionAHdl);
	DisposeRgn(regionBHdl);
	DisposeRgn(regionCHdl);
	DisposeRgn(scrollRegionHdl);
	
	PenPat(&qd.black);
}

//  doCopyDeepMask

void  doCopyDeepMask(void)
{		
	WindowPtr			sourceWindowPtr;
	PicHandle			picture1Hdl,picture2Hdl;
	Rect					sourceRect, maskRect, destRect, maskDisplayRect;
	CGrafPtr			windowPortPtr;	
	GDHandle			deviceHdl;
	GWorldPtr			gworldPortPtr;
	PixMapHandle	gworldPixMapHdl;
	RgnHandle			regionHdl;
	SInt32				finalTicks;

	RGBForeColor(&gBlackColour);
	RGBBackColor(&gWhiteColour);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	if(!(sourceWindowPtr = GetNewCWindow(rImageWindow,NULL,(WindowPtr)-1)))
		ExitToShell();
	SetPort(sourceWindowPtr);

	if(!(picture1Hdl = GetPicture(rPicture)))
		ExitToShell();
	HNoPurge((Handle) picture1Hdl);
	SetRect(&sourceRect,10,10,167,122);
	DrawPicture(picture1Hdl,&sourceRect);
	HPurge((Handle) picture1Hdl);

	SetRect(&maskRect,0,0,157,112);
	GetGWorld(&windowPortPtr,&deviceHdl);	
	NewGWorld(&gworldPortPtr,0,&maskRect,NULL,NULL,0);	
	SetGWorld(gworldPortPtr,NULL);
	gworldPixMapHdl = GetGWorldPixMap(gworldPortPtr);
	LockPixels(gworldPixMapHdl);
	EraseRect(&(gworldPortPtr->portRect));	
	if(!(picture2Hdl = GetPicture(rPicture+1)))
		ExitToShell();
	HNoPurge((Handle) picture2Hdl);
	DrawPicture(picture2Hdl,&maskRect);
	SetGWorld(windowPortPtr,deviceHdl);

	SetPort(gWindowPtr);
	SetRect(&maskDisplayRect,19,165,176,277);
	DrawPicture(picture2Hdl,&maskDisplayRect);
	HPurge((Handle) picture2Hdl);
	MoveTo(43,160);
	DrawString("\pCopy of offscreen mask");
	
	SetRect(&destRect,220,20,470,275);
	regionHdl = NewRgn();
	OpenRgn();
	FrameOval(&destRect);
	CloseRgn(regionHdl);

	PenSize(1,1);
	PenPat(&qd.ltGray);
	FrameRgn(regionHdl);
	MoveTo(315,150);
	DrawString("\pThe region");

	SetWTitle(sourceWindowPtr,"\pClick Mouse to Copy");
	while(!Button()) ;
	FillRect(&destRect,&qd.white);

	CopyDeepMask(&((GrafPtr) sourceWindowPtr)->portBits,
							 &((GrafPtr) gworldPortPtr)->portBits,
							 &((GrafPtr) gWindowPtr)->portBits,
							 &sourceRect,&maskRect,&destRect,srcCopy+ditherCopy,regionHdl);

	SetWTitle(sourceWindowPtr,"\pClick Mouse to Close");
	Delay(60,&finalTicks);

	while(!Button()) ;
	FillRect(&(gWindowPtr->portRect),&qd.white);
	
	UnlockPixels(gworldPixMapHdl);
	DisposeGWorld(gworldPortPtr);	

	ReleaseResource((Handle) picture1Hdl);
	ReleaseResource((Handle) picture2Hdl);
	DisposeRgn(regionHdl);
	DisposeWindow(sourceWindowPtr);

	PenPat(&qd.black);
}

//  doTransferModes

void  doTransferModes(void)
{
	SInt16		monitorCheckResult, transferMode, stringIndex;
	WindowPtr	sourceWindowPtr;
	PicHandle	sourceHdl, destinationHdl;
	Rect			sourceRect, destRect, blankRect;
	Str255		modeString;
	SInt32		finalTicks;

	if(!(monitorCheckResult = doCheckMonitor()))
		return;

	RGBForeColor(&gBlackColour);
	RGBBackColor(&gWhiteColour);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	if(!(sourceWindowPtr = GetNewCWindow(rImageWindow,NULL,(WindowPtr)-1)))
		ExitToShell();
	SetWTitle(sourceWindowPtr,"\pSource Image");

	SetPort(sourceWindowPtr);
	if(!(sourceHdl = GetPicture(rPicture)))
		ExitToShell();
	HNoPurge((Handle) sourceHdl);
	SetRect(&sourceRect,10,10,167,122);
	DrawPicture(sourceHdl,&sourceRect);
	HPurge((Handle) sourceHdl);

	SetPort(gWindowPtr);
	if(!(destinationHdl = GetPicture(rPicture+2)))
		ExitToShell();
	HNoPurge((Handle) destinationHdl);
	SetRect(&destRect,19,165,176,277);
	DrawPicture(destinationHdl,&destRect);
	MoveTo(55,160);
	DrawString("\pDestination Image");
	
	SetRect(&destRect,270,95,427,207);
	DrawPicture(destinationHdl,&destRect);
	SetRect(&blankRect,270,50,427,207);

	for(transferMode=0,stringIndex=5;transferMode<40;transferMode++,stringIndex++)
	{
		if(transferMode == 8)
			transferMode = 32;

		GetIndString(modeString,rIndexedStrings,stringIndex);
		MoveTo(270,70);
		DrawString("\pClick Mouse for ");
		DrawString(modeString);

		while(!Button()) ;

		FillRect(&blankRect,&qd.white);
		DrawPicture(destinationHdl,&destRect);
		Delay(30,&finalTicks);

		CopyBits(&((GrafPtr) sourceWindowPtr)->portBits,
						 &((GrafPtr) gWindowPtr)->portBits,
						 &sourceRect,&destRect,transferMode + ditherCopy,NULL);

		MoveTo(270,92);
		if(transferMode < 8)
			DrawString("\pBoolean mode: ");
		else
			DrawString("\pArithmetic mode: ");
		DrawString(modeString);
		Delay(60,&finalTicks);
	}

	MoveTo(270,70);
	DrawString("\pClick Mouse to exit");
	while(!Button()) ;
	
	FillRect(&(gWindowPtr->portRect),&qd.white);

	ReleaseResource((Handle) sourceHdl);
	ReleaseResource((Handle) destinationHdl);	
	DisposeWindow(sourceWindowPtr);

	if(monitorCheckResult != 2)
		doRestoreMonitor(monitorCheckResult);
}

//  doHighlighting

void  doHighlighting(void)
{
	RGBColor	oldHighlightColour;
	SInt16		a;
	Rect			theRect;
	UInt8			hiliteVal;
	SInt32		finalTicks;

	RGBBackColor(&gWhiteColour);

	FillRect(&(gWindowPtr->portRect),&qd.white);

	LMGetHiliteRGB(&oldHighlightColour);

	for(a=0;a<3;a++)
	{
		MoveTo(20,a*80+40);
		DrawString("\pClearing the highlight bit and calling InvertRect.");
		Delay(60,&finalTicks);
		SetRect(&theRect,10,a * 80 + 20,490,a * 80 + 80);

		hiliteVal = LMGetHiliteMode();
		BitClr(&hiliteVal,pHiliteBit);
		LMSetHiliteMode(hiliteVal);

		if(a == 1)
			HiliteColor(&gOchreColour);
		else if(a == 2)
			HiliteColor(&gGreenColour);
		InvertRect(&theRect);

		MoveTo(20,a*80+55);
		Delay(60,&finalTicks);
		DrawString("\pClick mouse to unhighlight.  ");
		DrawString("\p(Note:  The call to InvertRect reset the highlight bit ...");

		while(!Button()) ;

		MoveTo(20,a*80+70);
		DrawString("\p... so we clear the highlight bit again before calling InvertRect.)");
		Delay(60,&finalTicks);

		LMSetHiliteMode(hiliteVal);
		InvertRect(&theRect);	
	}

	HiliteColor(&oldHighlightColour);
	Delay(60,&finalTicks);
	MoveTo(20,260);
	DrawString("\pOriginal highlight colour has been reset.");
}

//  doColourTable

void  doColourTable(void)
{
	PixMapHandle	pixMapHdl;
	CTabHandle		colorTableHdl;
	SInt16				entries, a, b, c = 0;
	Rect					theRect;
	RGBColor			theColour;

	RGBForeColor(&gBlackColour);
	FillRect(&(gWindowPtr->portRect),&qd.black);

	pixMapHdl = ((CGrafPtr) gWindowPtr)->portPixMap;
	colorTableHdl = (*pixMapHdl)->pmTable;
	entries = (*colorTableHdl)->ctSize;

	if(entries == 0)
	{	
		RGBForeColor(&gWhiteColour);
		MoveTo(90,135);
		DrawString("\pYou need to set the monitor to 256 colours or less to get some");
		MoveTo(90,150);
		DrawString("\pentries in the colour table.  At present, we have zero entries.");
	}
	
	for(a=12;a<463;a+=30)
		for(b=5;b<276;b+=18)
		{
			if(c > entries)
				break;
			SetRect(&theRect,a,b,a+28,b+17);
			theColour = (*colorTableHdl)->ctTable[c++].rgb;
			RGBForeColor(&theColour);
			PaintRect(&theRect);
		}
}

//  doCheckMonitor

SInt16  doCheckMonitor(void)
{
	GDHandle			mainDeviceHdl;
	Boolean				result;
	Str255				alertString;	
	PixMapHandle	pixMapHdl;
	SInt16				pixelDepth;

	mainDeviceHdl = LMGetMainDevice();
	result = HasDepth(mainDeviceHdl,16,0,0);

	if(result == 0)
	{
		GetIndString(alertString,rIndexedStrings,sNeedMonitor);
		ParamText(alertString,NULL,NULL,NULL);
		NoteAlert(rAlert,NULL);
		return(0);
	}
	else
	{
		pixMapHdl = (**mainDeviceHdl).gdPMap;
		pixelDepth = (**pixMapHdl).pixelSize;
		if(pixelDepth < 16)
		{
			GetIndString(alertString,rIndexedStrings,sSettingMonitor);
			ParamText(alertString,NULL,NULL,NULL);
			NoteAlert(rAlert,NULL);
			SetDepth(mainDeviceHdl,16,0,0);
			return(pixelDepth);
		}
		return(2);
	}
}

//  doRestoreMonitor

void  doRestoreMonitor(SInt16 monitorCheckResult)
{
	Str255		alertString;	
	GDHandle	mainDeviceHdl;

	GetIndString(alertString,rIndexedStrings,sRestoringMonitor);
	ParamText(alertString,NULL,NULL,NULL);
	NoteAlert(rAlert,NULL);

	mainDeviceHdl = LMGetMainDevice();
	SetDepth(mainDeviceHdl,monitorCheckResult,0,0);
}

// 