// 
// MoreResources.c
// 
// 
// This program uses custom resources to:
//
// 	Store application preferences in the resource fork of a preferences file, and also
//		to assist in the initial creation of the preferences file.
//
// 	Store,in the resource fork of a document file, the user state and current state of
//		the window associated with the document.
//
// 	Store, in the resource fork of a document file, the width and height of the
//		printable area of the paper size chosen in the print Style dialog box. 
//
// The program utilises the following standard resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Demonstration
//		menus (preload, non-purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially invisible).  
//
// 	An 'ALRT' resource (purgeable) and associated 'DITL' resource (purgeable) 
//		associated with the display of error messages.  
//
// 	A 'DLOG' resource (purgeable) and associated 'DITL' resource (purgeable) associated
//		with the display of, and user modification of, current application preferences. 
//
// 	A 'STR#' resource (purgeable) containing the required name of the preferences file
//		created by the program.
//
// 	A 'STR ' resource (purgeable) containing the application-missing string, which is
//		copied to the resource fork of the preferences file.
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents and is32BitCompatible flags
//		set.
//
// The program utilises the following custom resources:
//
// 	A 'PrFn' (preferences) resource comprising three boolean values, which is located
//		in the program's resource file, which contains default preference values, and which
//		is copied to the resource fork of a preferences file created when the program is
//		run for the first time.  Thereafter, the 'PrFn' resource in the preferences file
//		is used for the storage and retrieval of application preferences set by the user.
//
// 	A 'WiSt' (window state) resource, which is created in the resource fork of the
//		document file used by the program, and which is used to store the associated
//		window's user state rectangle (a Rect value) and zoom state (a Boolean value).
//
// 	A 'PrAr' (printable area) resource, which is created in the resource fork of the
//		document file used by the program, and which is used to store the printable width
//		and height of the paper size chosen in the print Style dialog box.		
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Resources.h>
#include <StandardFile.h>
#include <Printing.h>
#include <Folders.h>

//  defines

#define mApple					128
#define mFile						129
#define  iOpen					2
#define  iClose					4
#define  iPageSetup			8
#define  iQuit					11
#define mDemonstration	131
#define  iPreferences		1
#define rNewWindow			128
#define rMenubar				128
#define rAlertBox				128	
#define rModalDialog		129
#define  iSoundOn				4	
#define  iFullScreenOn	5
#define  iAutoScrollOn	6
#define rStringList			128
#define  iPrefsFileName	1
#define rTypePrintRect	'PrAr'
#define  kPrintRectID		128
#define rTypeWinState		'WiSt'
#define  kWinStateID		128
#define rTypePrefs			'PrFn'
#define  kPrefsID				128
#define rTypeAppMiss		'STR '
#define  kAppMissID			-16397
#define MAXLONG					0x7FFFFFFF
#define topLeft(r)			(((Point *) &(r))[0])
#define botRight(r)			(((Point *) &(r))[1])

//  typedefs

typedef struct
{
	FSSpec		fileFSSpec;
}	docRecord, *docRecordPointer, **docRecordHandle;

typedef struct
{
	Boolean		soundOn;
	Boolean		fullScreenOn;
	Boolean		autoScrollOn;
}	appPrefs, *appPrefsPointer, **appPrefsHandle;

typedef struct
{
	Rect			userStateRect;
	Boolean		zoomState;
} winState, *winStatePtr,**winStateHandle;

typedef RectPtr *rectHandle;

//  global variables

Boolean		gDone;
Boolean		gInBackground;
THPrint		gTPrintHdl;
WindowPtr	gWindowPtr;
Boolean		gWindowOpen = false;
Boolean		gPrintStyleChanged = false;
Rect			gPrintRect;
Boolean		gSoundOn;
Boolean		gFullScreenOn;
Boolean		gAutoScrollOn;	
SInt16		gAppResFileRefNum;
SInt16		gPrefsFileRefNum = 0;

//  function prototypes

void	main											(void);
void	doInitManagers						(void);
void	doEvents									(EventRecord *);
void	doMouseDown								(EventRecord *);
void	doUpdateWindow						(WindowPtr);
void	doAdjustMenus							(void);
void	doMenuChoice							(long);
void	doFileMenu								(SInt16);
void	invalidateScrollBarArea		(WindowPtr);
void	doError										(SInt16);
void	doOpenCommand							(void);
void	doCloseCommand						(void);
void	doPreferencesDialog				(void);
void	doPrintStyleDialog				(void);
void	doGetPreferences					(void);
OSErr	doCopyResource						(ResType,SInt16,SInt16,SInt16);
void	doSavePreferences					(void);
void	doGetandSetWindowPosition	(WindowPtr);
void	doSaveWindowPosition			(WindowPtr);
void	doSetWindowState					(WindowPtr,Rect,Rect);
void	doGetPrintableSize				(WindowPtr);
void	doSavePrintableSize				(WindowPtr);

//  main

void  main(void)
{
	Handle			menubarHdl;
	MenuHandle	menuHdl;
	EventRecord	eventRec;

	//  initialise managers

	doInitManagers();

	//  set current resource file to application resource fork

	gAppResFileRefNum = CurResFile();

	//  set up menu bar and menus

	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		doError(MemError());
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		doError(MemError());
	else
		AppendResMenu(menuHdl,'DRVR');

	//  create and initialise a TPrint record

	PrOpen();
	gTPrintHdl = (THPrint) NewHandleClear(sizeof(TPrint));
	PrintDefault(gTPrintHdl);
	PrClose();

	//  read in application preferences

	doGetPreferences();

	//  enter event loop

	gDone = false;
	
	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL))
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
	FlushEvents(everyEvent,0);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt8			charCode;

	windowPtr = (WindowPtr) eventRecPtr->message;		

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuKey(charCode));
			}
			break;

		case updateEvt:
			BeginUpdate(windowPtr);
			EraseRgn(windowPtr->visRgn);
			doUpdateWindow(windowPtr);
			DrawGrowIcon(windowPtr);
			EndUpdate(windowPtr);
			break;

		case osEvt:
			switch((eventRecPtr->message >> 24) & 0x000000FF)
			{
				case suspendResumeMessage:
					gInBackground = (eventRecPtr->message & resumeFlag) == 0;
					break;
			}
			HiliteMenu(0);
			break;
	}
}
	
//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;
	Rect			growRect;
	long			newSize;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inMenuBar:
			doAdjustMenus();
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				doCloseCommand();
			break;

		case inGrow:
			growRect = qd.screenBits.bounds;
			growRect.top 	= 145; 
			growRect.left = 345;
			newSize = GrowWindow(windowPtr,eventRecPtr->where,&growRect);
			if(newSize != 0)
			{
				invalidateScrollBarArea(windowPtr);
				SizeWindow(windowPtr,LoWord(newSize),HiWord(newSize),true);
				invalidateScrollBarArea(windowPtr);
			}
			break;

		case inZoomIn:
		case inZoomOut:
			if(TrackBox(windowPtr,eventRecPtr->where,partCode))
			{
				SetPort(windowPtr);
				EraseRect(&windowPtr->portRect);
				ZoomWindow(windowPtr,partCode,false);
				InvalRect(&windowPtr->portRect);
			}
			break;
	}
}

//  doUpdateWindow

void  doUpdateWindow(WindowPtr windowPtr)
{
	Str255 string;

	SetPort(windowPtr);

	MoveTo(10,20);
	DrawString("\pCurrent Application Preferences:");
	MoveTo(10,35);
	DrawString("\pSound On: ");
	if(gSoundOn)  DrawString("\pYES");
	else  DrawString("\pNO");
	MoveTo(10,50);
	DrawString("\pFull Screen On: ");
	if(gFullScreenOn)  DrawString("\pYES");
	else  DrawString("\pNO");
	MoveTo(10,65);
	DrawString("\pAutoScroll On: ");
	if(gAutoScrollOn)  DrawString("\pYES");
	else  DrawString("\pNO");

	if(gPrintRect.bottom != 0)
	{
		MoveTo(10,85);
		DrawString("\pInformation from printable area ('PrAr') resource:"); 
		NumToString((long) gPrintRect.bottom,string);
		MoveTo(10,100);
		DrawString("\pPage print area height in screen pixels: ");
		DrawString(string);
		NumToString((long) gPrintRect.right,string);
		MoveTo(10,115);
		DrawString("\pPage print area width in screen pixels:  ");
		DrawString(string);
	}
	else
	{
		MoveTo(10,85);
		DrawString("\pNo printable area ('PrAr') resource saved yet"); 
	}
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle	menuHdl;

	if(gWindowOpen)
	{
		menuHdl = GetMenuHandle(mFile);
		DisableItem(menuHdl,iOpen);
		EnableItem(menuHdl,iClose);
		EnableItem(menuHdl,iPageSetup);
	}
	else
	{
		menuHdl = GetMenuHandle(mFile);
		EnableItem(menuHdl,iOpen);
		DisableItem(menuHdl,iClose);
		DisableItem(menuHdl,iPageSetup);
	}

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(long menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
			daDriverRefNum = OpenDeskAcc(itemName);
			break;

		case mFile:
			doFileMenu(menuItem);
			break;

		case mDemonstration:
			if(menuItem == iPreferences)
				doPreferencesDialog();
			break;
	}

	HiliteMenu(0);
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	switch(menuItem)
	{
		case iClose:
			doCloseCommand();
			break;

		case iOpen:
			doOpenCommand();
			break;

		case iPageSetup:
			doPrintStyleDialog();
			break;

		case iQuit:
			while(FrontWindow())
				doCloseCommand();
			gDone = true;
			break;
	}
}

//  invalidateScrollBarArea

void  invalidateScrollBarArea(WindowPtr windowPtr)
{
	Rect	tempRect;

	SetPort(windowPtr);

	tempRect = windowPtr->portRect;
	tempRect.left = tempRect.right - 15;
	InvalRect(&tempRect);

	tempRect = windowPtr->portRect;
	tempRect.top = tempRect.bottom - 15;
	InvalRect(&tempRect);
}	

//  doError

void  doError(SInt16 errorCode)
{
	Str255	errorString;

	NumToString((long) errorCode,errorString);
	ParamText(errorString,NULL,NULL,NULL);

	if(errorCode == memFullErr)
	{
		StopAlert(rAlertBox,NULL);
		ExitToShell();
	}	
	else
		CautionAlert(rAlertBox,NULL);
}

//  doOpenCommand

void  doOpenCommand(void)
{
	SFTypeList				fileTypes;
	StandardFileReply	fileReply;
	docRecordHandle		docRecHdl;
	OSErr							osError = 0;

	fileTypes[0] = 'TEXT';
	
	StandardGetFile(NULL,1,fileTypes,&fileReply);
	if(!(fileReply.sfGood))
		return;

	if(!(gWindowPtr = GetNewWindow(rNewWindow,NULL,(WindowPtr)-1)))
		return;

	if(!(docRecHdl = (docRecordHandle) NewHandle(sizeof(docRecord))))
	{
		DisposeWindow(gWindowPtr);
		return;
	}

	gWindowOpen = true; 
	SetPort(gWindowPtr);

	SetWRefCon(gWindowPtr,(long) docRecHdl);
	(*docRecHdl)->fileFSSpec = fileReply.sfFile;
	SetWTitle(gWindowPtr,(*docRecHdl)->fileFSSpec.name);	

	doGetandSetWindowPosition(gWindowPtr);
	doGetPrintableSize(gWindowPtr);

	ShowWindow(gWindowPtr);
}

//  doCloseCommand

void  doCloseCommand(void)
{
	WindowPtr				windowPtr;
	docRecordHandle	docRecHdl;
	OSErr						osError = 0;

	windowPtr = FrontWindow();
	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	doSaveWindowPosition(windowPtr);

	if(gPrintStyleChanged)
		doSavePrintableSize(gWindowPtr);

	DisposeHandle((Handle) docRecHdl);
	DisposeWindow(windowPtr);
	gWindowOpen = false;
}

//  doPreferencesDialog

void  doPreferencesDialog(void)
{
	DialogPtr 	modalDlgPtr;
	GrafPtr			oldPort;
	PenState		oldPenState;
	SInt16 			buttonOval, itemHit, itemType;
	Handle 			itemHdl;
	Rect 				itemRect;

	if(!(modalDlgPtr = GetNewDialog(rModalDialog,NULL,(WindowPtr) -1)))
		return;

	GetDialogItem(modalDlgPtr,iSoundOn,&itemType,&itemHdl,&itemRect);
	SetControlValue((ControlHandle) itemHdl,gSoundOn);
	GetDialogItem(modalDlgPtr,iFullScreenOn,&itemType,&itemHdl,&itemRect);
	SetControlValue((ControlHandle) itemHdl,gFullScreenOn);
	GetDialogItem(modalDlgPtr,iAutoScrollOn,&itemType,&itemHdl,&itemRect);
	SetControlValue((ControlHandle) itemHdl,gAutoScrollOn);

	ShowWindow(modalDlgPtr);

	GetPort(&oldPort);
	GetPenState(&oldPenState);
	GetDialogItem(modalDlgPtr,1,&itemType,&itemHdl,&itemRect);
	SetPort((*(ControlHandle) itemHdl)->contrlOwner);
	InsetRect(&itemRect,-4,-4);
	PenPat(&qd.black);
	PenSize(3,3);
	buttonOval = (itemRect.bottom - itemRect.top) / 2 + 2;
	FrameRoundRect(&itemRect,buttonOval,buttonOval);
	SetPenState(&oldPenState);
	SetPort(oldPort);

	do
	{
		ModalDialog(NULL,&itemHit);
		GetDialogItem(modalDlgPtr,itemHit,&itemType,&itemHdl,&itemRect);
		SetControlValue((ControlHandle) itemHdl,!GetControlValue((ControlHandle) itemHdl));
	} while((itemHit != 1) && (itemHit != 2));

	if(itemHit == 1)
	{
		GetDialogItem(modalDlgPtr,iSoundOn,&itemType,&itemHdl,&itemRect);
		gSoundOn = GetControlValue((ControlHandle) itemHdl);
		GetDialogItem(modalDlgPtr,iFullScreenOn,&itemType,&itemHdl,&itemRect);
		gFullScreenOn = GetControlValue((ControlHandle) itemHdl);
		GetDialogItem(modalDlgPtr,iAutoScrollOn,&itemType,&itemHdl,&itemRect);
		gAutoScrollOn = GetControlValue((ControlHandle) itemHdl);				
	}

	DisposeDialog(modalDlgPtr);

	if(gWindowPtr)
		InvalRect(&gWindowPtr->portRect);

	doSavePreferences();
}

//  doPrintStyleDialog

void  doPrintStyleDialog(void)
{
	Boolean	clickedOK;

	PrOpen();

	if(clickedOK = PrStlDialog(gTPrintHdl))
	{
		gPrintStyleChanged = true;
		gPrintRect = (*gTPrintHdl)->prInfo.rPage;
		InvalRect(&gWindowPtr->portRect);
	}

	PrClose();
}

//  doGetPreferences

void  doGetPreferences(void)
{
	Str255					prefsFileName;
	OSErr						osError;
	SInt16					volRefNum;
	long						directoryID;
	FSSpec 					fileSSpec;
	SInt16					fileRefNum;
	appPrefsHandle	appPrefsHdl;

	GetIndString(prefsFileName,rStringList,iPrefsFileName);

	osError = FindFolder(kOnSystemDisk,kPreferencesFolderType,kDontCreateFolder,&volRefNum,
											 &directoryID);

	if(osError == noErr)
		osError = FSMakeFSSpec(volRefNum,directoryID,prefsFileName,&fileSSpec);
	if(osError == noErr || osError == fnfErr)
		fileRefNum = FSpOpenResFile(&fileSSpec,fsCurPerm);

	if(fileRefNum == -1)
	{
		FSpCreateResFile(&fileSSpec,'PpPp','pref',smSystemScript);
		osError = ResError();

		if(osError == noErr)
		{
			fileRefNum = FSpOpenResFile(&fileSSpec,fsCurPerm);
			if(fileRefNum != -1 )
			{
				UseResFile(gAppResFileRefNum);
					
				osError = doCopyResource(rTypePrefs,kPrefsID,gAppResFileRefNum,fileRefNum);
				if(osError == noErr)
					osError = doCopyResource(rTypeAppMiss,kAppMissID,gAppResFileRefNum,fileRefNum);
				if(osError != noErr)
				{
					CloseResFile(fileRefNum);
					osError = FSpDelete(&fileSSpec);
					fileRefNum = -1;
				}
			}
		}
	}

	if(fileRefNum != -1)
	{
		UseResFile(fileRefNum); 

		appPrefsHdl = (appPrefsHandle) Get1Resource(rTypePrefs,kPrefsID);
		if(appPrefsHdl == NULL)
			return;

		gSoundOn 			= (*appPrefsHdl)->soundOn;
		gFullScreenOn	= (*appPrefsHdl)->fullScreenOn;
		gAutoScrollOn	= (*appPrefsHdl)->autoScrollOn;

		gPrefsFileRefNum = fileRefNum;

		UseResFile(gAppResFileRefNum);
	}
}

//  doCopyResource

OSErr  doCopyResource(ResType resType,SInt16 resID,SInt16 sourceFileRefNum,
											SInt16 destFileRefNum)
{
	SInt16	oldResFileRefNum;
	Handle	sourceResourceHdl;
	ResType	ignoredType;
	SInt16	ignoredID;
	Str255 	resourceName;
	SInt16	resAttributes;
	OSErr		osError;

	oldResFileRefNum = CurResFile();
	UseResFile(sourceFileRefNum);

	sourceResourceHdl = Get1Resource(resType,resID);

	if(sourceResourceHdl != NULL)
	{
		GetResInfo(sourceResourceHdl,&ignoredID,&ignoredType,resourceName);
		resAttributes = GetResAttrs(sourceResourceHdl);
		DetachResource(sourceResourceHdl);
		UseResFile(destFileRefNum);
		if(ResError() == noErr)
			AddResource(sourceResourceHdl,resType,resID,resourceName);
		if(ResError() == noErr)
			SetResAttrs(sourceResourceHdl,resAttributes);
		if(ResError() == noErr)
			ChangedResource(sourceResourceHdl);
		if(ResError() == noErr)
			WriteResource(sourceResourceHdl);
	}

	osError = ResError();

	ReleaseResource(sourceResourceHdl);
	UseResFile(oldResFileRefNum);

	return(osError);
}

//  doSavePreferences

void  doSavePreferences(void)
{
	appPrefsHandle	appPrefsHdl;
	Handle 					existingResHdl;
	Str255 					resourceName = "\pPreferences";

	if(gPrefsFileRefNum == -1)
		return;

	appPrefsHdl = (appPrefsHandle) NewHandleClear(sizeof(appPrefs));

	HLock((Handle) appPrefsHdl);

	(*appPrefsHdl)->soundOn = gSoundOn;
	(*appPrefsHdl)->fullScreenOn = gFullScreenOn;
	(*appPrefsHdl)->autoScrollOn = gAutoScrollOn;

	UseResFile(gPrefsFileRefNum);

	existingResHdl = Get1Resource(rTypePrefs,kPrefsID);
	if(existingResHdl != NULL)
	{
		RemoveResource(existingResHdl);
		if(ResError() == noErr)
			AddResource((Handle) appPrefsHdl,rTypePrefs,kPrefsID,resourceName);
		if(ResError() == noErr)
			WriteResource((Handle) appPrefsHdl);
	}

	HUnlock((Handle) appPrefsHdl);

	ReleaseResource((Handle) appPrefsHdl);
	UseResFile(gAppResFileRefNum);
}

//  doGetandSetWindowPosition

void  doGetandSetWindowPosition(WindowPtr windowPtr)
{
	Rect						userStateRect, stdStateRect, displayRect;
	docRecordHandle	docRecHdl;
	SInt16					fileRefNum;
	winStateHandle	winStateHdl;
	Boolean					gotResource;
	OSErr						osError;

	userStateRect = qd.screenBits.bounds;	
	SetRect(&userStateRect,userStateRect.left + 3,userStateRect.top + 42,
					userStateRect.right - 40,userStateRect.bottom - 6);

	stdStateRect = qd.screenBits.bounds;	
	SetRect(&stdStateRect,stdStateRect.left + 3,stdStateRect.top + 42,
					stdStateRect.right - 3,stdStateRect.bottom - 6);

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	fileRefNum = FSpOpenResFile(&(*docRecHdl)->fileFSSpec,fsRdWrPerm);
	if(fileRefNum < 0)
	{
		osError = ResError();
		doError(osError);
		return;
	}

	winStateHdl = (winStateHandle) Get1Resource(rTypeWinState,kWinStateID);
	if(winStateHdl != NULL )
	{
		gotResource = true;
		userStateRect = (*winStateHdl)->userStateRect;
	}
	else
		gotResource = false;

	if(gotResource)
	{
		if((*winStateHdl)->zoomState) 
			displayRect = stdStateRect;
		else
			displayRect = userStateRect;
	}
	else
	{
		displayRect = userStateRect;
	}

	MoveWindow(windowPtr,displayRect.left,displayRect.top,false);

	GlobalToLocal(&topLeft(displayRect));
	GlobalToLocal(&botRight(displayRect));
	SizeWindow(windowPtr,displayRect.right,displayRect.bottom,true);

	doSetWindowState(windowPtr,userStateRect,stdStateRect);

	ReleaseResource((Handle) winStateHdl);
	CloseResFile(fileRefNum);
}

//  doSaveWindowPosition

void  doSaveWindowPosition(WindowPtr windowPtr)
{
	docRecordHandle	docRecHdl;
	SInt16					fileRefNum;
	WindowPeek			windowRecPtr;
	WStateData 		 	*winStateDataPtr;
	Rect						stdRect, userRect;
	RgnHandle				contentRgnHdl;
	winState				userRectAndZoomState;
	winStateHandle	winStateHdl;
	OSErr						osError;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	fileRefNum = FSpOpenResFile(&(*docRecHdl)->fileFSSpec,fsRdWrPerm);
	if(fileRefNum < 0)
	{
		osError = ResError();
		doError(osError);
		return;
	}

	windowRecPtr = (WindowPeek) windowPtr;	
	winStateDataPtr = (WStateData *) *(windowRecPtr->dataHandle);		
	stdRect = winStateDataPtr->stdState;
	userRect = winStateDataPtr->userState;

	contentRgnHdl = windowRecPtr->contRgn;
	userRectAndZoomState.userStateRect = (*contentRgnHdl)->rgnBBox;
	userRectAndZoomState.zoomState = EqualRect(&userRectAndZoomState.userStateRect,&stdRect);
	if(userRectAndZoomState.zoomState)
		userRectAndZoomState.userStateRect = userRect;

	winStateHdl = (winStateHandle) Get1Resource(rTypeWinState,kWinStateID);
	if(winStateHdl != NULL)
	{	
		**winStateHdl = userRectAndZoomState;
		ChangedResource((Handle) winStateHdl);
		osError = ResError();
		if(osError != noErr)
			doError(osError);
	}
	else
	{
		winStateHdl = (winStateHandle) NewHandle(sizeof(winState));
		if(winStateHdl != NULL)
		{
			**winStateHdl = userRectAndZoomState;
			AddResource((Handle) winStateHdl,rTypeWinState,kWinStateID,"\pLast window state");
		}
	}

	if(winStateHdl != NULL)
	{
		UpdateResFile(fileRefNum);
		osError = ResError();
		if(osError != noErr)
			doError(osError);

		ReleaseResource((Handle) winStateHdl);
	}

	CloseResFile(fileRefNum);
}

//  doSetWindowState

void  doSetWindowState(WindowPtr windowPtr,Rect userStateRect,Rect stdStateRect)
{
	WindowPeek	windowRecPtr;
	WStateData	*winStateDataPtr;

	windowRecPtr = (WindowPeek) windowPtr;
	winStateDataPtr = (WStateData *) *(windowRecPtr->dataHandle);
	winStateDataPtr->userState = userStateRect;
	winStateDataPtr->stdState = stdStateRect;
}

//  doGetPrintableSize

void  doGetPrintableSize(WindowPtr windowPtr)
{
	docRecordHandle		docRecHdl;
	SInt16						fileRefNum;
	OSErr							osError;
	rectHandle				printRectHdl;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	fileRefNum = FSpOpenResFile(&(*docRecHdl)->fileFSSpec,fsRdWrPerm);
	if(fileRefNum < 0)
	{
		osError = ResError();
		doError(osError);
		return;
	}

	printRectHdl = (rectHandle) Get1Resource(rTypePrintRect,kPrintRectID);
	if(printRectHdl != NULL)
	{
		gPrintRect = **printRectHdl;
		ReleaseResource((Handle) printRectHdl);
	}

	CloseResFile(fileRefNum);
}

//  doSavePrintableSize

void  doSavePrintableSize(WindowPtr windowPtr)
{
	docRecordHandle	docRecHdl;
	SInt16					fileRefNum;
	rectHandle			printRectHdl;
	OSErr						osError;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	fileRefNum = FSpOpenResFile(&(*docRecHdl)->fileFSSpec,fsRdWrPerm);
	if(fileRefNum < 0)
	{
		osError = ResError();
		doError(osError);
		return;
	}

	printRectHdl = (rectHandle) Get1Resource(rTypePrintRect,kPrintRectID);
	if(printRectHdl != NULL)
	{	
		**printRectHdl = (*gTPrintHdl)->prInfo.rPage;
		ChangedResource((Handle) printRectHdl);
		osError = ResError();
		if(osError != noErr)
			doError(osError);
	}
	else
	{
		printRectHdl = (rectHandle) NewHandle(sizeof(Rect));
		if(printRectHdl != NULL)
		{
			**printRectHdl = (*gTPrintHdl)->prInfo.rPage;
			AddResource((Handle) printRectHdl,rTypePrintRect,kPrintRectID,"\pPrint rectangle");
		}
	}

	if(printRectHdl != NULL)
	{
		UpdateResFile(fileRefNum);
		osError = ResError();
		if(osError != noErr)
			doError(osError);

		ReleaseResource((Handle) printRectHdl);
	}
	
	gPrintStyleChanged = false;

	CloseResFile(fileRefNum);
}

// 