// 
// Scrap.c
// 
// 
// This program utilises the desk scrap and Scrap Manager routines to allow the user to:
//
// 	Cut, copy and clear pictures from, and paste pictures to, two windows opened by the
//		program.
//
// 	Paste pictures cut or copied from another application to the two windows opened
//		by the program.
//
// 	Open and close a Clipboard window, in which the current contents of the desk scrap
//		are displayed.
//
// In addition to the pictures cut and copied from either the program's windows or from
// another application's windows, the Clipboard window will display text copied to the
// desk scrap as a result of text cut and copy operations in another application.  The
// program, however, does not support the pasting of this text to documents displayed in
// the program's windows.  (The demonstration program at Chapter 17  Text and TextEdit
// shows how to cut, copy and paste text from and to a TextEdit edit record using the
// desk scrap.)   
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, and Edit menus (preload, 
//		non-purgeable).  
//
// 	Three 'WIND' resources (purgeable) (initially visible), two for the program's main
//		windows and one for the Clipboard window.  
//
// 	A 'PICT' resource (non-purgeable) containing a picture which may be cut, copied,
//		and pasted between the windows.
//
// 	An 'ALRT' resource (purgeable) and associated 'DITL' resource (purgeable) for use
//		by an error Alert.  
//
// 	A 'STR#' resource (purgeable) containing strings to be displayed in the error
//		Alert.  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible flags set.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Scrap.h>

//  defines

#define mApple						128
#define  iAbout						1
#define mFile							129
#define  iClose						4
#define  iQuit						11
#define mEdit							130
#define  iCut							3
#define  iCopy						4
#define  iPaste						5
#define  iClear						6
#define  iClipboard				9
#define rMenubar					128
#define rWindow						128
#define rClipboardWindow	130
#define rPicture					128
#define rAlertBox					128	
#define rErrorStrings			128
#define  eFailMenu				1
#define  eFailWindow			2
#define  eFailDocRec			3
#define  eZeroScrap				4
#define  ePutScrap				5
#define  eNoPictInScrap		6
#define kDocumentType			1
#define kClipboardType		2

//  typedefs

typedef struct
{
	PicHandle	pictureHdl;
	Boolean		selectFlag;
	SInt16		windowType;
} DocRec, **DocRecHandle;

//  global variables

Boolean		gDone;
Boolean		gInBackground;
WindowPtr	gWindowPtrs[2];
WindowPtr	gClipboardWindowPtr	= NULL;
Boolean		gClipboardShowing		= false;
Pattern		gMarqueePattern			= { 0x1F, 0x3E, 0x7C, 0xF8, 0xF1, 0xE3, 0xC7, 0x8F };

//  function prototypes

void	main									(void);
void	doInitManagers				(void);
void	doIdle								(void);
void	doEvents							(EventRecord *);
void	doMouseDown						(EventRecord *);
void	doUpdate							(EventRecord *);
void	doOSEvent							(EventRecord *);
void	doAdjustMenus					(void);
void	doMenuChoice					(SInt32);
void	doEditMenu						(SInt16);
void	doOpenWindows					(void);
void	doCloseWindow					(void);
void	doErrorAlert					(SInt16);
void	doInContent						(Point);
void	doCutCopyCommand			(Boolean);
void	doPasteCommand				(void);
void	doClearCommand				(void);
void	doClipboardCommand		(void);
void	doDrawClipboardWindow	(void);
void	doDrawPictureWindow		(WindowPtr);
void	doOpenWindows					(void);
Rect	doSetDestRect					(Rect *,WindowPtr);

//  main

void  main(void)
{
	Handle				menubarHdl;
	MenuHandle		menuHdl;
	Boolean				gotEvent;
	EventRecord		eventRec;
	
	//  initialise managers

	doInitManagers();
	
	//  set up menu bar and menus
	
	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		doErrorAlert(eFailMenu);
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		doErrorAlert(eFailMenu);
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open windows 

	doOpenWindows();

	//  enter eventLoop
	
	gDone = false;

	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventRec,2,NULL);

		if(gotEvent)
			doEvents(&eventRec);
		else
			doIdle();
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
	FlushEvents(everyEvent,0);
}

//  doIdle

void  doIdle(void)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	GrafPtr				oldPort;
	Rect					marqueeRect;
	UInt8					lastByte;
	SInt16				a;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	if(((*docRecHdl)->windowType == kClipboardType) || (*docRecHdl)->selectFlag == false)
	{
		return;
	}
	else
	{
		GetPort(&oldPort);
		SetPort(windowPtr);

		marqueeRect = doSetDestRect(&(*(*docRecHdl)->pictureHdl)->picFrame,windowPtr);
		InsetRect(&marqueeRect,-2,-2);

		lastByte = gMarqueePattern.pat[7];
		for(a=7;a>0;--a)
			gMarqueePattern.pat[a] = gMarqueePattern.pat[a - 1];
		gMarqueePattern.pat[0] = lastByte;
		
		PenPat(&gMarqueePattern);
		FrameRect(&marqueeRect);
		
		SetPort(oldPort);
	}
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	SInt8	charCode;

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuKey(charCode));
			}
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			HiliteMenu(0);
			break;
	}
}
	
//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	SInt16		partCode;
	WindowPtr	windowPtr;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			doAdjustMenus();
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			else
				doInContent(eventRecPtr->where);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				doCloseWindow();
			break;
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	SInt32				windowType;

	windowPtr = (WindowPtr) eventRecPtr->message;
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);
	windowType = (*docRecHdl)->windowType;

	BeginUpdate(windowPtr);

	if(windowType == kDocumentType)
	{
		if((*docRecHdl)->pictureHdl != NULL)
			doDrawPictureWindow(windowPtr);
	}
	else if(windowType == kClipboardType)
		doDrawClipboardWindow();

	EndUpdate(windowPtr);
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	WindowPtr windowPtr;

	windowPtr = FrontWindow();

	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			gInBackground = (eventRecPtr->message & resumeFlag) == 0;
			if(gClipboardWindowPtr && gClipboardShowing)
			{
				if(gInBackground)
					HideWindow(gClipboardWindowPtr);
				else
					ShowWindow(gClipboardWindowPtr);
			}
			break;

		case mouseMovedMessage:
			break;
	}
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle		fileMenuHdl, editMenuHdl;
	DocRecHandle	docRecHdl;
	SInt32				scrapOffset;

	fileMenuHdl = GetMenuHandle(mFile);
	editMenuHdl = GetMenuHandle(mEdit);

	docRecHdl = (DocRecHandle) GetWRefCon(FrontWindow());

	if((*docRecHdl)->windowType == kClipboardType)
		EnableItem(fileMenuHdl,iClose);
	else
		DisableItem(fileMenuHdl,iClose);

	if((*docRecHdl)->pictureHdl && (*docRecHdl)->selectFlag)
	{
		EnableItem(editMenuHdl,iCut);
		EnableItem(editMenuHdl,iCopy);
		EnableItem(editMenuHdl,iClear);
	}
	else
	{
		DisableItem(editMenuHdl,iCut);
		DisableItem(editMenuHdl,iCopy);
		DisableItem(editMenuHdl,iClear);
	}

	if(GetScrap(NULL,'PICT',&scrapOffset)  && (*docRecHdl)->windowType != kClipboardType)
		EnableItem(editMenuHdl,iPaste);
	else
		DisableItem(editMenuHdl,iPaste);

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			if(menuItem == iClose)
				doCloseWindow();
			else if(menuItem == iQuit)
				gDone = true;
			break;

		case mEdit:
			doEditMenu(menuItem);
			break;		
	}

	HiliteMenu(0);
}

//  doEditMenu

void	doEditMenu(SInt16 menuItem)
{
	switch(menuItem)
	{
		case iCut:
			doCutCopyCommand(true);
			break;

		case iCopy:
			doCutCopyCommand(false);
			break;

		case iPaste:
			doPasteCommand();
			break;

		case iClear:
			doClearCommand();
			break;

		case iClipboard:
			doClipboardCommand();
			break;
	}
}

//  doCloseWindow

void  doCloseWindow(void)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	MenuHandle		editMenuHdl;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	if((*docRecHdl)->windowType == kClipboardType)
	{
		DisposeWindow(windowPtr);
		gClipboardWindowPtr = NULL;
		gClipboardShowing = false;
		editMenuHdl = GetMenu(mEdit);
		SetItem(editMenuHdl,iClipboard,"\pShow Clipboard");
	}
}

//  doErrorAlert

void  doErrorAlert(SInt16 errorCode)
{
	Str255	errorString;

	GetIndString(errorString,rErrorStrings,errorCode);
	ParamText(errorString,NULL,NULL,NULL);

	if(errorCode < ePutScrap)
	{
		StopAlert(rAlertBox,NULL);
		ExitToShell();
	}
	else
		CautionAlert(rAlertBox,NULL);
}

//  doInContent

void  doInContent(Point mouseXY)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	GrafPtr				oldPort;
	Rect					pictRect;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	if((*docRecHdl)->windowType == kClipboardType)
		return;

	GetPort(&oldPort);
	SetPort(windowPtr);

	if((*docRecHdl)->pictureHdl != NULL)
	{
		pictRect = doSetDestRect(&(*(*docRecHdl)->pictureHdl)->picFrame,windowPtr);
		InsetRect(&pictRect,-2,-2);

		GlobalToLocal(&mouseXY);

		if(PtInRect(mouseXY,&pictRect))
		{
			(*docRecHdl)->selectFlag = true;
		}
		else
		{
			(*docRecHdl)->selectFlag = false;

			PenPat(&qd.black);
			ForeColor(whiteColor);
			FrameRect(&pictRect);	
			ForeColor(blackColor);
		}
	}

	SetPort(oldPort);
}

//  doCutCopyCommand

void  doCutCopyCommand(Boolean cutFlag)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;	
	Size					dataLength;
	SInt32				errorCode;
	GrafPtr				oldPort;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	if((*docRecHdl)->selectFlag == false)
		return;

	if(ZeroScrap() == noErr)
	{
		dataLength = GetHandleSize((Handle) (*docRecHdl)->pictureHdl);
		HLock((Handle) (*docRecHdl)->pictureHdl);

		errorCode = PutScrap((SInt32) dataLength,'PICT',*((Handle) (*docRecHdl)->pictureHdl));
		if(errorCode != noErr)
			doErrorAlert(ePutScrap);

		HUnlock((Handle) (*docRecHdl)->pictureHdl);
	}
	else
		doErrorAlert(eZeroScrap);

	if(cutFlag)
	{
		GetPort(&oldPort);
		SetPort(windowPtr);

		DisposeHandle((Handle) (*docRecHdl)->pictureHdl);
		(*docRecHdl)->pictureHdl = NULL;
		(*docRecHdl)->selectFlag = false;
		EraseRect(&windowPtr->portRect);

		SetPort(oldPort);
	}

	if(gClipboardWindowPtr != NULL)
		doDrawClipboardWindow();
}

//  doPasteCommand

void  doPasteCommand(void)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;	
	GrafPtr				oldPort;
	SInt32				sizeOfPictData, scrapOffset;
	Handle				tempHdl;
	Rect					destRect;	

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	GetPort(&oldPort);
	SetPort(windowPtr);	

	sizeOfPictData = GetScrap(NULL,'PICT',&scrapOffset);
	if(sizeOfPictData > 0)
	{
		tempHdl = NewHandle((Size) sizeOfPictData);	
		HLock(tempHdl);

		sizeOfPictData = GetScrap(tempHdl,'PICT',&scrapOffset);

		EraseRect(&windowPtr->portRect);
		(*docRecHdl)->selectFlag = false;
		destRect = doSetDestRect(&(*(PicHandle) tempHdl)->picFrame,windowPtr);

		DrawPicture((PicHandle) tempHdl,&destRect);

		if((*docRecHdl)->pictureHdl != NULL)
			DisposeHandle((Handle) (*docRecHdl)->pictureHdl);

		(*docRecHdl)->pictureHdl = (PicHandle) NewHandle((Size) sizeOfPictData);
		BlockMoveData(*tempHdl,*((*docRecHdl)->pictureHdl),(Size) sizeOfPictData);

		HUnlock(tempHdl);
		DisposeHandle(tempHdl);
	}

	SetPort(oldPort);
}

//  doClearCommand

void  doClearCommand(void)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;	
	GrafPtr				oldPort;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	GetPort(&oldPort);
	SetPort(windowPtr);

	DisposeHandle((Handle) (*docRecHdl)->pictureHdl);
	(*docRecHdl)->pictureHdl = NULL;
	(*docRecHdl)->selectFlag = false;
	EraseRect(&windowPtr->portRect);

	SetPort(oldPort);
}

//  doClipboardCommand

void  doClipboardCommand(void)
{
	MenuHandle		editMenuHdl;
	DocRecHandle	docRecHdl;	

	editMenuHdl = GetMenu(mEdit);

	if(gClipboardWindowPtr == NULL)
	{
		if(!(gClipboardWindowPtr = GetNewWindow(rClipboardWindow,NULL,(WindowPtr)-1)))
			doErrorAlert(eFailWindow);
		if(!(docRecHdl = (DocRecHandle) NewHandle(sizeof(DocRec))))
			doErrorAlert(eFailDocRec);
		SetWRefCon(gClipboardWindowPtr,(SInt32) docRecHdl);
		(*docRecHdl)->windowType = kClipboardType;

		gClipboardShowing = true;

		SetMenuItemText(editMenuHdl,iClipboard,"\pHide Clipboard");
	}
	else
	{
		if(gClipboardShowing)
		{
			HideWindow(gClipboardWindowPtr);
			gClipboardShowing = false;
			SetMenuItemText(editMenuHdl,iClipboard,"\pShow Clipboard");
		}
		else
		{
			ShowWindow(gClipboardWindowPtr);
			gClipboardShowing = true;
			SetMenuItemText(editMenuHdl,iClipboard,"\pHide Clipboard");				
		}
	}
}

//  doDrawClipboardWindow

void  doDrawClipboardWindow(void)
{
	GrafPtr	oldPort;
	SInt32	sizeOfPictData, sizeOfTextData, scrapOffset;
	Handle	tempHdl;
	Rect		destRect;	

	GetPort(&oldPort);
	SetPort(gClipboardWindowPtr);	

	EraseRect(&gClipboardWindowPtr->portRect);

	MoveTo(0,18);
	LineTo(505,18);
	MoveTo(0,20);
	LineTo(505,20);

	TextFont(applFont);
	TextSize(9);
	MoveTo(4,13);
	DrawString("\pClipboard contents:");

	sizeOfPictData = GetScrap(NULL,'PICT',&scrapOffset);
	if(sizeOfPictData > 0)
	{
		MoveTo(95,13);
		DrawString("\ppicture");

		tempHdl = NewHandle((Size) sizeOfPictData);	
		HLock(tempHdl);

		sizeOfPictData = GetScrap(tempHdl,'PICT',&scrapOffset);

		destRect = (*(PicHandle) tempHdl)->picFrame;
		OffsetRect(&destRect,-((*(PicHandle) tempHdl)->picFrame.left - 2),
												 -((*(PicHandle) tempHdl)->picFrame.top - 22));
		DrawPicture((PicHandle) tempHdl,&destRect);

		HUnlock(tempHdl);
		DisposeHandle(tempHdl);
	}

	sizeOfTextData = GetScrap(NULL,'TEXT',&scrapOffset);
	if(sizeOfTextData > 0)
	{
		MoveTo(95,13);
		DrawString("\ptext");
	
		tempHdl = NewHandle((Size) sizeOfTextData);	
		HLock(tempHdl);
	
		sizeOfTextData = GetScrap(tempHdl,'TEXT',&scrapOffset);

		destRect = gClipboardWindowPtr->portRect;
		destRect.top += 20;
		InsetRect(&destRect,2,2);

		TETextBox(*tempHdl,sizeOfTextData,&destRect,0);

		HUnlock(tempHdl);
		DisposeHandle(tempHdl);
	}

	SetPort(oldPort);
}

//  doDrawPictureWindow

void  doDrawPictureWindow(WindowPtr windowPtr)
{
	GrafPtr				oldPort;
	Rect  				destRect;
	DocRecHandle	docRecHdl;

	GetPort(&oldPort);
	SetPort(windowPtr);

	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);
	destRect = doSetDestRect(&(*(*docRecHdl)->pictureHdl)->picFrame,windowPtr);

	DrawPicture((*docRecHdl)->pictureHdl,&destRect);

	if((*docRecHdl)->selectFlag)
	{
		InsetRect(&destRect,-2,-2);
		PenPat(&gMarqueePattern);
		FrameRect(&destRect);	
	}

	SetPort(oldPort);
}

//  doOpenWindows

void  doOpenWindows(void)
{
	SInt16				a;
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;

	for(a=0;a<2;a++)
	{
		if(!(windowPtr = GetNewWindow(rWindow + a,NULL,(WindowPtr)-1)))
			doErrorAlert(eFailWindow);
		gWindowPtrs[a] = windowPtr;

		if(!(docRecHdl = (DocRecHandle) NewHandle(sizeof(DocRec))))
			doErrorAlert(eFailDocRec);
		SetWRefCon(windowPtr,(SInt32) docRecHdl);

		(*docRecHdl)->pictureHdl = NULL;
		(*docRecHdl)->windowType = kDocumentType;
		(*docRecHdl)->selectFlag = false;
	}

	SetPort(windowPtr);

	(*docRecHdl)->pictureHdl = GetPicture(rPicture);
}

//  doSetDestRect

Rect  doSetDestRect(Rect *picFrame,WindowPtr windowPtr)
{
	Rect		destRect;
	SInt16	diffX, diffY;

	destRect = *picFrame;

	OffsetRect(&destRect,-(*picFrame).left,-(*picFrame).top);

	diffX = (windowPtr->portRect.right - windowPtr->portRect.left) - 
					((*picFrame).right - (*picFrame).left);
	diffY = (windowPtr->portRect.bottom - windowPtr->portRect.top) - 
					((*picFrame).bottom - (*picFrame).top);
						
	OffsetRect(&destRect,diffX / 2,diffY / 2);
	
	return(destRect);
}

// 