// 
// CDEFandVBL.c
// 
//
// This program opens a window containing a slider control panel.  The slider control
// panel contains two radio button controls and a slider control.  The radio buttons
// activate and deactivate the slider control.  
//
// The slider control uses a custom control definition function (CDEF).  The CDEF 
// utilises a VBL task to delay the drawing of a moved indicator in the graphics port 
// until the vertical blank period is entered.  The radio buttons also use a custom CDEF.
// On colour or grayscale displays, the appearance of the controls conforms to that 
// specified in the document Apple Grayscale Appearance for System 7.5 published by Apple
// Computer, Inc.
//
// This program also includes a demonstration of an animated cursor which utilises a 
// system-based VBL task to increment the frames of the animation.  This demonstration
// is invoked by choosing the VBL Task Animated Cursor item in the Demonstration menu.
// and may be cancelled by pressing any key.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Demonstration
//		menus (preload, non-purgeable).
//
// 	A 'WIND' resource (purgeable) (initially visible) and a 'wctb' resource (purgeable)
//		for the window containing the slider control panel.  
//
// 	'CNTL' resources  (purgeable) for the radio button and slider controls.
//
// 	The 'CDEF' code resources (non-purgeable).
//
// 	An 'acur' resource (purgeable) and 'CURS' resources (purgeable) for the animated
//		cursor.
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible flags set.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Gestalt.h>
#include <Resources.h>
#include <Retrace.h>
#include <Palettes.h>
#include <LowMem.h>

//  defines

#define mApple						128
#define  iAbout						1
#define mFile							129
#define  iQuit						11
#define mDemonstration		131
#define  iVBLAnimCursor		1
#define rMenubar					128
#define rWindow						128
#define rFingersCursor		128
#define rStartRadioButton	128
#define rStopRadioButton	129
#define rSliderControl		130
#define MAXLONG						0x7FFFFFFF

//  typedefs

typedef struct
{
	SInt16			numberOfFrames;
	SInt16			whichFrame;
	CursHandle	frame[];
} animCurs, *animCursPtr, **animCursHandle;

typedef struct
{
	VBLTask	vblTaskRec;
	SInt32	thisApplicationsA5;
} VBLRec, *VBLRecPtr;

//  global variables

Boolean					gColorQuickDrawPresent = false;
Boolean					gColourDisplay				 = false;
Boolean					gDone;
SInt32					gSleepTime;
Boolean					gInBackground;
WindowPtr				gWindowPtr;
animCursHandle	gAnimCursHdl;
VBLRec					gVBLRec;
SInt16					gVBLCount;
Boolean					gAnimatedCursorActive  = false;
RGBColor				gWindowColour					 = { 0xDDDD, 0xDDDD, 0xDDDD };
ControlHandle		gSliderControlHdl;
ControlHandle		gStartControlHdl;
ControlHandle		gStopControlHdl;

//  function prototypes

void		main										(void);
void		doInitManagers					(void);
void		doEvents								(EventRecord *);
void		doMouseDown							(EventRecord *);
void		doUpdate								(EventRecord *);
void		doActivate							(EventRecord *);
void		doActivateWindow				(Boolean);
void		doOSEvent								(EventRecord *);
void		doInContent							(EventRecord *,WindowPtr);
void		doMenuChoice						(SInt32 menuChoice);
void		doGetSliderControlSuite	(void);
void		doDrawControlsPanel			(void);
void		doStartAnimCursor				(void);
Boolean doGetAnimCursor					(SInt16);
void		doInstallSystemVBLTask	(void);
void		animCursVBLTask					(void);
void		doStopAnimCursor				(void);

//  in-line glue for GetVBLRec

pascal SInt32  GetVBLRec (void) = 0x2E88;

//  main

void  main(void)
{
	OSErr					osErr;
	SInt32				response;
	GDHandle			mainDeviceHdl;
	SInt16				bitsPerPixel;
	Handle				menubarHdl;
	MenuHandle		menuHdl;
	EventRecord		eventRec;

	//  initialise managers

	doInitManagers();
		
	//  check for Color QuickDraw

	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response >= gestalt8BitQD)
	{
		gColorQuickDrawPresent = true;

		mainDeviceHdl = LMGetMainDevice();
		bitsPerPixel = (*(*mainDeviceHdl)->gdPMap)->pixelSize;
		if(bitsPerPixel > 1)
			gColourDisplay = true;
	}

	//  set up menu bar and menus

	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open window 

	if(gColorQuickDrawPresent)
	{
		if(!(gWindowPtr = GetNewCWindow(rWindow,NULL,(WindowPtr)-1)))
			ExitToShell();
	}
	else
	{
		if(!(gWindowPtr = GetNewWindow(rWindow,NULL,(WindowPtr)-1)))
			ExitToShell();
	}

	SetPort(gWindowPtr);
		
	//  get slider control suite

	doGetSliderControlSuite();

	//  enter eventLoop

	gDone = false;
	gSleepTime = MAXLONG;

	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,gSleepTime,NULL))
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
	FlushEvents(everyEvent,0);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			if(gAnimatedCursorActive)
				doStopAnimCursor();
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case activateEvt:
			doActivate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			HiliteMenu(0);
			break;
	}
}
	
//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	SInt16			partCode;
	WindowPtr		windowPtr;
	MenuHandle	menuHdl;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	menuHdl = GetMenuHandle(mDemonstration);

	switch(partCode)
	{
		case inMenuBar:
			if(gAnimatedCursorActive)
				DisableItem(menuHdl,iVBLAnimCursor);
			else
				EnableItem(menuHdl,iVBLAnimCursor);
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			else
				doInContent(eventRecPtr,windowPtr);
			break;
			
		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	windowPtr = (WindowPtr) eventRecPtr->message;

	BeginUpdate(windowPtr);

	SetPort(windowPtr);
	UpdateControls(windowPtr,windowPtr->visRgn);
	doDrawControlsPanel();

	EndUpdate(windowPtr);
}

//  doActivate

void  doActivate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventRecPtr->message;

	becomingActive = ((eventRecPtr->modifiers & activeFlag) == activeFlag);
	doActivateWindow(becomingActive);
}

//  doActivateWindow

void  doActivateWindow(Boolean becomingActive)
{	
	SInt16 controlVal;

	if(becomingActive)
	{
		controlVal = GetControlValue(gStartControlHdl);
		if(controlVal == 1)
			HiliteControl(gSliderControlHdl,0);
		HiliteControl(gStartControlHdl,0);
		HiliteControl(gStopControlHdl,0);
	}
	else
	{
		HiliteControl(gSliderControlHdl,255);
		HiliteControl(gStartControlHdl,255);
		HiliteControl(gStopControlHdl,255);
	}
	
	doDrawControlsPanel();
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			gInBackground = (eventRecPtr->message & resumeFlag) == 0;
			doActivateWindow(!gInBackground);
			break;

		case mouseMovedMessage:
			break;
	}
}

//  doInContent

void  doInContent(EventRecord *eventRecPtr,WindowPtr windowPtr)
{
	ControlHandle	controlHdl;
	SInt16				partCode;
	Rect					theRect;
	Str255				theString;
	
	GlobalToLocal(&eventRecPtr->where);

	partCode = FindControl(eventRecPtr->where,windowPtr,&controlHdl);

	if(controlHdl == gSliderControlHdl)
	{
		if(partCode == kControlIndicatorPart)
			TrackControl(controlHdl,eventRecPtr->where,NULL);

		RGBBackColor(&gWindowColour);
		SetRect(&theRect,253,107,390,119);
		FillRect(&theRect,&qd.white);
		MoveTo(255,117);
		DrawString("\pSlider Control Value: ");
		NumToString((long) GetControlValue(controlHdl), theString);
		DrawString(theString);
	}
	else if(controlHdl == gStartControlHdl || controlHdl == gStopControlHdl)
	{
		if(TrackControl(controlHdl,eventRecPtr->where,NULL))
		{
			if(controlHdl == gStartControlHdl)
			{
				HiliteControl(gSliderControlHdl,0);
				SetControlValue(gStartControlHdl,1)	;	
				SetControlValue(gStopControlHdl,0);		
			}
			else if(controlHdl == gStopControlHdl)
			{
				SetControlValue(gSliderControlHdl,0);
				HiliteControl(gSliderControlHdl,255);
				SetControlValue(gStartControlHdl,0);	
				SetControlValue(gStopControlHdl,1);

				RGBBackColor(&gWindowColour);
				SetRect(&theRect,253,107,390,119);
				FillRect(&theRect,&qd.white);
			}		
		}
	}
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			if(menuItem == iQuit)
			{
				DisposeWindow(gWindowPtr);
				gDone = true;
			}
			break;

		case mDemonstration:
			if(menuItem == iVBLAnimCursor)
				doStartAnimCursor();
			break;
	}

	HiliteMenu(0);
}

//  doGetSliderControlSuite

void  doGetSliderControlSuite(void)
{
	gSliderControlHdl = GetNewControl(rSliderControl,gWindowPtr);
	HiliteControl(gSliderControlHdl,255);

	gStartControlHdl = GetNewControl(rStartRadioButton,gWindowPtr);
	gStopControlHdl = GetNewControl(rStopRadioButton,gWindowPtr);

	doDrawControlsPanel();
}

//  doDrawControlsPanel

void  doDrawControlsPanel(void)
{
	GDHandle	mainDeviceHdl;
	SInt16		bitsPerPixel;
	SInt16		fontNum, a;
	RGBColor	gray8	= { 0x7777,0x7777,0x7777 };

	GetFNum("\pChicago",&fontNum);
	TextFont(fontNum);
	TextSize(12);
		
	mainDeviceHdl = LMGetMainDevice();
	bitsPerPixel = (*(*mainDeviceHdl)->gdPMap)->pixelSize;

	if(bitsPerPixel > 1)
		gColourDisplay = true;
	else
		gColourDisplay = false;

	for(a=0;a<2;a++)
	{
		if(a == 0)
			ForeColor(whiteColor);
		else
		{
			if(!gInBackground)
			{
				if(gColorQuickDrawPresent && gColourDisplay)
					ForeColor(blackColor);
				else
				{
					ForeColor(blackColor);
					PenPat(&qd.black);
					TextMode(srcOr);
				}
			}
			else	
			{
				if(gColorQuickDrawPresent && gColourDisplay)
					RGBForeColor(&gray8);
				else
				{
					ForeColor(blackColor);
					PenPat(&qd.gray);
					TextMode(grayishTextOr);
				}
			}
		}
		
		if(a == 0 && !gColourDisplay)
			continue;
		else
		{
			MoveTo(156-a,22-a);
			LineTo(152-a,22-a);
			LineTo(152-a,230-a);	
			LineTo(246-a,230-a);	
			LineTo(246-a,22-a);	
			LineTo(242-a,22-a);	

			MoveTo(163-a,26-a);
			DrawString("\pEngine RPM");
		}
	}

	ForeColor(blackColor);

	GetFNum("\pGeneva",&fontNum);
	TextFont(fontNum);
	TextSize(10);
	PenPat(&qd.black);
	TextMode(srcOr);
}

//  doStartAnimCursor

void  doStartAnimCursor(void)
{
	gVBLCount = 30;
	gSleepTime = 0;

	if(doGetAnimCursor(rFingersCursor) == false)
		ExitToShell();

	doInstallSystemVBLTask();

	gAnimatedCursorActive = true;

	MoveTo(40,110);
	DrawString("\pPress any key to");
	MoveTo(30,125);
	DrawString("\pstop animated cursor");
}

//  doGetAnimCursor

Boolean  doGetAnimCursor(SInt16 resourceID)
{
	SInt16	cursorID, a = 0;
	Boolean	noError = false;

	if((gAnimCursHdl = (animCursHandle) GetResource('acur',resourceID)))
	{
		noError = true;
		while((a < (*gAnimCursHdl)->numberOfFrames)  && noError)
		{
			cursorID = (SInt16) HiWord((SInt32) (*gAnimCursHdl)->frame[a]);

			(*gAnimCursHdl)->frame[a] = GetCursor(cursorID);
			if((*gAnimCursHdl)->frame[a])
				a++;
			else
				noError = false;
		}
	}

	if(noError)
		(*gAnimCursHdl)->whichFrame = 0;

	return(noError);
}

//  doInstallSystemVBLTask

void  doInstallSystemVBLTask(void)
{
	gVBLRec.vblTaskRec.qType		= vType;	
	gVBLRec.vblTaskRec.vblAddr	= (VBLUPP) animCursVBLTask;
	gVBLRec.vblTaskRec.vblCount	= gVBLCount;	
	gVBLRec.vblTaskRec.vblPhase	= 0;

	gVBLRec.thisApplicationsA5 = SetCurrentA5();

	VInstall((QElemPtr) &gVBLRec.vblTaskRec);
}

//  animCursVBLTask

void  animCursVBLTask(void)
{
	VBLRecPtr	vblRecPtr;
	SInt32 		currentA5;

	vblRecPtr = (VBLRecPtr) GetVBLRec();
	currentA5 = SetA5(vblRecPtr->thisApplicationsA5);	

	SetCursor(*((*gAnimCursHdl)->frame[(*gAnimCursHdl)->whichFrame++]));
	if((*gAnimCursHdl)->whichFrame == (*gAnimCursHdl)->numberOfFrames)
		(*gAnimCursHdl)->whichFrame = 0;

	vblRecPtr->vblTaskRec.vblCount = gVBLCount;

	currentA5 = SetA5(currentA5);
}

//  doStopAnimCursor

void  doStopAnimCursor(void)
{
	SInt16		a;
	Rect			theRect;

	VRemove((QElemPtr) &gVBLRec.vblTaskRec);
	
	for(a=0;a<(*gAnimCursHdl)->numberOfFrames;a++)
		ReleaseResource((Handle) (*gAnimCursHdl)->frame[a]);

	ReleaseResource((Handle) gAnimCursHdl);

	gAnimatedCursorActive = false;
	gSleepTime = MAXLONG;

	SetCursor(&qd.arrow);

	SetRect(&theRect,30,100,150,130);
	RGBBackColor(&gWindowColour);
	FillRect(&theRect,&qd.white);
}

// 