// 
// WDEF.c													 Custom Window Definition Function for Floating Windows
// 
//
// This WDEF creates a utility window whose appearance conforms to that specified in the
// document titled Apple Grayscale Appearance for System 7.5 and published by Apple 
// Computer, Inc.  On black-and-white displays, the WDEF is drawn in black-and-white with
// an appearance similar to the black-and-white floating windows found in many commercial
// applications.  The WDEF supports only one variation code.  It provides for a close box 
// but not for a zoom box or window title.  
//
// The WDEF utilises three 'cicn' resources (unpurgeable), one for the close box in the
// normal state, one for the close box in the pressed state, and one to paint the 
// checkered pattern in the title bar.
//
// 

//  includes

#include <A4Stuff.h>
#include <Gestalt.h>
#include <ToolUtils.h>
#include <LowMem.h>

//  defines

#define	rCloseEnabledIcon	128
#define	rClosePressedIcon	129
#define	rCheckPatternIcon	130

//  global variables

Boolean 		gColorQuickDrawPresent = false;
Boolean			gColourDisplay				 = false;
Pattern			gBlackPattern;
CIconHandle	gCloseEnabledHdl;
CIconHandle	gClosePressedHdl;
CIconHandle	gCheckPatternHdl;
RGBColor		gWhite	= { 0xFFFF,0xFFFF,0xFFFF };
RGBColor		gGray1	= { 0xEEEE,0xEEEE,0xEEEE };	
RGBColor		gGray2	= { 0xDDDD,0xDDDD,0xDDDD };	
RGBColor		gGray3	= { 0xCCCC,0xCCCC,0xCCCC };
RGBColor		gGray4	= { 0xBBBB,0xBBBB,0xBBBB };
RGBColor		gGray6	= { 0x9999,0x9999,0x9999 };
RGBColor		gGray7	= { 0x8888,0x8888,0x8888 };
RGBColor		gGray8	= { 0x7777,0x7777,0x7777 };
RGBColor		gGray10	= { 0x5555,0x5555,0x5555 };
RGBColor		gBlack	= { 0x0000,0x0000,0x0000 };
Boolean			gToggle;

//  function prototypes

void		doInitMessage					(void);
void		doDrawMessage					(WindowPeek,SInt32);
SInt32	doHitMessage					(WindowPeek,SInt32);
void		doCalcRgnsMessage			(WindowPeek);
void		drawWindowColour			(WindowPeek);
void		drawWindowMono				(WindowPeek);
void		toggleGoAway					(WindowPeek);
void		drawGoAwayBox					(WindowPeek);
void		drawGoAwayBoxPressed	(WindowPeek);
void		getGoAwayRect					(WindowPeek,Rect *);
void		getContentRect				(WindowPeek,Rect *);
void		getStructRect					(WindowPeek,Rect *);
void		syncPorts							(void);

//  main

pascal SInt32  main(SInt16 varCode,WindowPeek windowPeek,SInt16 message,SInt32 param)
{
	SInt32		result;
	GrafPtr		oldPort;
	PenState	oldPenState;
	
	EnterCodeResource();

	GetPenState(&oldPenState);
	GetPort(&oldPort);

	if(gColorQuickDrawPresent)
		syncPorts();

	result = 0;

	switch(message)
	{
		case wNew:
			doInitMessage();
			break;

		case wDraw:
			if(windowPeek->visible)
				doDrawMessage(windowPeek,param);
			break;

		case wHit:
			result = doHitMessage(windowPeek,param);
			break;

		case wCalcRgns:
			doCalcRgnsMessage(windowPeek);
			break;
	}

	SetPenState(&oldPenState);
	SetPort(oldPort);

	ExitCodeResource();
	
	return(result);
}

//  doInitMessage

void  doInitMessage(void)
{
	OSErr			osErr;
	SInt32		response;
	GDHandle	mainDeviceHdl;
	SInt16		bitsPerPixel, a;

	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response >= gestalt8BitQD)
	{
		gColorQuickDrawPresent = true;

		mainDeviceHdl = LMGetMainDevice();
		bitsPerPixel = (*(*mainDeviceHdl)->gdPMap)->pixelSize;
		if(bitsPerPixel > 1)
			gColourDisplay = true;
	}

	for(a=0;a<8;a++)
		gBlackPattern.pat[a] = 0xFF;

	if(gColorQuickDrawPresent)
	{
		gCloseEnabledHdl = GetCIcon(rCloseEnabledIcon);
		gClosePressedHdl = GetCIcon(rClosePressedIcon);
		gCheckPatternHdl = GetCIcon(rCheckPatternIcon);
	}

	gToggle = false;	
}

//  doDrawMessage

void  doDrawMessage(WindowPeek windowPeek,SInt32 param)
{
	param &= 0x0000FFFF;

	switch(param)
	{
		case wNoHit:
			if(gColorQuickDrawPresent && gColourDisplay)
				drawWindowColour(windowPeek);
			else
				drawWindowMono(windowPeek);
			break;

		case wInGoAway:
			toggleGoAway(windowPeek);
			break;

		default:
			return;
	}
}

//  doHitMessage

SInt32  doHitMessage(WindowPeek windowPeek,SInt32 param)
{
	Point	where;
	Rect	goAwayRect;

	where.v = HiWord(param);
	where.h = LoWord(param);

	if(PtInRgn(where,windowPeek->contRgn))
		return(wInContent);
	else if(PtInRgn(where,windowPeek->strucRgn))
	{
		if(windowPeek->goAwayFlag)
		{
			getGoAwayRect(windowPeek,&goAwayRect);
			if(PtInRect(where,&goAwayRect))
				return(wInGoAway);
		}

		return(wInDrag);
	}

	return(wNoHit);
}

//  doCalcRgnsMessage

void  doCalcRgnsMessage(WindowPeek windowPeek)
{
	RgnHandle	tempRgn;
	Rect			theRect;

	tempRgn = NewRgn();

	getContentRect(windowPeek,&theRect);
	RectRgn(windowPeek->contRgn,&theRect);

	getStructRect(windowPeek,&theRect);
	RectRgn(windowPeek->strucRgn,&theRect);
	OffsetRect(&theRect,1,1);
	theRect.left += 1;
	theRect.top += 1;
	RectRgn(tempRgn,&theRect);
	UnionRgn(tempRgn,windowPeek->strucRgn,windowPeek->strucRgn);

	DisposeRgn(tempRgn);
}

//  drawWindowColour

void  drawWindowColour(WindowPeek windowPeek)
{
	RGBColor	oldForeColour;
	RGBColor	oldBackColour;
	Rect			contentRect, structRect, theRect;
	SInt16		a, b;
	
	GetForeColor(&oldForeColour);
	GetBackColor(&oldBackColour);
	PenSize(1,1);
	PenPat(&gBlackPattern);
	PenMode(patCopy);
	
	if(windowPeek->hilited)
	{
		getContentRect(windowPeek,&contentRect);

		RGBForeColor(&gBlack);
		InsetRect(&contentRect,-1,-1);
		FrameRect(&contentRect);

		getStructRect(windowPeek,&structRect);
		
		RGBForeColor(&gGray3);
		SetRect(&theRect,structRect.left + 2,structRect.top + 2,structRect.right - 2,
						structRect.top + 12);
		PaintRect(&theRect);
		
		SetRect(&theRect,structRect.left + 14,structRect.top + 3,structRect.left + 22,
					structRect.top + 11);
		b = (((structRect.right - 4) - (structRect.left + 14)) / 9) + 1;
		for(a=0;a<b+1;a++)
		{
			PlotCIcon(&theRect,gCheckPatternHdl);
			OffsetRect(&theRect,9,0);
		}
		
		if(windowPeek->goAwayFlag)
			drawGoAwayBox(windowPeek);
		
		RGBForeColor(&gBlack);
		FrameRect(&structRect);
		MoveTo(structRect.left + 2,structRect.bottom);
		LineTo(structRect.right,structRect.bottom);
		LineTo(structRect.right,structRect.top + 2);
		
		RGBForeColor(&gWhite);
		MoveTo(structRect.left + 1,structRect.bottom - 3);
		LineTo(structRect.left + 1,structRect.top + 1);
		LineTo(structRect.right - 3,structRect.top + 1);
		MoveTo(structRect.left + 3,structRect.top + 11);
		LineTo(structRect.left + 11,structRect.top + 11);
		LineTo(structRect.left + 11,structRect.top + 3);
		
		RGBForeColor(&gGray1);
		MoveTo(structRect.left + 2,structRect.bottom - 3);
		LineTo(structRect.right - 3,structRect.bottom - 3);
		LineTo(structRect.right - 3,structRect.top + 13);
		
		RGBForeColor(&gGray3);
		MoveTo(structRect.right - 2,structRect.top + 1);
		LineTo(structRect.right - 2,structRect.top + 1);
		RGBForeColor(&gGray4);
		MoveTo(structRect.left + 1,structRect.bottom - 2);
		LineTo(structRect.left + 1,structRect.bottom - 2);
	
		RGBForeColor(&gGray6);
		MoveTo(structRect.left + 2,structRect.bottom - 2);
		LineTo(structRect.right - 2,structRect.bottom - 2);
		LineTo(structRect.right - 2,structRect.top + 2);
		MoveTo(structRect.right - 3,structRect.top + 12);
		LineTo(structRect.left + 2,structRect.top + 12);
		LineTo(structRect.left + 2,structRect.bottom - 4);
	
		RGBForeColor(&gGray7);
		MoveTo(structRect.left + 2,structRect.top + 10);
		LineTo(structRect.left + 2,structRect.top + 2);
		LineTo(structRect.left + 10,structRect.top + 2);
	}	
	else
	{
		RGBForeColor(&gGray10);
		getContentRect(windowPeek,&contentRect);
		InsetRect(&contentRect,-1,-1);
		FrameRect(&contentRect);

		getStructRect(windowPeek,&structRect);
		FrameRect(&structRect);
		MoveTo(structRect.left + 2,structRect.bottom);
		LineTo(structRect.right,structRect.bottom);
		LineTo(structRect.right,structRect.top + 2);

		RGBForeColor(&gGray2);
		InsetRect(&structRect,1,1);
		PenSize(2,2);
		FrameRect(&structRect);
		structRect.bottom = structRect.top + 12;
		PaintRect(&structRect);
	}

	RGBForeColor(&oldForeColour);
	RGBBackColor(&oldBackColour);
}

//  drawWindowMono

void  drawWindowMono(WindowPeek windowPeek)
{
	Rect		contentRect, structRect, theRect;
	SInt16	a, b;
	UInt8		pattern;
	Pattern	checkPattern;
					
	PenSize(1,1);
	PenPat(&gBlackPattern);
	PenMode(patCopy);
	
	if(windowPeek->goAwayFlag && windowPeek->hilited)
	{
		ForeColor(blackColor);
		BackColor(whiteColor);

		getContentRect(windowPeek,&contentRect);

		InsetRect(&contentRect,-1,-1);
		FrameRect(&contentRect);

		getStructRect(windowPeek,&structRect);

		FrameRect(&structRect);
		MoveTo(structRect.left + 2,structRect.bottom);
		LineTo(structRect.right,structRect.bottom);
		LineTo(structRect.right,structRect.top + 2);

		SetRect(&theRect,structRect.left + 1,structRect.top + 1, structRect.right - 1,
						structRect.top + 10);
		EraseRect(&theRect);

		for(a=0;a<8;a++)
			checkPattern.pat[a] = 0x00;

		if(structRect.left & 1)
			pattern = 0xAA;
		else
			pattern = 0x55;

		if(structRect.top & 1)
			b = 1;
		else
			b = 0;
			
		for(a=b;a<8;a+=2)
			checkPattern.pat[a] = pattern;

		PenPat(&checkPattern);
		SetRect(&theRect,structRect.left + 11,structRect.top + 2, structRect.right - 2,
						structRect.top + 9);
		PaintRect(&theRect);

		PenPat(&gBlackPattern);
		
		if(windowPeek->goAwayFlag)
			drawGoAwayBox(windowPeek);
	}	
	else
	{
		SetRect(&theRect,structRect.left + 1,structRect.top + 1, structRect.right - 1,
						structRect.top + 13);
		EraseRect(&theRect);
	}
}

//  toggleGoAway

void  toggleGoAway(WindowPeek windowPeek)
{
	gToggle = !gToggle;

	if(gToggle)
		drawGoAwayBoxPressed(windowPeek);
	else
		drawGoAwayBox(windowPeek);
}

//  drawGoAwayBox

void  drawGoAwayBox(WindowPeek windowPeek)
{
	Rect	theRect;

	getGoAwayRect(windowPeek,&theRect);

	if(gColorQuickDrawPresent && gColourDisplay)
		PlotCIcon(&theRect,gCloseEnabledHdl);
	else
	{
		EraseRect(&theRect);
		PenSize(1,1);
		FrameRect(&theRect);
	}
}

//  drawGoAwayBoxPressed

void  drawGoAwayBoxPressed(WindowPeek windowPeek)
{
	Rect	theRect;

	getGoAwayRect(windowPeek,&theRect);

	if(gColorQuickDrawPresent && gColourDisplay)
		PlotCIcon(&theRect,gClosePressedHdl);
	else
	{
		PenSize(2,2);
		FrameRect(&theRect);
	}
}
	
//  getGoAwayRect

void  getGoAwayRect(WindowPeek windowPeek,Rect *theRect)
{
	getStructRect(windowPeek,theRect);

	if(gColorQuickDrawPresent && gColourDisplay)
	{
		theRect->top += 3;
		theRect->left += 3;
		theRect->bottom = theRect->top + 8;
		theRect->right = theRect->left + 8;
	}
	else
	{
		theRect->top += 2;
		theRect->left += 2;
		theRect->bottom = theRect->top + 7;
		theRect->right = theRect->left + 7;
	}
}

//  getContentRect		

void  getContentRect(WindowPeek windowPeek,Rect *theRect)
{
	GrafPtr	oldPort;

	*theRect = windowPeek->port.portRect;

	GetPort(&oldPort);
	SetPort((GrafPtr) windowPeek);

	LocalToGlobal((Point *) &(theRect->top));
	LocalToGlobal((Point *) &(theRect->bottom));

	SetPort(oldPort);
}

//  getStructRect

void  getStructRect(WindowPeek windowPeek,Rect *theRect)
{
	getContentRect(windowPeek,theRect);

	if(gColorQuickDrawPresent && gColourDisplay)
	{
		InsetRect(theRect,-4,-4);
		theRect->top -= 10;
	}
	else
	{
		theRect->top -= 10;
		InsetRect(theRect,-1,-1);
	}
}

//  syncPorts

void  syncPorts(void)
{
	GrafPtr		bwPort;
	CGrafPtr	colorPort;

	GetWMgrPort(&bwPort);
	GetCWMgrPort(&colorPort);
	SetPort((GrafPtr) colorPort);

	BlockMoveData(&bwPort->pnLoc,&colorPort->pnLoc,10);
	BlockMoveData(&bwPort->pnVis,&colorPort->pnVis,14);

	PenPat(&bwPort->pnPat);
	BackPat(&bwPort->bkPat);
}

// 