// 
// AppleEventsPPC.c
// 
// 
// This program:
//
// 	Installs handlers for the required Apple events.
//
// 	Responds to the receipt of required Apple events by displaying descriptive text in
//		a window opened for that purpose, and by opening simulated document windows as
//		appropriate.  These responses result from the user:
//
//				Double clicking on the application's icon, or selecting the icon and choosing
//				Open from the Finder's File menu, thus causing the receipt of an Open
//				Application event.
//
//				Double clicking on one of the document icons, selecting one or both of the
//				document icons and choosing Open from the Finder's File menu, or dragging one
//				or both of the document icons onto the application's icon, thus causing the
//				receipt of an Open Documents event.
//
//				Selecting one or both of the document icons and choosing Print from the
//				Finder's file menu, thus causing the receipt of a Print Documents event and,
//				if the application was not already running, a subsequent Quit Application event.
//
//				While the application is running, choosing Shut Down or Restart from the
//				Finder's Special menu, thus causing the receipt of a Quit Application event.   
//
// The program, which is intended to be run as a built application and not from within
// CodeWarrior, utilises the following resources:
//
// 	'WIND' resources (purgeable) (initially visible) for the descriptive text display
//		window and simulated document windows.
//
// 	'MBAR' and 'MENU' resources (preload, non-purgeable).
//
// 	An 'ALRT' resource, and associated 'DITL' and 'STR#' resources, for displaying
//		error messages (purgeable).
//
// 	'ICN#', 'ics#', 'ics4', 'ics8', 'icl4', and 'icl8' resources (that is, an icon
//		family) for the application and for the application's documents. (Purgeable.)
//
// 	'FREF' resources for the application and the application's 'TEXT' documents, which
//		link the icons with the file types they represent, and which allow users to launch
//		the application by dragging the document icons to the application icon.  (Non-
//		purgeable.)
//
// 	The application's signature resource (non-purgeable), which enables the Finder to
//		identify and start up the application when the user double clicks the application's
//		document icons.  
//
// 	A 'BNDL' resource (non-purgeable), which groups together the application's
//		signature, icon and 'FREF' resources.  
//
// 	An 'hfdr' resource (purgeable), which provides the customised help balloon for the
//		application icon. 
//
// 	A 'vers' resource (purgeable), which allows users to ascertain the version number
//		of the application.
//
// 	A 'SIZE' resource with the isHighLevelEventAware,	acceptSuspendResumeEvents, and
//		and is32BitCompatible flags set.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <AppleEvents.h>

//  defines

#define mApple								128
#define mFile									129
#define  iQuit								11
#define rMenubar							128
#define rDisplayWindow				128
#define rDocWindow						129
#define rErrorAlert						128
#define rErrorStrings					128
#define eInstallHandler				1
#define eGetRequiredParam			2
#define eGetDescriptorRecord	3
#define eMissedRequiredParam	4
#define eCannotOpenFile				5
#define eCannotPrintFile			6
#define eCannotOpenWindow			7
#define eMenus								8	
//  global variables

Boolean		gDone;
WindowPtr	gWindowPtr;
WindowPtr	gWindowPtrs[10];
SInt16		gNumberOfWindows		= 0;
Boolean		gApplicationWasOpen = false;

AEEventHandlerUPP	doOpenAppEventRD;																						 // PowerPC
AEEventHandlerUPP	doOpenDocsEventRD;																					 // PowerPC
AEEventHandlerUPP	doPrintDocsEventRD;																					 // PowerPC
AEEventHandlerUPP	doQuitAppEventRD;																						 // PowerPC

//  function prototypes

void					main									(void);
void					doInitManagers				(void);
void					doInstallAEHandlers		(void);
void					doEvents							(EventRecord *);
void					doMouseDown						(EventRecord *);
pascal OSErr	doOpenAppEvent				(AppleEvent *,AppleEvent *,SInt32);
pascal OSErr	doOpenDocsEvent				(AppleEvent *,AppleEvent *,SInt32);
pascal OSErr	doPrintDocsEvent			(AppleEvent *,AppleEvent *,SInt32);
pascal OSErr	doQuitAppEvent				(AppleEvent *,AppleEvent *,SInt32);
OSErr					hasGotRequiredParams	(AppleEvent *);
Boolean				doOpenFile						(FSSpec *,SInt32,SInt32);
Boolean				doPrintFile						(FSSpec *,SInt32,SInt32);
void					doPrepareToTerminate	(void);
WindowPtr			doNewWindow						(void);
void					doMenuChoice					(SInt32);
void					doAppleMenu						(SInt16);
void					doFileMenu						(SInt16);
void					doError								(SInt16);
void					drawTextString				(Str255);

//  main

void  main(void)
{
	EventRecord	eventRec;
	Handle			menubarHdl;
	MenuHandle	menuHdl;

	//  initialise managers

	doInitManagers();
	
	//  create routine decriptors

	doOpenAppEventRD		= NewAEEventHandlerProc((ProcPtr) doOpenAppEvent);			 // PowerPC
	doOpenDocsEventRD		= NewAEEventHandlerProc((ProcPtr) doOpenDocsEvent);			 // PowerPC
	doPrintDocsEventRD	= NewAEEventHandlerProc((ProcPtr) doPrintDocsEvent);		 // PowerPC
	doQuitAppEventRD		= NewAEEventHandlerProc((ProcPtr) doQuitAppEvent);			 // PowerPC
	
	//  open a window
	
	if(!(gWindowPtr = GetNewWindow(rDisplayWindow,NULL,(WindowPtr) -1)))
	{
		doError(eCannotOpenWindow);
		ExitToShell();
	}

	SetPort(gWindowPtr);
	
	TextSize(10);

	//  set up menu bar and menus
	
	if(!(menubarHdl = GetNewMBar(rMenubar)))
		doError(eMenus);
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	if(!(menuHdl = GetMenuHandle(mApple)))
		doError(eMenus);
	else
		AppendResMenu(menuHdl,'DRVR');

	//  install Apple event handlers

	doInstallAEHandlers();

	//  event loop
	
	gDone = false;

	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,180,NULL))
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doInstallAEHandlers

void  doInstallAEHandlers(void)
{
	OSErr	err;

	err=AEInstallEventHandler(kCoreEventClass,kAEOpenApplication,
														doOpenAppEventRD,0L,false);												 // PowerPC
	if(err != noErr)	doError(eInstallHandler);

	err=AEInstallEventHandler(kCoreEventClass,kAEOpenDocuments,
													 doOpenDocsEventRD,0L,false);												 // PowerPC
	if(err != noErr)	doError(eInstallHandler);

	err=AEInstallEventHandler(kCoreEventClass,kAEPrintDocuments,
													 doPrintDocsEventRD,0L,false);											 // PowerPC
	if(err != noErr)	doError(eInstallHandler);

	err=AEInstallEventHandler(kCoreEventClass,kAEQuitApplication,
													 doQuitAppEventRD,0L,false);												 // PowerPC
	if(err != noErr)	doError(eInstallHandler);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	SInt8	charCode;

	switch(eventRecPtr->what)
	{
		case kHighLevelEvent:
			AEProcessAppleEvent(eventRecPtr);
			break;

		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doMenuChoice(MenuKey(charCode));
			}
			break;

		case updateEvt:
			BeginUpdate((WindowPtr)eventRecPtr->message);
			EndUpdate((WindowPtr)eventRecPtr->message);
			break;

		case osEvt:
			HiliteMenu(0);
			break;
	}
}
	
//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;
	SInt32		menuChoice;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			menuChoice = MenuSelect(eventRecPtr->where);
			doMenuChoice(menuChoice);
			break;
			
		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;
	}
}

//  doOpenAppEvent

pascal OSErr  doOpenAppEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefCon)
{
	OSErr			osErr;
	WindowPtr	windowPtr;
	SInt32		finalTicks;

	gApplicationWasOpen = true;

	osErr = hasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{
		drawTextString("\pReceived an Apple event: OPEN APPLICATION.");
		drawTextString("\p     Opening an untitled window in reponse.");
		Delay(100,&finalTicks);

		windowPtr = doNewWindow();
		SetWTitle(windowPtr,"\pUntitled 1");

		return(noErr);
	}
	else
		return(osErr);
}

//  doOpenDocsEvent

pascal OSErr  doOpenDocsEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefcon)
{
	FSSpec			fileSpec;
	AEDescList	docList;
	OSErr				osErr, ignoreErr;
	SInt32			index, numberOfItems;
	Size				actualSize;
	AEKeyword		keyWord;
	DescType		returnedType;
	Boolean			result;

	osErr = AEGetParamDesc(appEvent,keyDirectObject,typeAEList,&docList);

	if(osErr == noErr)
	{
		osErr = hasGotRequiredParams(appEvent);
		if(osErr == noErr)
		{
			AECountItems(&docList,&numberOfItems);
			if(osErr == noErr)
			{
				for(index=1;index<=numberOfItems;index++)
				{
					osErr = AEGetNthPtr(&docList,index,typeFSS,&keyWord,&returnedType,
															(Ptr) &fileSpec,sizeof(fileSpec),&actualSize);
					if(osErr == noErr)
					{
						if(!(result = doOpenFile(&fileSpec,index,numberOfItems)))
							doError(eCannotOpenFile);
					}
					else
						doError(eGetDescriptorRecord);
				}
			}
		}
		else
			doError(eMissedRequiredParam);

		ignoreErr = AEDisposeDesc(&docList);
	}
	else
		doError(eGetRequiredParam);

	return(osErr);
}

//  doPrintDocsEvent

pascal OSErr  doPrintDocsEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefcon)
{
	FSSpec			fileSpec;
	AEDescList	docList;
	OSErr				osErr, ignoreErr;
	SInt32			index, numberOfItems;
	Size				actualSize;
	AEKeyword		keyWord;
	DescType		returnedType;
	Boolean			result;
	
	osErr = AEGetParamDesc(appEvent,keyDirectObject,typeAEList,&docList);

	if(osErr == noErr)
	{
		osErr = hasGotRequiredParams(appEvent);
		if(osErr == noErr)
		{
			AECountItems(&docList,&numberOfItems);
			if(osErr == noErr)
			{
				for(index=1;index<=numberOfItems;index++)
				{
					osErr = AEGetNthPtr(&docList,index,typeFSS,&keyWord,&returnedType,
															(Ptr) &fileSpec,sizeof(fileSpec),&actualSize);
					if(osErr == noErr)
					{
						if(!(result = doPrintFile(&fileSpec,index,numberOfItems)))
							doError(eCannotPrintFile);
					}
					else
						doError(eGetDescriptorRecord);
				}
			}
		}
		else
			doError(eMissedRequiredParam);

		ignoreErr = AEDisposeDesc(&docList);
	}
	else
		doError(eGetRequiredParam);

	return(osErr);
}

//  doQuitAppEvent

pascal OSErr  doQuitAppEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefcon)
{
	OSErr		osErr;

	osErr = hasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{
		doPrepareToTerminate();
		return(noErr);
	}
	else
		return(osErr);
}

//  hasGotRequiredParams

OSErr  hasGotRequiredParams(AppleEvent *appEvent)
{
	DescType	returnedType;
	Size			actualSize;
	OSErr			osErr;

	osErr = AEGetAttributePtr(appEvent,keyMissedKeywordAttr,typeWildCard,&returnedType,
														NULL,0,&actualSize);

	if(osErr == errAEDescNotFound)
		return(noErr);
	else if(osErr == noErr)
		return(errAEParamMissed);
}

//  doOpenFile

Boolean  doOpenFile(FSSpec *fileSpecPtr,SInt32 index,SInt32 numberOfItems)
{
	WindowPtr	windowPtr;
	SInt32		finalTicks;

	gApplicationWasOpen = true;

	if(index == 1)
		drawTextString("\pReceived an Apple event: OPEN DOCUMENTS.");

	if(numberOfItems == 1)
	{
		drawTextString("\p     The file to open is: ");
		DrawString(fileSpecPtr->name);
		drawTextString("\p     Opening titled window in reponse.");
		Delay(100,&finalTicks);
	}
	else
	{
		if(index == 1)
		{		
			drawTextString("\p     The files to open are: ");
			DrawString(fileSpecPtr->name);
		}
		else
		{
			DrawString("\p and ");
			DrawString(fileSpecPtr->name);
			drawTextString("\p     Opening titled windows in reponse.");
			Delay(100,&finalTicks);
		}
	}			

	if(windowPtr = doNewWindow())
	{
		SetWTitle(windowPtr,fileSpecPtr->name);
		return(true);
	}	
	else
		return(false);
}

//  doPrintFile

Boolean  doPrintFile(FSSpec *fileSpecPtr,SInt32 index,SInt32 numberOfItems)
{
	WindowPtr	windowPtr;
	SInt32		finalTicks;

	if(index == 1)
		drawTextString("\pReceived an Apple event: PRINT DOCUMENTS");

	if(numberOfItems == 1)
	{
		drawTextString("\p     The file to print is: ");
		DrawString(fileSpecPtr->name);
		windowPtr = doNewWindow();
		SetWTitle(windowPtr,fileSpecPtr->name);
		drawTextString("\p     I would present the Print dialog box first and then print");
		drawTextString("\p      the document when the user has made his settings.");
		Delay(100,&finalTicks);
		drawTextString("\p     Assume that I am now printing the document.");
	}
	else
	{
		if(index == 1)
		{		
			drawTextString("\p     The first file to print is: ");
			DrawString(fileSpecPtr->name);
			drawTextString("\p      I would present the Print dialog box for the first file");
			drawTextString("\p      only and use the user's settings to print both files.");
		}
		else
		{
			Delay(200,&finalTicks);
			drawTextString("\p     The second file to print is: ");
			DrawString(fileSpecPtr->name);
			drawTextString("\p      I am using the Print dialog box settings used for the");
			drawTextString("\p      first file.");
		}

		windowPtr = doNewWindow();
		SetWTitle(windowPtr,fileSpecPtr->name);
		Delay(200,&finalTicks);
		drawTextString("\p     Assume that I am now printing the document.");
		Delay(200,&finalTicks);
	}

	if(numberOfItems == index)
	{
		if(!gApplicationWasOpen)
		{
			drawTextString("\p      Since the application was not already open, I expect to"); 
			drawTextString("\p      receive a QUIT APPLICATION event when I have finished.");
		}
		else
		{
			drawTextString("\p      Since the application was already open, I do NOT expect"); 
			drawTextString("\p      to receive a QUIT APPLICATION event when I have finished.");
		}
	
		Delay(500,&finalTicks);
		drawTextString("\p     Finished print job.");
	}

	DisposeWindow(windowPtr);
	return(true);
}

//  doPrepareToTerminate

void  doPrepareToTerminate(void)
{
	SInt32	finalTicks;

	drawTextString("\pReceived an Apple event: QUIT APPLICATION");

	if(gApplicationWasOpen)
	{
		drawTextString("\p     I would now ask the user to save any unsaved files before");
		drawTextString("\p      terminating myself in reponse to the event.");
		drawTextString("\p     Click the mouse when ready to terminate.");
		while(!Button()) ;
	}
	else
	{
		drawTextString("\p     Terminating myself in response");
		Delay(300,&finalTicks);
	}

	// If the user did not click the Cancel button in a Save dialog box:

	gDone = true;
}

//  doNewWindow

WindowPtr  doNewWindow(void)
{	
	if(!(gWindowPtrs[gNumberOfWindows] = GetNewWindow(rDocWindow,NULL,(WindowPtr) -1)))
		doError(eCannotOpenWindow);

	gNumberOfWindows++;

	return(gWindowPtrs[gNumberOfWindows -1]);
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16  menuID, menuItem;

	menuID	=HiWord(menuChoice);
	menuItem=LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			doAppleMenu(menuItem);
			break;
			
		case mFile:
			doFileMenu(menuItem);
			break;
	}

	HiliteMenu(0);
}

//  doAppleMenu

void	doAppleMenu(SInt16 menuItem)
{
	Str255	itemName;
	SInt16	daDriverRefNum;

	GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
	daDriverRefNum = OpenDeskAcc(itemName);
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	if(menuItem  == iQuit)
		gDone = true;
}

//  doError

void  doError(SInt16 errorType)
{
	Str255	errorString;
	
	SetCursor(&qd.arrow);

	GetIndString(errorString,rErrorStrings,errorType);
	ParamText(errorString,NULL,NULL,NULL);
	if(errorType < 7)
		CautionAlert(rErrorAlert,NULL);
	else
	{
		StopAlert(rErrorAlert,NULL);
		ExitToShell();
	}
}

//  drawTextString

void	drawTextString(Str255 eventString)
{
	RgnHandle	tempRegion;

	tempRegion = NewRgn();

	ScrollRect(&gWindowPtr->portRect,0,-15,tempRegion);
	DisposeRgn(tempRegion);

	MoveTo(8,176);
	DrawString(eventString);
}

// 
