// 
// ListsPPC.c
// 
// 
// This program allows the user to open a dialog box by choosing the Dialog With Lists
// item in the Demonstration menu.
//
// The dialog box contains two lists.  The cells of one list contain text.  The cells of
// the other list contain icon-like pictures and their titles.
//
// The text list uses the default list definition function.
// 
// The picture list uses a custom list definition function.  The source code for the
// custom list definition function is at the file LDEF.c in the LDEF folder.
//
// The currently active list is outlined by a two-pixel-wide border.  The currently
// active list can be changed by clicking in the non-active list or by pressing the tab
// key.
//
// The text list uses the default cell-selection algorithm; accordingly, multiple cells,
// including discontiguous multiple cells, may be selected.  The picture list also
// supports arrow key selection (of single or multiple cells) and type selection.
//
// The constant lOnlyOne is assigned to the selFlags field of the picture list's list
// record.  Accordingly, the selection of multiple items is not possible in this list.
// Arrow key selection (of single cells) is, however, supported.
//
// When the dialog is dismissed by clicking on the OK button, or by double-clicking on a
// cell in the active list, the user's selections are displayed in a window opened by the
// program at program launch.  (Note that the use of the Return, Enter, Esc and
// Command-period keys as alternatives to clicking the OK and Cancel buttons in the
// dialog box is not supported in this program.)
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Demonstration
//		menus (preload, non-purgeable).
//
// 	A 'WIND' resource (purgeable) (initially visible) for the window in which the
//		user's selections are displayed.  
//
// 	A'DLOG' resource (purgeable) and associated 'DITL' resource (purgeable) for the
//		dialog box.
//
// 	'STR#' resources (purgeable) containing the text strings for the text list.
//
// 	'PICT' resources (non-purgeable) containing the images for the picture list.
//
// 	An 'LDEF' resource (non-purgeable) containing the custom list definition function
//		used by the picture list.
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible flags set.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Lists.h>
#include <LowMem.h>  

//  defines

#define mApple							128
#define  iAbout							1
#define mFile								129
#define  iQuit							11
#define mDemonstration			131
#define  iDialog						1
#define rMenubar						128
#define rWindow							128
#define rDialog							129
#define  iOK								1
#define  iCancel						2
#define  iUserItemText			3
#define  iUserItemPict			4
#define rListCellStrings		128
#define rListCellPicts			128
#define rListCellPictTitles	129
#define kUpArrow						0x1e
#define kDownArrow					0x1f
#define kTab								0x09
#define kScrollBarWidth			15
#define kMaxKeyThresh				120
#define kSystemLDEF					0
#define kCustomLDEF					128
#define MAXLONG							0x7FFFFFFF

//  typedefs

typedef struct
{
	ListHandle	textListHdl;
	ListHandle	pictListHdl;
} ListsRec, **ListsRecHandle;

//  global variables

Boolean			gDone;
Boolean			gInBackground;
WindowPtr		gWindowPtr;
ListHandle	gCurrentListHdl;
Str255			gTSString;	
SInt16			gTSResetThreshold;	
SInt32			gTSLastKeyTime;
ListHandle	gTSLastListHit;	

ListSearchUPP doSearchPartialMatchRD;																					 // PowerPC

//  function prototypes

void					main												(void);
void					doInitManagers							(void);
void					doEvents										(EventRecord *);
void					doMouseDown									(EventRecord *);
void					doKeyDown										(SInt8,EventRecord *);
void					doUpdate										(EventRecord *);
void					doUpdateLists								(WindowPtr);
void					doActivate									(EventRecord *);
void					doActivateDialog						(WindowPtr,Boolean);
void					doOSEvent										(EventRecord *);
void					doInContent									(EventRecord *);
void					doItemHitInDialog						(DialogPtr,SInt16);
void					doCreateDialogWithLists			(void);
ListHandle		doCreateTextList						(DialogPtr,Rect,SInt16,SInt16);
void					doAddRowsAndDataToTextList	(ListHandle,SInt16);
void					doAddTextItemAlphabetically	(ListHandle,Str255);
ListHandle		doCreatePictList						(DialogPtr,Rect,SInt16,SInt16);
void					doAddRowsAndDataToPictList	(ListHandle,SInt16);
void					doHandleArrowKey						(SInt8,EventRecord *,Boolean);
void					doArrowKeyMoveSelection			(ListHandle,SInt8,Boolean);
void					doArrowKeyExtendSelection		(ListHandle,SInt8,Boolean);
void					doTypeSelectSearch					(ListHandle,EventRecord *);
pascal SInt16	doSearchPartialMatch				(Ptr,Ptr,SInt16,SInt16);
Boolean				doFindFirstSelectedCell			(ListHandle,Cell *);
void					doFindLastSelectedCell			(ListHandle,Cell *);
void					doFindNewCellLoc						(ListHandle,Cell,Cell *,SInt8,Boolean);
void					doSelectOneCell							(ListHandle,Cell);
void					doMakeCellVisible						(ListHandle,Cell);
void					doResetTypeSelection				(void);
void					doRotateCurrentList					(void);
void					doDrawListsBorders					(ListHandle,ListHandle);
void					doDrawActiveListBorder			(ListHandle);
void					doDisplaySelections					(void);
void					doAdjustMenus								(void);
void					doMenuChoice								(SInt32);
void					doDrawDialogDefaultButton		(DialogPtr);

//  main

void  main(void)
{
	Handle				menubarHdl;
	MenuHandle		menuHdl;
	EventRecord		eventRec;

	//  initialise managers

	doInitManagers();
	
	//  create routine descriptor

	doSearchPartialMatchRD = NewListSearchProc((ProcPtr) doSearchPartialMatch);	 // PowerPC

	//  set up menu bar and menus
	
	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');
	
	//  open window 

	if(!(gWindowPtr = GetNewWindow(rWindow,NULL,(WindowPtr)-1)))
			ExitToShell();

	SetPort(gWindowPtr);
	TextSize(10);

	//  enter eventLoop
	
	gDone = false;
	
	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL))
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
	FlushEvents(everyEvent,0);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	SInt8	charCode;

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuKey(charCode));
			}
			doKeyDown(charCode,eventRecPtr);
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case activateEvt:
			doActivate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			HiliteMenu(0);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	SInt16		partCode;
	WindowPtr	windowPtr;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			doAdjustMenus();
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
			{
				if(((WindowPeek) (FrontWindow()))->windowKind == dialogKind)
					SysBeep(10);
				else
					SelectWindow(windowPtr);
			}
			else
			{
				if(((WindowPeek) (FrontWindow()))->windowKind == dialogKind)
					doInContent(eventRecPtr);
			}
			break;
			
		case inDrag:
			if((((WindowPeek) FrontWindow())->windowKind == dialogKind) && 
				 (((WindowPeek) windowPtr)->windowKind != dialogKind))
			{
				SysBeep(10);
				return;
			}
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;
	}
}

//  doKeyDown

void  doKeyDown(SInt8 charCode,EventRecord *eventRecPtr)
{
	ListsRecHandle	listsRecHdl;
	Boolean					allowExtendSelect;

	if(((WindowPeek) FrontWindow())->windowKind == dialogKind)
	{
		listsRecHdl = (ListsRecHandle) GetWRefCon(FrontWindow());

		if(charCode == kTab)
			doRotateCurrentList(); 
		else if(charCode == kUpArrow || charCode == kDownArrow)
		{
			if(gCurrentListHdl == (*listsRecHdl)->textListHdl)
				allowExtendSelect = true;
			else
				allowExtendSelect = false;
			doHandleArrowKey(charCode,eventRecPtr,allowExtendSelect);
		}
		else
		{
			if(gCurrentListHdl == (*listsRecHdl)->textListHdl)
				doTypeSelectSearch((*listsRecHdl)->textListHdl,eventRecPtr);
		}
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	
	windowPtr = (WindowPtr) eventRecPtr->message;

	BeginUpdate(windowPtr);

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		UpdateDialog(windowPtr,windowPtr->visRgn);
		doDrawDialogDefaultButton(windowPtr);	
		doUpdateLists(windowPtr);
	}

	EndUpdate(windowPtr);

}

//  doUpdateLists

void  doUpdateLists(WindowPtr windowPtr)
{
	ListsRecHandle	listsRecHdl;
	ListHandle			textListHdl, pictListHdl;

	listsRecHdl = (ListsRecHandle) GetWRefCon(windowPtr);
	
	textListHdl = (*listsRecHdl)->textListHdl;
	pictListHdl = (*listsRecHdl)->pictListHdl;

	SetPort((*textListHdl)->port);

	LUpdate((*(*textListHdl)->port).visRgn,textListHdl);
	LUpdate((*(*pictListHdl)->port).visRgn,pictListHdl);

	doDrawListsBorders(textListHdl,pictListHdl);
	doDrawActiveListBorder(textListHdl);
	doDrawActiveListBorder(pictListHdl);
}

//  doActivate

void  doActivate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventRecPtr->message;
	becomingActive = ((eventRecPtr->modifiers & activeFlag) == activeFlag);

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
		doActivateDialog(windowPtr,becomingActive);
}

//  doActivateDialog

void  doActivateDialog(WindowPtr windowPtr,Boolean becomingActive)
{
	ListsRecHandle	listRecsHdl;
	ListHandle			textListHdl, pictListHdl;
	SInt16 					itemType;
	Handle 					itemHdl;
	Rect 						itemRect;
			
	listRecsHdl = (ListsRecHandle) GetWRefCon(windowPtr);
	textListHdl = (*listRecsHdl)->textListHdl;
	pictListHdl = (*listRecsHdl)->pictListHdl;
	
	if(becomingActive)
	{
		GetDialogItem((DialogPtr) windowPtr,iOK,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,0);
		GetDialogItem((DialogPtr) windowPtr,iCancel,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,0);
		doDrawDialogDefaultButton(windowPtr);

		LActivate(true,textListHdl);
		LActivate(true,pictListHdl);

		doDrawActiveListBorder(gCurrentListHdl);
		doResetTypeSelection();
	}
	else
	{
		GetDialogItem((DialogPtr) windowPtr,iOK,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
		GetDialogItem((DialogPtr) windowPtr,iCancel,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
		doDrawDialogDefaultButton(windowPtr);

		LActivate(false,textListHdl);
		LActivate(false,pictListHdl);

		doDrawActiveListBorder(gCurrentListHdl);
	}
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			gInBackground = (eventRecPtr->message & resumeFlag) == 0;
			if(((WindowPeek) (FrontWindow()))->windowKind == dialogKind)
				doActivateDialog(FrontWindow(),!gInBackground);
			break;

		case mouseMovedMessage:
			break;
	}
}

//  doInContent

void  doInContent(EventRecord *eventRecPtr)
{
	GrafPtr					oldPort;
	ListsRecHandle	listsRecHdl;
	ListHandle			textListHdl, pictListHdl;
	Rect						textListRect, pictListRect, gCurrentListRect;
	Point						mouseXY;
	Boolean					isDoubleClick;
	DialogPtr				dialogPtr;
	SInt16					itemHit;

	GetPort(&oldPort);

	listsRecHdl = (ListsRecHandle) GetWRefCon(FrontWindow());
	textListHdl = (*listsRecHdl)->textListHdl;
	pictListHdl = (*listsRecHdl)->pictListHdl;

	textListRect = (*(*listsRecHdl)->textListHdl)->rView;
	pictListRect = (*(*listsRecHdl)->pictListHdl)->rView;
	gCurrentListRect = (*gCurrentListHdl)->rView;
	textListRect.right += kScrollBarWidth;	
	pictListRect.right += kScrollBarWidth;	
	gCurrentListRect.right += kScrollBarWidth;

	mouseXY = eventRecPtr->where;
	GlobalToLocal(&mouseXY);

	if((PtInRect(mouseXY,&textListRect) && gCurrentListHdl != textListHdl) ||
		 (PtInRect(mouseXY,&pictListRect) && gCurrentListHdl != pictListHdl))
	{
		doRotateCurrentList();
	}
	else if(PtInRect(mouseXY,&gCurrentListRect))
	{
		SetPort((*gCurrentListHdl)->port);
		isDoubleClick = LClick(mouseXY,eventRecPtr->modifiers,gCurrentListHdl);
		if(isDoubleClick)
			doItemHitInDialog(FrontWindow(),iOK);
	}
	else
	{
		if(DialogSelect(eventRecPtr,&dialogPtr,&itemHit))
			doItemHitInDialog(dialogPtr,itemHit);
	}

	SetPort(oldPort);
}

//  doItemHitInDialog

void  doItemHitInDialog(DialogPtr dialogPtr,SInt16 itemHit)
{
	ListsRecHandle	listsRecHdl;

	if(itemHit == iOK || itemHit == iCancel)
	{
		if(itemHit == iOK)
		 doDisplaySelections();

		listsRecHdl = (ListsRecHandle) GetWRefCon(dialogPtr);

		LDispose((*listsRecHdl)->textListHdl);
		LDispose((*listsRecHdl)->pictListHdl);
		DisposeHandle((Handle) listsRecHdl);
		DisposeDialog(dialogPtr);		

		doAdjustMenus();
	}
}

//  doCreateDialogWithLists

void  doCreateDialogWithLists(void)
{	
	DialogPtr				modalDlgPtr;
	ListsRecHandle	listsRecHdl;
	SInt16					fontNum, itemType;
	Handle					itemHdl;
	Rect						itemRect;
	ListHandle			textListHdl, pictListHdl;

	if(!(modalDlgPtr = GetNewDialog(rDialog,NULL,(WindowPtr) -1)))
		ExitToShell();

	if(!(listsRecHdl = (ListsRecHandle) NewHandle(sizeof(ListsRec))))
		ExitToShell();
	SetWRefCon(modalDlgPtr,(SInt32) listsRecHdl);

	SetPort(modalDlgPtr);

	GetFNum("\pChicago",&fontNum);
	TextFont(fontNum);
	TextSize(12);

	GetDialogItem(modalDlgPtr,iUserItemText,&itemType,&itemHdl,&itemRect);
	textListHdl = doCreateTextList(modalDlgPtr,itemRect,1,kSystemLDEF);

	GetDialogItem(modalDlgPtr,iUserItemPict,&itemType,&itemHdl,&itemRect);
	pictListHdl = doCreatePictList(modalDlgPtr,itemRect,1,kCustomLDEF);

	(*listsRecHdl)->textListHdl = textListHdl;
	(*listsRecHdl)->pictListHdl	= pictListHdl;

	(*textListHdl)->refCon = (SInt32) pictListHdl;
	(*pictListHdl)->refCon = (SInt32) textListHdl;

	gCurrentListHdl = textListHdl;

	ShowWindow(modalDlgPtr);
	doAdjustMenus();
}

//  doCreateTextList

ListHandle  doCreateTextList(DialogPtr dialogPtr,Rect listRect,SInt16 numCols,SInt16 lDef)
{
	Rect				dataBounds;
	Point				cellSize;
	ListHandle	textListHdl;
	Cell				theCell;

	SetRect(&dataBounds,0,0,numCols,0);
	SetPt(&cellSize,0,0);

	listRect.right = listRect.right - kScrollBarWidth;

	textListHdl = LNew(&listRect,&dataBounds,cellSize,lDef,dialogPtr,true,false,false,true);

	doAddRowsAndDataToTextList(textListHdl,rListCellStrings);

	SetPt(&theCell,0,0);
	LSetSelect(true,theCell,textListHdl);

	doResetTypeSelection();

	return(textListHdl);
}

//  doAddRowsAndDataToTextList

void  doAddRowsAndDataToTextList(ListHandle textListHdl,SInt16 stringListID)
{
	SInt16	stringIndex;
	Str255	theString;

	for(stringIndex=1;stringIndex<16;stringIndex++) 
	{
		GetIndString(theString,stringListID,stringIndex);
		doAddTextItemAlphabetically(textListHdl,theString);	
	}
}

//  doAddTextItemAlphabetically

void  doAddTextItemAlphabetically(ListHandle listHdl,Str255 theString)
{
	Boolean	found;
	SInt16	totalRows, currentRow, cellDataOffset, cellDataLength;
	Cell		aCell;

	found = false;

	totalRows = (*listHdl)->dataBounds.bottom - (*listHdl)->dataBounds.top;
	currentRow = -1;

	while(!found)
	{
		currentRow += 1;
		if(currentRow == totalRows)
			found = true;
		else
		{
			SetPt(&aCell,0,currentRow);
			LGetCellDataLocation(&cellDataOffset,&cellDataLength,aCell,listHdl);

			MoveHHi((Handle) (*listHdl)->cells);
			HLock((Handle) (*listHdl)->cells);

			if(IUMagPString((Ptr) theString + 1,((Ptr) (*listHdl)->cells[0] + cellDataOffset),
											Length(theString),cellDataLength,NULL) == -1)
			{
				found = true;
			}

			HUnlock((Handle)(*listHdl)->cells);
		}
	}

	currentRow = LAddRow(1,currentRow,listHdl);
	SetPt(&aCell,0,currentRow);

	LSetCell((Ptr) theString + 1,(SInt16) Length(theString),aCell,listHdl);
}

//  doCreatePictList

ListHandle  doCreatePictList(DialogPtr dialogPtr,Rect listRect,SInt16 numCols,SInt16 lDef)
{
	Rect				dataBounds;
	Point				cellSize;
	ListHandle	pictListHdl;
	Cell				theCell;
		
	SetRect(&dataBounds,0,0,numCols,0);
	SetPt(&cellSize,48,48);

	listRect.right = listRect.right - kScrollBarWidth;

	pictListHdl = LNew(&listRect,&dataBounds,cellSize,lDef,dialogPtr,true,false,false,true);

	(*pictListHdl)->selFlags = lOnlyOne;

	doAddRowsAndDataToPictList(pictListHdl,rListCellPicts);

	SetPt(&theCell,0,0);
	LSetSelect(true,theCell,pictListHdl);

	return(pictListHdl);
}

//  doAddRowsAndDataToPictList

void  doAddRowsAndDataToPictList(ListHandle pictListHdl,SInt16 pictListID)
{
	SInt16		rowNumber, pictIndex;
	PicHandle	pictureHdl;
	Cell			theCell;

	rowNumber = (*pictListHdl)->dataBounds.bottom;

	for(pictIndex=pictListID;pictIndex<(pictListID + 6);pictIndex++)
	{	
		pictureHdl = GetPicture(pictIndex);

		rowNumber = LAddRow(1,rowNumber,pictListHdl);
		SetPt(&theCell,0,rowNumber);
		LSetCell(&pictureHdl,sizeof(PicHandle),theCell,pictListHdl);
	
		rowNumber += 1;
	}
}

//  doHandleArrowKey

void  doHandleArrowKey(SInt8 charCode,EventRecord *eventRecPtr,Boolean allowExtendSelect)
{
	Boolean	moveToTopBottom = false;

	if(eventRecPtr->modifiers & cmdKey)
		moveToTopBottom = true;

	if(allowExtendSelect && (eventRecPtr->modifiers & shiftKey))
		doArrowKeyExtendSelection(gCurrentListHdl,charCode,moveToTopBottom);
	else
		doArrowKeyMoveSelection(gCurrentListHdl,charCode,moveToTopBottom);
}

//  doArrowKeyMoveSelection

void  doArrowKeyMoveSelection(ListHandle listHdl,SInt8 charCode,Boolean moveToTopBottom)
{
	Cell	currentSelection, newSelection;

	if(doFindFirstSelectedCell(listHdl,&currentSelection))
	{
		if(charCode == kDownArrow)
			doFindLastSelectedCell(listHdl,&currentSelection);

		doFindNewCellLoc(listHdl,currentSelection,&newSelection,charCode,moveToTopBottom);

		doSelectOneCell(listHdl,newSelection);
		doMakeCellVisible(listHdl,newSelection);
	}
}

//  doArrowKeyExtendSelection

void  doArrowKeyExtendSelection(ListHandle listHdl,SInt8 charCode,Boolean moveToTopBottom)
{
	Cell	currentSelection, newSelection;

	if(doFindFirstSelectedCell(listHdl,&currentSelection))
	{
		if(charCode == kDownArrow)
			doFindLastSelectedCell(listHdl,&currentSelection);

		doFindNewCellLoc(listHdl,currentSelection,&newSelection,charCode,moveToTopBottom);

		if(!(LGetSelect(false,&newSelection,listHdl)))
			LSetSelect(true,newSelection,listHdl);

		doMakeCellVisible(listHdl,newSelection);
	}
}

//  doTypeSelectSearch

void  doTypeSelectSearch(ListHandle listHdl,EventRecord *eventRecPtr)
{
	SInt8	newChar;
	Cell	theCell;

	newChar = eventRecPtr->message & charCodeMask;

	if((gTSLastListHit != listHdl) || ((eventRecPtr->when - gTSLastKeyTime) >= 
		 gTSResetThreshold) || (Length(gTSString) == 255))
		doResetTypeSelection();

	gTSLastListHit = listHdl;
	gTSLastKeyTime = eventRecPtr->when;

	gTSString[0] = (SInt8) (Length(gTSString) + 1);
	gTSString[Length(gTSString)] = newChar;

	SetPt(&theCell,0,0);

	if(LSearch((Ptr) gTSString+1,Length(gTSString),doSearchPartialMatchRD,
						 &theCell,listHdl))																								 // PowerPC
	{
		LSetSelect(true,theCell,listHdl);
		doSelectOneCell(listHdl,theCell);
		doMakeCellVisible(listHdl,theCell);
	}
}

//  doSearchPartialMatch

pascal SInt16  doSearchPartialMatch(Ptr searchDataPtr,Ptr cellDataPtr,SInt16 cellDataLen,
																		SInt16 searchDataLen)
{
	SInt16	result;

	if((cellDataLen > 0) && (cellDataLen >= searchDataLen))
		result = IUMagIDString(cellDataPtr,searchDataPtr,searchDataLen,searchDataLen);
	else
		result = 1;

	return(result);
}	

//  doFindFirstSelectedCell

Boolean  doFindFirstSelectedCell(ListHandle listHdl,Cell *theCell)
{
	Boolean	result;

	SetPt(theCell,0,0);
	result = LGetSelect(true,theCell,listHdl);

	return(result);
}

//  doFindLastSelectedCell

void  doFindLastSelectedCell(ListHandle listHdl,Cell *theCell)
{
	Cell		aCell;
	Boolean	moreCellsInList;

	if(doFindFirstSelectedCell(listHdl,&aCell))
	{
		while(LGetSelect(true,&aCell,listHdl))
		{
			*theCell = aCell;
			moreCellsInList = LNextCell(true,true,&aCell,listHdl);
		} 
	}
}

//  doFindNewCellLoc

void  doFindNewCellLoc(ListHandle listHdl,Cell oldCellLoc,Cell *newCellLoc,SInt8 charCode,
											 Boolean moveToTopBottom)
{
	SInt16	listRows;

	listRows = (*listHdl)->dataBounds.bottom - (*listHdl)->dataBounds.top;
	*newCellLoc = oldCellLoc;

	if(moveToTopBottom)
	{
		if(charCode == kUpArrow)
			(*newCellLoc).v = 0;
		else if(charCode == kDownArrow)
			(*newCellLoc).v = listRows - 1;
	}
	else
	{
		if(charCode ==  kUpArrow)
		{
			if(oldCellLoc.v != 0)
				(*newCellLoc).v = oldCellLoc.v - 1;
		}	
		else if(charCode == kDownArrow)
		{
			if(oldCellLoc.v != listRows - 1)
				(*newCellLoc).v = oldCellLoc.v + 1;
		}
	}
}

//  doSelectOneCell

void  doSelectOneCell(ListHandle listHdl,Cell theCell) 
{
	Cell		nextSelectedCell;
	Boolean	moreCellsInList;

	if(doFindFirstSelectedCell(listHdl,&nextSelectedCell))
	{
		while(LGetSelect(true,&nextSelectedCell,listHdl))
		{
			if(nextSelectedCell.v != theCell.v)
				LSetSelect(false,nextSelectedCell,listHdl);
			else
				moreCellsInList = LNextCell(true,true,&nextSelectedCell,listHdl);
		} 

		LSetSelect(true,theCell,listHdl);
	}
}

//  doMakeCellVisible

void  doMakeCellVisible(ListHandle listHdl,Cell newSelection)
{
	Rect		visibleRect;
	SInt16	dRows;

	visibleRect = (*listHdl)->visible;

	if(!(PtInRect(newSelection,&visibleRect)))
	{
		if(newSelection.v > visibleRect.bottom - 1)
			dRows = newSelection.v - visibleRect.bottom + 1;
		else if(newSelection.v < visibleRect.top)
			dRows = newSelection.v - visibleRect.top;

		LScroll(0,dRows,listHdl);
	}
}

//  doResetTypeSelection

void  doResetTypeSelection(void)
{	
	gTSString[0] = 0;
	gTSLastListHit = NULL;
	gTSLastKeyTime = 0;
	gTSResetThreshold = 2 * LMGetKeyThresh();	
	if(gTSResetThreshold > kMaxKeyThresh)
		gTSResetThreshold = kMaxKeyThresh;	
}

//  doRotateCurrentList

void  doRotateCurrentList(void)
{
	WindowPtr 	windowPtr;
	ListHandle	oldListHdl, newListHdl;

	windowPtr = FrontWindow();
	if(((WindowPeek) windowPtr)->windowKind != dialogKind)
		return;	

	oldListHdl = gCurrentListHdl;
	newListHdl = (ListHandle) (*gCurrentListHdl)->refCon;
	gCurrentListHdl = newListHdl;

	doDrawActiveListBorder(oldListHdl);
	doDrawActiveListBorder(newListHdl);
}

//  doDrawListsBorders

void  doDrawListsBorders(ListHandle textListHdl,ListHandle pictListHdl)
{
	PenState	oldPenState;
	Rect			borderRect;

	GetPenState(&oldPenState);
	PenSize(1,1);

	borderRect = (*textListHdl)->rView;
	InsetRect(&borderRect,-1,-1);
	FrameRect(&borderRect);

	borderRect = (*pictListHdl)->rView;
	InsetRect(&borderRect,-1,-1);
	FrameRect(&borderRect);

	SetPenState(&oldPenState);
}

//  doDrawActiveListBorder

void  doDrawActiveListBorder(ListHandle listHdl)
{
	PenState	oldPenState;
	Rect			borderRect;

	GetPenState(&oldPenState);
	PenSize(2,2);

	borderRect = (*listHdl)->rView;
	borderRect.right += kScrollBarWidth;
	InsetRect(&borderRect,-4,-4);
	
	if(listHdl == gCurrentListHdl  && (*listHdl)->lActive)
		PenPat(&qd.black);
	else
		PenPat(&qd.white);

	FrameRect(&borderRect);		

	SetPenState(&oldPenState);
}

//  doDisplaySelections

void  doDisplaySelections(void)
{
	ListsRecHandle	listsRecHdl;
	ListHandle			textListHdl, pictListHdl;
	SInt16					nextLine = 15, cellIndex;
	Cell						theCell;
	Str255					theString;
	SInt16					offset, dataLen;

	listsRecHdl = (ListsRecHandle) GetWRefCon(FrontWindow());
	textListHdl = (*listsRecHdl)->textListHdl;
	pictListHdl = (*listsRecHdl)->pictListHdl;

	HideWindow(FrontWindow());	
	SetPort(gWindowPtr);

	MoveTo(10,nextLine);
	DrawString("\pTimber:");
	MoveTo(120,nextLine);
	DrawString("\pTool:");

	for(cellIndex=0;cellIndex < (*textListHdl)->dataBounds.bottom;cellIndex++)
	{
		SetPt(&theCell,0,cellIndex);
		if(LGetSelect(false,&theCell,textListHdl))
		{
			LGetCellDataLocation(&offset,&dataLen,theCell,textListHdl);
			LGetCell((Ptr) theString + 1,&dataLen,theCell,textListHdl);
			theString[0] = (SInt8) dataLen;

			MoveTo(10,nextLine += 15);
			DrawString(theString);
		}
	}

	SetPt(&theCell,0,0);
	LGetSelect(true,&theCell,pictListHdl);
	GetIndString(theString,rListCellPictTitles,theCell.v + 1);
	MoveTo(120,30);
	DrawString(theString);
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle	fileMenuHdl, demoMenuHdl;

	fileMenuHdl = GetMenuHandle(mFile);
	demoMenuHdl = GetMenuHandle(mDemonstration);

	if(((WindowPeek) FrontWindow())->windowKind == dialogKind)
	{
		DisableItem(fileMenuHdl,0);
		DisableItem(demoMenuHdl,0);
	}
	else
	{
		EnableItem(fileMenuHdl,0);
		EnableItem(demoMenuHdl,0);
	}

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;
			
		case mFile:
			if(menuItem == iQuit)
				gDone = true;
			break;
			
		case mDemonstration:
			if(menuItem == iDialog)
			{
				SetPort(gWindowPtr);
				EraseRect(&gWindowPtr->portRect);
				doCreateDialogWithLists();
			}
			break;
	}

	HiliteMenu(0);
}

//  doDrawDialogDefaultButton

void  doDrawDialogDefaultButton(DialogPtr dialogPtr)
{
	WindowPtr	oldPort;
	PenState	oldPenState;
	SInt16		itemType;
	Handle		itemHandle;
	Rect			itemRect;
	SInt16		buttonOval;

	GetPort(&oldPort);
	GetPenState(&oldPenState);

	GetDialogItem(dialogPtr,iOK,&itemType,&itemHandle,&itemRect);
	SetPort((*(ControlHandle) itemHandle)->contrlOwner);
	InsetRect(&itemRect,-4,-4);
	buttonOval = (itemRect.bottom - itemRect.top) / 2 + 2;

	if((*(ControlHandle) itemHandle)->contrlHilite == 255)
		PenPat(&qd.gray);
	else
		PenPat(&qd.black);

	PenSize(3,3);
	FrameRoundRect(&itemRect,buttonOval,buttonOval);

	SetPenState(&oldPenState);
	SetPort(oldPort);
}

// 