// 
// CDEF2PPC.c												Custom control definition function for slider control
// 
//
// This CDEF displays:
//
// 	A 3D coloured slider control on colour displays set to pixel depths greater than 1.
//
// 	A black-and-white slider control on colour displays set to pixel depths less than
//		2.
//
// 	A black-and-white slider control if Color QuickDraw is not present.
//
// The CDEF utilises two 'PICT' resources (purgeable).  One resource contains the colour
// version of the slider control components.  The other comprises the black and white 
// version.  The appearance of the coloured slider conforms to the specification for
// slider controls contained in the document Apple Greyscale Appearance for System 7.5
// published by Apple Computer Inc.  
//
//  

//  includes

#include <ToolUtils.h>
#include <QDOffscreen.h>
#include <Resources.h>
#include <Gestalt.h>
#include <Fonts.h>
#include <Retrace.h>
#include <Traps.h>
#include <Devices.h>
#include <A4Stuff.h>
#include <LowMem.h>

//  defines

#define kInactive					255
#define kIndicatorHeight	16
#define rTrackPict				128

//  typedefs

typedef struct
{
	VBLTask		vblTaskRec;
	Boolean		inVBlankPeriod;
	SInt32		thisApplicationsA5;
} VBLRec, *VBLRecPtr;

typedef struct
{
	GWorldPtr	offScreenPort;
	Rect			offScreenPortRect;
	Rect			trackActiveRect;
	Rect			trackInactiveRect;
	Rect			indicatorActiveRect;
	Rect			indicatorPressedRect;
	Rect			indicatorInactiveRect;
	Rect			compositeRect;
	GWorldPtr	currentPort;
	GDHandle	currentDevice;
} SliderDataRec, *SliderDataPtr, **SliderDataHdl;

//  global variables

Boolean		gColorQuickDrawPresent = true;
SInt16		gMainSlotNumber;
Boolean		gSlotVInstallPresent;
Boolean		gDragMessageFlag			 = false;
Boolean		gVBLInstallFail				 = true;
VBLRec		gVBLRec;

VBLUPP		theVBLTaskRD;																												 // PowerPC

//  function prototypes

pascal SInt32		main(SInt16 varCode,ControlHandle theControl,SInt16 message,SInt32 param);

void			doInitMessage								(ControlHandle);
void			doDrawMessage								(ControlHandle);
SInt32		doTestMessage								(ControlHandle,SInt32);
SInt32		doDragMessage								(ControlHandle);
void			doDisposeMessage						(ControlHandle);
void			createOffScreenGWorld				(ControlHandle);
void			pixelDepthCheck							(ControlHandle);
void			drawControlActive						(ControlHandle);
void			drawControlInactive					(ControlHandle);
Rect			calcIndicatorRect						(ControlHandle);
OSErr			installVBLTask							(void);
void			removeVBLTask								(void);
void			theVBLTask									(void);
Boolean		checkSlotVInstallAvailable	(void);
Boolean		checkTrapAvailable					(SInt16);

//  in-line glue for GetVBLRec

#if GENERATING68K																															 // PowerPC
pascal SInt32  GetVBLRec (void) = 0x2E88;
#endif																																				 // PowerPC

//  main

pascal SInt32  main(SInt16 varCode,ControlHandle theControl,SInt16 message,SInt32 param)
{
	PenState	oldPenState;
	SInt32		returnValue;
	
#if GENERATING68K																															 // PowerPC
	EnterCodeResource();				 
#endif																																				 // PowerPC

	GetPenState(&oldPenState);

	switch(message)
	{
		case initCntl:
			doInitMessage(theControl);
			returnValue = 0;
			break;

		case drawCntl:
			if((*theControl)->contrlVis)
				doDrawMessage(theControl);
			returnValue = 0;
			break;

		case testCntl:
			returnValue = doTestMessage(theControl,param);
			break;

		case dragCntl:
			returnValue = doDragMessage(theControl);
			break;

		case dispCntl:
			doDisposeMessage(theControl);
			returnValue = 0;
			break;

		default:
			returnValue = 0;
	}

	SetPenState(&oldPenState);

#if GENERATING68K																															 // PowerPC
	ExitCodeResource();				 
#endif																																				 // PowerPC

	return returnValue;
}

//  doInitMessage

void  doInitMessage(ControlHandle theControl)
{
	OSErr				osErr;
	SInt32			response;
	GDHandle  	mainDeviceHdl;
	SInt16			mainDeviceRefNum;
	DCtlHandle	deviceCtlEntryHdl;

	theVBLTaskRD = NewVBLProc((ProcPtr) theVBLTask);														 // PowerPC

	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response < gestalt8BitQD)
		gColorQuickDrawPresent = false;

	HLock((Handle) theControl);

	(*theControl)->contrlData = NewHandle(sizeof(SliderDataRec));
	if((*theControl)->contrlData != NULL)
		createOffScreenGWorld(theControl);

	HUnlock((Handle) theControl);

	gSlotVInstallPresent = checkSlotVInstallAvailable();
	if(gSlotVInstallPresent)
	{
		mainDeviceHdl 		= LMGetMainDevice();
		mainDeviceRefNum 	= (*mainDeviceHdl)->gdRefNum;
		deviceCtlEntryHdl = GetDCtlEntry(mainDeviceRefNum);
		gMainSlotNumber 	= (SInt16) (*((AuxDCEHandle) deviceCtlEntryHdl))->dCtlSlot;
	}
}

//  doDrawMessage

void  doDrawMessage(ControlHandle theControl)
{
	if(gColorQuickDrawPresent)
		pixelDepthCheck(theControl);

	if((*theControl)->contrlHilite == kInactive)
		drawControlInactive(theControl);
	else
		drawControlActive(theControl);
}

//  doTestMessage

SInt32  doTestMessage(ControlHandle theControl,SInt32 param)
{
	Rect	indicatorRect;
	Point	mouseXY;

	indicatorRect = calcIndicatorRect(theControl);

	mouseXY.v = HiWord(param);
	mouseXY.h = LoWord(param);
	
	if(PtInRect(mouseXY,&indicatorRect))
	{
		gDragMessageFlag = true;
		drawControlActive(theControl);
		gDragMessageFlag = false;
		return kControlIndicatorPart;
	}
	else 
		return 0;
}

//  doDragMessage

SInt32  doDragMessage(ControlHandle theControl)
{
	Rect			indicatorRect, slopRect, trackRect;
	SInt16		indicatorHeight, indicatorHalfHeight, indicatorCentre, trackHeight;
	Point			startMouseXY, currentMouseXY;
	SInt16		controlValueRange, differenceMouseY;
	float			ratio;
	WindowPtr	windowPtr;
	OSErr			osErr;

	gDragMessageFlag = true;

	HLock((Handle) theControl);

	indicatorHeight = kIndicatorHeight;
	indicatorHalfHeight = indicatorHeight / 2;

	trackRect = (*theControl)->contrlRect;
	InsetRect(&trackRect,0,indicatorHalfHeight + 4);
	trackRect.bottom += 1;
	trackHeight = trackRect.bottom - trackRect.top;

	controlValueRange = (*theControl)->contrlMax - (*theControl)->contrlMin;
	ratio = (float) ((float) controlValueRange) / ((float) trackHeight);

	windowPtr = (*theControl)->contrlOwner;
	slopRect = windowPtr->portRect;

	osErr = installVBLTask();
	if(osErr == noErr)
		gVBLInstallFail = false;
	else
		gVBLInstallFail = true;
	
	indicatorRect = calcIndicatorRect(theControl);

	GetMouse(&startMouseXY);

	while(StillDown())
	{
		GetMouse(&currentMouseXY);
		differenceMouseY = startMouseXY.v - currentMouseXY.v;

		if(differenceMouseY != 0 && PtInRect(currentMouseXY,&slopRect))
		{
			indicatorRect.top -= differenceMouseY;
			indicatorRect.bottom -= differenceMouseY;

			indicatorCentre = indicatorRect.top + indicatorHalfHeight;

			(*theControl)->contrlValue = (trackRect.bottom - indicatorCentre) * ratio;

			if((*theControl)->contrlValue > (*theControl)->contrlMax)
				(*theControl)->contrlValue = (*theControl)->contrlMax;
			if((*theControl)->contrlValue < (*theControl)->contrlMin)
				(*theControl)->contrlValue = (*theControl)->contrlMin;

			drawControlActive(theControl);

			startMouseXY = currentMouseXY;
		}
	}
	
	if(!gVBLInstallFail)
		removeVBLTask();

	gDragMessageFlag = false;
	drawControlActive(theControl);

	HUnlock((Handle) theControl);

	return 1;
}

//  doDisposeMessage

void  doDisposeMessage(ControlHandle theControl)
{
	Rect	theRect;

	DisposeRoutineDescriptor(theVBLTaskRD);																			 // PowerPC

	theRect = (*theControl)->contrlRect;
	theRect.right = theRect.right + (theRect.right - theRect.left);
	EraseRect(&theRect);

	if((**((SliderDataHdl) (*theControl)->contrlData)).offScreenPort != NULL)
		DisposeGWorld((**((SliderDataHdl) (*theControl)->contrlData)).offScreenPort);

	if((*theControl)->contrlData != NULL)
		DisposeHandle((*theControl)->contrlData);
}

//  createOffScreenGWorld

void  createOffScreenGWorld(ControlHandle theControl)
{
	SliderDataHdl	sliderDataHdl;
	SInt16				resourceOffset = 0;
	PixMapHandle	pixMapHdl;	
	PicHandle			pictureHdl;
	SInt16 				currentPortDepth = 1;

	sliderDataHdl = (SliderDataHdl) (*theControl)->contrlData;	

	(*sliderDataHdl)->compositeRect = (*theControl)->contrlRect;
	OffsetRect(&(*sliderDataHdl)->compositeRect,-(*sliderDataHdl)->compositeRect.left,
						 -(*sliderDataHdl)->compositeRect.top);
	SetRect(&(*sliderDataHdl)->trackActiveRect,50,0,100,139);
	SetRect(&(*sliderDataHdl)->trackInactiveRect,100,0,150,139);
	SetRect(&(*sliderDataHdl)->indicatorActiveRect,0,139,16,154);
	SetRect(&(*sliderDataHdl)->indicatorPressedRect,16,139,32,154);
	SetRect(&(*sliderDataHdl)->indicatorInactiveRect,32,139,48,154);
	SetRect(&(*sliderDataHdl)->offScreenPortRect,0,0,150,154);
	
	GetGWorld(&(*sliderDataHdl)->currentPort,&(*sliderDataHdl)->currentDevice);

	NewGWorld(&(*sliderDataHdl)->offScreenPort,0,&(*sliderDataHdl)->offScreenPortRect,
						NULL,NULL,0);

	HLock((Handle) sliderDataHdl);

	pixMapHdl = GetGWorldPixMap((*sliderDataHdl)->offScreenPort);
	LockPixels(pixMapHdl);

	SetGWorld((*sliderDataHdl)->offScreenPort,nil);

	EraseRect(&(*sliderDataHdl)->offScreenPortRect);
	
	if(gColorQuickDrawPresent)
	{
		pixMapHdl = GetGWorldPixMap((*sliderDataHdl)->currentPort);
		currentPortDepth = (*pixMapHdl)->pixelSize;
	}

	if(!gColorQuickDrawPresent || currentPortDepth < 2)
		resourceOffset = 1;

	if(pictureHdl = GetPicture(rTrackPict + resourceOffset))
	{
		HNoPurge((Handle) pictureHdl);
		DrawPicture(pictureHdl,&(*sliderDataHdl)->offScreenPortRect);	
		HPurge((Handle) pictureHdl);
	}
	
	SetGWorld((*sliderDataHdl)->currentPort,(*sliderDataHdl)->currentDevice);
	UnlockPixels(pixMapHdl);
	HUnlock((Handle) sliderDataHdl);
}

//  pixelDepthCheck

void  pixelDepthCheck(ControlHandle theControl)
{
	SliderDataHdl	sliderDataHdl;
	PixMapHandle	pixMapHdl;
	SInt16				currentPortDepth, gworldPortDepth;
	
	sliderDataHdl = (SliderDataHdl) (*theControl)->contrlData;	

	pixMapHdl = GetGWorldPixMap((*sliderDataHdl)->currentPort);
	currentPortDepth = (*pixMapHdl)->pixelSize;
	pixMapHdl = GetGWorldPixMap((*sliderDataHdl)->offScreenPort);
	gworldPortDepth = (*pixMapHdl)->pixelSize;

	if(currentPortDepth != gworldPortDepth)
	{
		DisposeGWorld((*sliderDataHdl)->offScreenPort);
		createOffScreenGWorld(theControl);
	}
}

//  drawControlActive

void  drawControlActive(ControlHandle theControl)
{
	RGBColor			oldForeColour, oldBackColour;
	SliderDataHdl	sliderDataHdl;
	WindowPtr			windowPtr;	
	PixMapHandle	pixMapHdl;
	Rect					indicatorRect;

	GetForeColor(&oldForeColour);
	GetBackColor(&oldBackColour);

	HLock((Handle) theControl);

	sliderDataHdl = (SliderDataHdl) (*theControl)->contrlData;
	HLock((Handle) sliderDataHdl);

	windowPtr = (WindowPtr) (*theControl)->contrlOwner;
	SetPort(windowPtr);

	pixMapHdl = GetGWorldPixMap((*sliderDataHdl)->offScreenPort);
	LockPixels(pixMapHdl);

	ForeColor(blackColor);
	BackColor(whiteColor);
	 
	CopyBits(&((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
					 &((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
					 &(*sliderDataHdl)->trackActiveRect,&(*sliderDataHdl)->compositeRect,
					 srcCopy,NULL);

	indicatorRect = calcIndicatorRect(theControl);
	OffsetRect(&indicatorRect,-(*theControl)->contrlRect.left,
						 -(*theControl)->contrlRect.top);

	if(gDragMessageFlag)
	{
		CopyBits(&((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
						 &((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
						 &(*sliderDataHdl)->indicatorPressedRect,&indicatorRect,srcCopy,NULL);
	}
	else
	{
		CopyBits(&((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
						 &((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
						 &(*sliderDataHdl)->indicatorActiveRect,&indicatorRect,srcCopy,NULL);
	}

	if(gDragMessageFlag && !gVBLInstallFail) 
	{  
		if(gVBLRec.inVBlankPeriod)
		{
			gVBLRec.inVBlankPeriod = false;

			CopyBits(&((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
							 &((GrafPtr) windowPtr)->portBits,&(*sliderDataHdl)->compositeRect,
							 &(*theControl)->contrlRect,srcCopy,NULL);
		}
	}
	else
	{
		CopyBits(&((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
						 &((GrafPtr) windowPtr)->portBits,&(*sliderDataHdl)->compositeRect,
						 &(*theControl)->contrlRect,srcCopy,NULL);
	}

	UnlockPixels(pixMapHdl);
	HUnlock((Handle) sliderDataHdl);
	HUnlock((Handle) theControl);

	RGBForeColor(&oldForeColour);
	RGBBackColor(&oldBackColour);
}

//  drawControlInactive

void drawControlInactive(ControlHandle theControl)
{
	RGBColor			oldForeColour, oldBackColour;
	SliderDataHdl	sliderDataHdl;
	WindowPtr			windowPtr;	
	PixMapHandle	pixMapHdl;
	Rect					indicatorRect;
	
	GetForeColor(&oldForeColour);
	GetBackColor(&oldBackColour);

	HLock((Handle) theControl);

	sliderDataHdl = (SliderDataHdl) (*theControl)->contrlData;
	HLock((Handle) sliderDataHdl);

	windowPtr = (WindowPtr) (*theControl)->contrlOwner;
	SetPort(windowPtr);

	pixMapHdl = GetGWorldPixMap((*sliderDataHdl)->offScreenPort);
	LockPixels(pixMapHdl);

	ForeColor(blackColor);
	BackColor(whiteColor);

	CopyBits(&((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
					 &((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
					 &(*sliderDataHdl)->trackInactiveRect,&(*sliderDataHdl)->compositeRect,
					 srcCopy,nil);

	indicatorRect = calcIndicatorRect(theControl);
	OffsetRect(&indicatorRect,-(*theControl)->contrlRect.left,
						 -(*theControl)->contrlRect.top);

	CopyBits(&((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
					 &((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
					 &(*sliderDataHdl)->indicatorInactiveRect,&indicatorRect,srcCopy,nil);
			
	CopyBits(&((GrafPtr) ((*sliderDataHdl)->offScreenPort))->portBits,
					 &((GrafPtr) windowPtr)->portBits,&(*sliderDataHdl)->compositeRect,
					 &(*theControl)->contrlRect,srcCopy,nil);

	UnlockPixels(pixMapHdl);
	HUnlock((Handle) sliderDataHdl);
	HUnlock((Handle) theControl);

	RGBForeColor(&oldForeColour);
	RGBBackColor(&oldBackColour);
}

//  calcIndicatorRect

Rect  calcIndicatorRect(ControlHandle theControl)
{
	SInt16	indicatorHeight, indicatorHalfHeight;
	Rect		trackRect, indicatorRect;
	SInt16	trackHeight, controlValue, controlMax, controlMin, indicatorCentre;
	float		ratio;

	indicatorHeight = kIndicatorHeight;
	indicatorHalfHeight = indicatorHeight / 2;

	trackRect = (*theControl)->contrlRect;
	InsetRect(&trackRect,0,indicatorHalfHeight + 4);
	trackRect.bottom += 1;
	trackHeight = trackRect.bottom - trackRect.top;

	controlValue = (*theControl)->contrlValue;
	controlMax = (*theControl)->contrlMax;
	controlMin = (*theControl)->contrlMin;

	ratio = ((float) controlValue) / ((float) (controlMax - controlMin));
	indicatorCentre = trackRect.bottom - (SInt16) (ratio * trackHeight) ;

	SetRect(&indicatorRect,trackRect.left,indicatorCentre - indicatorHalfHeight,
					trackRect.left + 16,indicatorCentre + indicatorHalfHeight - 1);

	return indicatorRect;
}

//  installVBLTask

OSErr installVBLTask(void)
{
	OSErr	osErr;

	gVBLRec.inVBlankPeriod = false;

	gVBLRec.vblTaskRec.qType		= vType;
	gVBLRec.vblTaskRec.vblAddr	= theVBLTaskRD;																	 // PowerPC
	gVBLRec.vblTaskRec.vblCount	= 1;
	gVBLRec.vblTaskRec.vblPhase	= 0;

#if GENERATING68K																															 // PowerPC
	gVBLRec.thisApplicationsA5 = SetCurrentA5();
#endif																																				 // PowerPC

	if(gSlotVInstallPresent)
		osErr = SlotVInstall((QElemPtr) &gVBLRec.vblTaskRec,gMainSlotNumber);
	else
		osErr = VInstall((QElemPtr) &gVBLRec.vblTaskRec);

	return osErr;
}

//  removeVBLTask

void  removeVBLTask(void)
{
	if(gSlotVInstallPresent)
		SlotVRemove((QElemPtr) &gVBLRec.vblTaskRec,gMainSlotNumber);
	else
		VRemove((QElemPtr) &gVBLRec.vblTaskRec);
}

//  theVBLTask

void theVBLTask(void)
{
#if GENERATING68K																															 // PowerPC
	VBLRecPtr	vblRecPtr;
	SInt32 			currentA5;

	vblRecPtr = (VBLRecPtr) GetVBLRec();
	currentA5 = SetA5(vblRecPtr->thisApplicationsA5);
#endif																																				 // PowerPC

#if GENERATING68K																															 // PowerPC
	vblRecPtr->inVBlankPeriod = true;
	vblRecPtr->vblTaskRec.vblCount = 1;
#else																																					 // PowerPC
	gVBLRec.inVBlankPeriod = true;																							 // PowerPC
	gVBLRec.vblTaskRec.vblCount = 1;																						 // PowerPC
#endif																																				 // PowerPC

#if GENERATING68K																															 // PowerPC
	(void) SetA5(currentA5);
#endif																																				 // PowerPC
}

//  checkSlotVInstallAvailable

Boolean  checkSlotVInstallAvailable(void)
{
	return checkTrapAvailable(_SlotVInstall);
}

//  checkTrapAvailable

Boolean  checkTrapAvailable(SInt16 theTrap)
{
	TrapType	trapType;
	SInt16		trapMask = 0x0800;
	SInt16		numToolboxTraps;

	if((theTrap & trapMask) > 0)
		trapType = ToolTrap;
	else
		trapType = OSTrap;

	if(trapType == ToolTrap)
		theTrap = theTrap & 0x07FF;

	if(NGetTrapAddress(_InitGraf,ToolTrap) == NGetTrapAddress(0xAA6E,ToolTrap))
		numToolboxTraps = 0x0200;
	else
		numToolboxTraps = 0x0400;

	if(theTrap >= numToolboxTraps)
		theTrap = _Unimplemented;

	return(NGetTrapAddress(theTrap,trapType) != NGetTrapAddress(_Unimplemented,ToolTrap));
}

// 