// 
// SoundPPC.c
// 
//
// This program opens a modal dialog containing eight button controls arranged in two
// groups, namely, a synchronous sound group and an asynchronous sound group.  Clicking
// on the buttons causes sound to be played back or recorded as follows:
//
// 	Synchronous group:
//
//		  Play sound resource.
//
//		  Play sound file.
//
//		  Record sound resource.	 	 
//
//		  Record sound file.
//
//		  Speak text string.
//
// 	Asynchronous group:
//
//		  Start and stop looped sound playback.
//
//		  Play unlooped sound.
//
//		  Speak text string.
//
// At startup, the program checks for play-from-disk, sound recording capability, speech
// capability, and multi-channel capability. If these are not available, the relevant 
// buttons are disabled. 
//
// The asynchronous sound sections of the program utilise a special library called
// AsyncSoundLib, which must be included in the CodeWarrior project.
//
// The program utilises the following resources:
//
// 	A 'DLOG' resource and associated 'DITL' and 'dctb' resources (all purgeable).
//
// 	Three 'snd ' resources, one for synchronous playback (purgeable), one for looped 
//		asynchronous playback (unpurgeable), and one for unlooped asynchronous playback
//		(purgeable).
//
// 	Two 'cicn' resources (purgeable) used to provide an animated display which halts
//		during synchronous playback and continues during asynchronous playback.
//
// 	Three 'STR#' resources containing error message strings and "speak text" strings
//		(all purgeable). 
//
// 	Two 'ALRT' resources (purgeable) for displaying error messages.
//
// In addition, the function doPlayFile utilises the file "soundfile.aiff".
//
// Each time is is invoked, the function doRecordResource creates a new 'snd' resource 
// with a unique ID in the application's resource fork.
//
// When first invoked, the function doRecordFile creates the file "test.aiff" in the
// chap21cw_demo folder.  All subsequent record-to-file is to this file.  
//
// 

//  includes

#include <SoundInput.h>
#include <Speech.h>
#include <Gestalt.h>
#include <Fonts.h>
#include <Resources.h>
#include <ToolUtils.h>
#include <SegLoad.h>
#include <Files.h>

//  defines

#define rDialog									128
#define  iQuit									1
#define  iPlayResource					2
#define  iPlayFile							3
#define  iRecordResource				4
#define  iRecordFile						5
#define  iSpeakTextSync					6
#define  iLoopedSound						7
#define  iUnloopedSound					8	
#define  iSpeakTextAsync				9
#define  iSynchSoundRect				10
#define  iAsynchSoundRect				11
#define rPlaySoundResource			8192
#define rLoopedSound						8193
#define rUnloopedSound					8194
#define	rSpeechStrings					130
#define rErrorAlert							129
#define rErrorStrings						128
#define  eOpenDialogFail				1
#define  eLoopedSoundSetUp			2
#define  eCannotInitialise			3
#define  eGetResource						4
#define  eNoChannelsAvailable		5
#define  ePlaySound							6
#define  eMemory								7
#define rErrorAlertWithCode			130
#define rErrorStringsWithCode		129
#define  eSndPlay								1		
#define  ePlayFile							2
#define  eSndRecord							3
#define  eWriteResource					4
#define  eRecordFile						5
#define  eSpeakString						6
#define  eSndDoImmediate				7
#define rColourIcon1						128
#define rColourIcon2						129
#define kMaxChannels						8
#define kOutOfChannels					1

//  global variables

Boolean				gDone;
DialogPtr			gDialogPtr;
SInt16				gAppResFileRefNum;
Boolean				gColorQuickDrawPresent = false;
Boolean				gHasSoundPlayDoubBuff;
Boolean				gHasSoundInputDevice;
Boolean				gHasSpeechmanager;
Boolean				gHasMultiChannel;
Boolean				gLoopedSoundOn = false;
SInt32				gLoopedSoundRefNum;
SndChannelPtr	gLoopedSoundChannel;
CIconHandle		gColourIconHdl1;
CIconHandle		gColourIconHdl2;

UserItemUPP		drawDialogRD;																										 // PowerPC

//  AsyncSoundLib attention flag

Boolean				gCallASLcloseChannel = false;

//  function prototypes

void				main									(void);
void				doInitManagers				(void);
void				doCheckSoundEnv				(void);
void				doInitialiseASL				(void);
Boolean			doLoopedSoundSetUp		(void);
void				eventLoop							(void);
void				doDialogHit						(SInt16);
void  			doPlayResource				(void);
void				doPlayFile						(void);
void				doRecordResource			(void);
void				doRecordFile					(void);
void				doSpeakStringSync			(void);
void				doLoopedSoundAsync		(void);
void				doUnloopedSoundAsync	(void);
void				doSpeakStringAsync		(void);
void				doSetUpDialog					(void);
pascal void	drawDialog						(DialogPtr,SInt16);
void				doAdjustItems					(void);
void				doErrorAlert					(SInt16);
void				doErrorAlertWithCode	(SInt16,SInt16);

//  AsyncSoundLib function prototypes

OSErr				ASLinitialise					(Boolean *,SInt16);
OSErr				ASLgetChannel					(SInt32,SndChannelPtr *);
OSErr				ASLplayID							(SInt16, SInt32 *);
OSErr				ASLplayHandle					(Handle,SInt32 *);
void				ASLcloseChannel				(void);
void				ASLcloseDown					(void);

//  main

void main(void)
{
	OSErr		osErr;
	SInt32	response;

	//  check for Color QuickDraw

	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response >= gestalt8BitQD)
		gColorQuickDrawPresent = true;

	//  initialise managers

	doInitManagers();

	//  create routine descriptor

	drawDialogRD = NewUserItemProc((ProcPtr) drawDialog);												 // PowerPC

	//  save reference number of application's resource file 
	
	gAppResFileRefNum	= CurResFile();

	//  check for sound recording equipment and speech capabilities

	doCheckSoundEnv();

	//  open and set up modal dialog, get colour icons

	if(!(gDialogPtr = GetNewDialog(rDialog,NULL,(WindowPtr) -1)))
	{
		doErrorAlert(eOpenDialogFail);
		ExitToShell();
	}

	SetPort(gDialogPtr);

	doSetUpDialog();

	if(gColorQuickDrawPresent)
	{
		gColourIconHdl1 = GetCIcon(rColourIcon1);
		gColourIconHdl2 = GetCIcon(rColourIcon2);
	}
	
	//  initialize AsychSoundLib

	doInitialiseASL();

	//  set up looped sound

	if(gHasMultiChannel)
	{
		if(!doLoopedSoundSetUp()) 
		{
			doErrorAlert(eLoopedSoundSetUp);
			ASLcloseDown();
			ExitToShell();
		}
	}

	//  enter event loop

	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doCheckSoundEnv

void  doCheckSoundEnv(void)
{
	OSErr		osErr;
	SInt32	response;

	osErr = Gestalt(gestaltSoundAttr,&response);

	if(osErr == noErr)
		gHasSoundPlayDoubBuff = BitTst(&response,31 - gestaltSndPlayDoubleBuffer);
	else
		gHasSoundPlayDoubBuff = false;

	if(osErr == noErr)
		gHasSoundInputDevice = BitTst(&response,31 - gestaltHasSoundInputDevice);
	else
		gHasSoundInputDevice = false;

	if(osErr == noErr)
		gHasSpeechmanager = BitTst(&response,31 - gestaltSpeechMgrPresent);
	else
		gHasSpeechmanager = false;

	if(osErr == noErr)
		gHasMultiChannel = BitTst(&response,31 - gestaltMultiChannels);		
	else
		gHasMultiChannel = false;
}

//  doInitialiseASL

void  doInitialiseASL(void)
{
	if(ASLinitialise(&gCallASLcloseChannel,kMaxChannels) != noErr)
	{
		doErrorAlert(eCannotInitialise);
		ExitToShell();
	}
}

//  doLoopedSoundSetUp

Boolean  doLoopedSoundSetUp(void)
{
	SInt16	error;
	OSErr		osErr;
	Handle	soundHdl;

	error = ASLplayHandle(NULL,&gLoopedSoundRefNum);
	if(error)
		return(false);
	else 
	{
		error = ASLgetChannel(gLoopedSoundRefNum,&gLoopedSoundChannel);
		if(error)
			return(false);
		
		soundHdl = GetResource('snd ',rLoopedSound);
		if(soundHdl) 
		{
			HLockHi(soundHdl);
			osErr = SndPlay(gLoopedSoundChannel,(SndListHandle) soundHdl,true);
			if(osErr != noErr)
				return(false);
		}
		else
			return(false);
	}
	
	return(true);
}

//  eventLoop

void  eventLoop(void)
{
	Rect				theRect, eraseRect;
	Boolean			gotEvent;
	EventRecord	eventRec;
	DialogPtr		theDialogPtr;
	SInt16			itemHit;
	SInt32 			finalTicks;

	gDone = false;

	SetRect(&theRect,10,273,35,299);
	SetRect(&eraseRect,45,273,125,299);

	while(!gDone) 
	{		
		if(gCallASLcloseChannel)
		{
			ASLcloseChannel();

			TextFont(geneva);
			TextSize(9);
			MoveTo(45,285);
			DrawString("\pASLcloseChannel");
			MoveTo(45,295);
			DrawString("\pcalled");
		}
				
		gotEvent = WaitNextEvent(everyEvent,&eventRec,10,NULL);

		if(gotEvent)
		{
			if(IsDialogEvent(&eventRec))
				if(DialogSelect(&eventRec,&theDialogPtr,&itemHit))
					doDialogHit(itemHit);
		}
		else
		{
			if(gColorQuickDrawPresent)
			{
				PlotCIcon(&theRect,gColourIconHdl1);
				Delay(15,&finalTicks);
				PlotCIcon(&theRect,gColourIconHdl2);
				Delay(15,&finalTicks);
				EraseRect(&eraseRect);
			}
		}
	}
	
	DisposeDialog(gDialogPtr);
	
	ASLcloseDown();
}

//  doDialogHit

void  doDialogHit(SInt16 item)
{
	switch(item) 
	{
		case iQuit:
			gDone = true;
			break;

		case iPlayResource:
			doPlayResource();
			break;

		case iPlayFile:
			doPlayFile();
			break;

		case iRecordResource:
			doRecordResource();
			break;

		case iRecordFile:
			doRecordFile();
			break;

		case iSpeakTextSync:
			doSpeakStringSync();
			break;

		case iLoopedSound:
			doLoopedSoundAsync();
			break;

		case iUnloopedSound:
			doUnloopedSoundAsync();
			break;
			
		case iSpeakTextAsync:
			doSpeakStringAsync();
			break;
	}			
}

//  doPlayResource

void  doPlayResource(void)
{
	SndListHandle	sndListHdl;
	SInt16				resErr;
	OSErr					osErr;

	sndListHdl = (SndListHandle) GetResource('snd ',rPlaySoundResource);
	resErr = ResError();
	if(resErr != noErr)
		doErrorAlert(eGetResource);		
	
	if(sndListHdl != NULL)
	{
		HLock((Handle) sndListHdl);
		osErr = SndPlay(NULL,sndListHdl,false);
		if(osErr != noErr)
			doErrorAlertWithCode(eSndPlay,osErr);			
		HUnlock((Handle) sndListHdl);
		ReleaseResource((Handle) sndListHdl);
	}
}

//  doPlayFile 

void  doPlayFile(void)
{
	OSErr		osErr;
	FSSpec	fileSysSpec;
	SInt16	fileRefNum;
			
	osErr = FSMakeFSSpec(0,0,"\p:soundfile.aiff",&fileSysSpec);
	if(osErr == noErr)
		osErr = FSpOpenDF(&fileSysSpec,fsRdPerm,&fileRefNum);
	if(osErr == noErr)
		SetFPos(fileRefNum,fsFromStart,0);
	if(osErr == noErr)
		osErr = SndStartFilePlay(NULL,fileRefNum,0,20480,NULL,NULL,NULL,false);
	if(osErr != noErr)
		doErrorAlertWithCode(ePlayFile,osErr);			

	FSClose(fileRefNum);
}	

//  doRecordResource

void  doRecordResource(void)
{
	SInt16	oldResFileRefNum;
	Point		topLeft;
	Handle	soundHdl;
	OSErr		osErr, memErr;
	SInt16	theResourceID, resErr;
			
	oldResFileRefNum = CurResFile();
	UseResFile(gAppResFileRefNum);

	topLeft.v = 40;
	topLeft.h = 250;
	
	soundHdl = NewHandle(25000);
	memErr = MemError();
	if(memErr != noErr)
	{
		doErrorAlert(eMemory);
		return;
	}
		
	osErr = SndRecord(NULL,topLeft,siBetterQuality,&(SndListHandle) soundHdl);
	if(osErr != noErr && osErr != userCanceledErr)
		doErrorAlertWithCode(eSndRecord,osErr);
	else
	{
		do
		{
			theResourceID = UniqueID('snd ');
		} while(theResourceID <= 8191 && theResourceID >= 0);

		AddResource((Handle) soundHdl,'snd ',theResourceID,"\pTest");
		resErr = ResError();
		if(resErr == noErr)
			UpdateResFile(gAppResFileRefNum);
		resErr = ResError();
		if(resErr != noErr)
			doErrorAlertWithCode(eWriteResource,resErr);
	}

	UseResFile(oldResFileRefNum);
}

//  doRecordFile

void  doRecordFile(void)
{
	Point		topLeft;
	OSErr		osErr;
	FSSpec	fileSysSpec;
	SInt16	fileRefNum;
			
	topLeft.v = 40;
	topLeft.h = 250;
	
	osErr = FSMakeFSSpec(0,0,"\p:test.aiff",&fileSysSpec);
	if(osErr == fnfErr)
		osErr = FSpCreate(&fileSysSpec,'????','AIFF',smSystemScript);
	if(osErr == noErr)
		osErr = FSpOpenDF(&fileSysSpec,fsWrPerm,&fileRefNum);
	if(osErr == noErr)
		SetFPos(fileRefNum,fsFromStart,0);
	if(osErr == noErr)
		osErr = SndRecordToFile(NULL,topLeft,siBetterQuality,fileRefNum);
	if(osErr != noErr && osErr != userCanceledErr)
		doErrorAlertWithCode(eRecordFile,osErr);			

	FSClose(fileRefNum);
}

//  doSpeakStringSync

void  doSpeakStringSync(void)
{
	SInt16	activeChannels;
	Str255	theString;
	OSErr		resErr, osErr;
		
	activeChannels = SpeechBusy();

	GetIndString(theString,rSpeechStrings,1);
	resErr = ResError();
	if(resErr != noErr)
	{
		doErrorAlert(eGetResource);
		return;
	}
	
	osErr = SpeakString(theString);
	if(osErr != noErr)
		doErrorAlertWithCode(eSpeakString,osErr);
		
	while(SpeechBusy() != activeChannels)
		;
}

//  doLoopedSoundAsync

void  doLoopedSoundAsync(void)
{
	SndCommand	sndCommand;
	OSErr				osErr;
			
	gLoopedSoundOn = !gLoopedSoundOn;

	doAdjustItems();

	sndCommand.param1 = 0;
	
	if(gLoopedSoundOn) 
	{
		sndCommand.cmd		= freqCmd;
		sndCommand.param2	= 0x3C;
	} 
	else 
	{
		sndCommand.cmd		= quietCmd;
		sndCommand.param2	= 0;
	}
	
	osErr = SndDoImmediate(gLoopedSoundChannel,&sndCommand);
	if(osErr != noErr)
		doErrorAlertWithCode(eSndDoImmediate,osErr);
}

//  doUnloopedSoundAsync

void  doUnloopedSoundAsync(void)
{
	SInt16	error;
	
	error = ASLplayID(rUnloopedSound,NULL);
	if(error == kOutOfChannels)
		doErrorAlert(eNoChannelsAvailable);
	else
		if(error != noErr)
			doErrorAlert(ePlaySound);
}

//  doSpeakStringAsync

void  doSpeakStringAsync(void)
{
	Str255	theString;
	OSErr		resErr, osErr;

	GetIndString(theString,rSpeechStrings,2);
	resErr = ResError();
	if(resErr != noErr)
	{
		doErrorAlert(eGetResource);
		return;
	}
	
	osErr = SpeakString(theString);
	if(osErr != noErr)
		doErrorAlertWithCode(eSpeakString,osErr);
}

//  doSetUpDialog

void  doSetUpDialog(void)
{
	SInt16	itemType;
	Handle	itemHdl;
	Rect		itemRect;
			
	GetDialogItem(gDialogPtr,iSynchSoundRect,&itemType,&itemHdl,&itemRect);
	SetDialogItem(gDialogPtr,iSynchSoundRect,itemType,
								(Handle) drawDialogRD,&itemRect);															 // PowerPC

	if(!gHasSoundPlayDoubBuff)
	{
		GetDialogItem(gDialogPtr,iPlayFile,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
	}

	if(!gHasSoundInputDevice)
	{
		GetDialogItem(gDialogPtr,iRecordResource,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
		GetDialogItem(gDialogPtr,iRecordFile,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
	}
	
	if(!gHasSpeechmanager)
	{
		GetDialogItem(gDialogPtr,iSpeakTextSync,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
		GetDialogItem(gDialogPtr,iSpeakTextAsync,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
	}

	if(!gHasMultiChannel)
	{
		GetDialogItem(gDialogPtr,iLoopedSound,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
	}	
}

//  drawDialog

pascal void  drawDialog(DialogPtr dialogPtr,SInt16 theItem)
{
	SInt16		itemType;
	Handle		itemHdl;
	Rect			itemRect;
	SInt16		buttonOval;

	GetDialogItem(dialogPtr,iSynchSoundRect,&itemType,&itemHdl,&itemRect);
	FrameRect(&itemRect);
	GetDialogItem(dialogPtr,iAsynchSoundRect,&itemType,&itemHdl,&itemRect);
	FrameRect(&itemRect);
	GetDialogItem(dialogPtr,iQuit,&itemType,&itemHdl,&itemRect);
	InsetRect(&itemRect,-4,-4);
	PenSize(3,3);
	buttonOval = (itemRect.bottom - itemRect.top) / 2 + 2;
	FrameRoundRect(&itemRect,buttonOval,buttonOval);
}

//  doAdjustItems

void  doAdjustItems(void)
{
	SInt16	itemType, a;
	Handle	itemHdl;
	Rect		itemRect;

	GetDialogItem(gDialogPtr,iLoopedSound,&itemType,&itemHdl,&itemRect);
	if(gLoopedSoundOn)
		SetControlTitle((ControlHandle) itemHdl,"\pSwitch Looped Sound Off");
	else
		SetControlTitle((ControlHandle) itemHdl,"\pSwitch Looped Sound On");

	for(a=iRecordResource;a<iRecordFile+1;a++)
	{
		GetDialogItem(gDialogPtr,a,&itemType,&itemHdl,&itemRect);
		if(gLoopedSoundOn)
			HiliteControl((ControlHandle) itemHdl,255);
		else
			HiliteControl((ControlHandle) itemHdl,0);
	}		
}

//  doErrorAlert

void doErrorAlert(SInt16 stringIndex)
{
	Str255	errorString;
	
	GetIndString(errorString,rErrorStrings,stringIndex);
	ParamText(errorString,NULL,NULL,NULL);
	StopAlert(rErrorAlert,NULL);
}

//  doErrorAlertWithCode

void doErrorAlertWithCode(SInt16 stringIndex,SInt16 resultCode)
{
	Str255	errorString, resultCodeString;
	
	GetIndString(errorString,rErrorStringsWithCode,stringIndex);
	NumToString((SInt32) resultCode,resultCodeString);

	ParamText(errorString,resultCodeString,NULL,NULL);
	StopAlert(rErrorAlertWithCode,NULL);
}

// 