{ 
// MoreResourcesPascal.p
// 
// 
// This program uses custom resources to:
//
//    Store application preferences in the resource fork of a preferences file, and also
//	   to assist in the initial creation of the preferences file.
//
//    Store, in the resource fork of a document file, the user state and current state of
//	   the window associated with the document.
//
//    Store, in the resource fork of a document file, the width and height of the
//	   printable area of the paper size chosen in the print Style dialog box. 
//
// The program utilizes the following standard resources:
//
//    An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Demonstration
//	   menus (preload, non-purgeable).  
//
//    A 'WIND' resource (purgeable) (initially invisible).  
//
//    An 'ALRT' resource (purgeable) and associated 'DITL' resource (purgeable) 
//	   associated with the display of error messages.  
//
//    A 'DLOG' resource (purgeable) and associated 'DITL' resource (purgeable) associated
//	   with the display of, and user modification of, current application preferences. 
//
//    A 'STR#' resource (purgeable) containing the required name of the preferences file
//	   created by the program.
//
//    A 'STR ' resource (purgeable) containing the application-missing string, which is
//	   copied to the resource fork of the preferences file.
//
//    A 'SIZE' resource with the acceptSuspendResumeEvents & is32BitCompatible flags set.
//
// The program utilizes the following custom resources:
//
//    A 'PrFn' (preferences) resource comprising three boolean values, which is located
//	   in the program's resource file, which contains default preference values, and which
//	   is copied to the resource fork of a preferences file created when the program is
//	   run for the first time.  Thereafter, the 'PrFn' resource in the preferences file
//	   is used for the storage and retrieval of application preferences set by the user.
//
//    A 'WiSt' (window state) resource, which is created in the resource fork of the
//	   document file used by the program, and which is used to store the associated
//	   window's user state rectangle (a Rect value) and zoom state (a Boolean value).
//
//    A 'PrAr' (printable area) resource, which is created in the resource fork of the
//	   document file used by the program, and which is used to store the printable width
//	   and height of the paper size chosen in the print Style dialog box.		
//
//  }

program MoreResourcesPascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Quickdraw, Dialogs, QuickdrawText, Processes, Types, 
	Memory, Events, TextUtils, ToolUtils, OSUtils, Devices, Resources, StandardFile, 
	Folders, Printing, Files, Errors, Script, Controls, SegLoad;

{  define the following constants }

const

mApple = 128;
mFile = 129;
	iOpen = 2;
	iClose = 4;
	iPageSetup = 8;
	iQuit = 11;
mDemonstration = 131;
	iPreferences = 1;

rNewWindow = 128;
rMenubar = 128;
rAlertBox = 128;	
rModalDialog = 129;
	iSoundOn = 4;
	iFullScreenOn = 5;
	iAutoScrollOn = 6;
rStringList = 128;
	iPrefsFileName = 1;
rTypePrintRect = 'PrAr';
	kPrintRectID = 128;
rTypeWinState = 'WiSt';
	kWinStateID = 128;
rTypePrefs = 'PrFn';
	kPrefsID = 128;
rTypeAppMiss = 'STR ';
	kAppMissID = -16397;

kMaxLong = $7FFFFFFF;

{  user defined types }

type

DocRecord = record
	fileFSSpec : FSSpec;
	end;
DocRecordPointer = ^DocRecord;
DocRecordHandle = ^DocRecordPointer;

AppPrefs = record
	soundOn : boolean;
	fullScreenOn : boolean;
	autoScrollOn : boolean;
	end;
AppPrefsPointer = ^AppPrefs;
AppPrefsHandle = ^AppPrefsPointer;

WinState = record
	userStateRect : Rect;
	zoomState : boolean;
	end;
WinStatePtr = ^WinState;
WinStateHandle = ^WinStatePtr;

RectHandle = ^RectPtr;

{  global variables }

var

gDone : boolean;
gInBackground : boolean;
gTPrintHdl : THPrint;
gWindowPtr : WindowPtr;
gWindowOpen : boolean;
gPrintStyleChanged : boolean;
gPrintRect : Rect;
gSoundOn : boolean;
gFullScreenOn : boolean;
gAutoScrollOn : boolean;	
gAppResFileRefNum : integer;
gPrefsFileRefNum : integer;
menubarHdl : Handle;
menuHdl : MenuHandle;
eventRec : EventRecord;

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DoError }

procedure DoError(errorCode : integer);

	var
	errorString : string;
	ignored : OSErr;

	begin
	NumToString(errorCode, errorString);
	ParamText(errorString, '', '', '');

	if (errorCode = memFullErr) then
		begin
		ignored := StopAlert(rAlertBox, nil);
		ExitToShell;
		end	
	else
		ignored := CautionAlert(rAlertBox, nil);
	end;
		{of procedure DoError}

{  DoSavePrintableSize }

procedure DoSavePrintableSize(myWindowPtr : WindowPtr);

	var
	docRecHdl : DocRecordHandle;
	fileRefNum : integer;
	printRectHdl : RectHandle;
	osError : OSErr;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	fileRefNum := FSpOpenResFile(docRecHdl^^.fileFSSpec, fsRdWrPerm);
	if (fileRefNum < 0) then
		begin
		osError := ResError;
		DoError(osError);
		Exit(DoSavePrintableSize);
		end;

	printRectHdl := RectHandle(Get1Resource(rTypePrintRect, kPrintRectID));
	if (printRectHdl <> nil) then
		begin	
		printRectHdl^^ := gTPrintHdl^^.prInfo.rPage;
		ChangedResource(Handle(printRectHdl));
		osError := ResError;
		if (osError <> noErr) then
			DoError(osError);
		end
	else
		begin
		printRectHdl := RectHandle(NewHandle(sizeof(Rect)));
		if (printRectHdl <> nil) then
			begin
			printRectHdl^^ := gTPrintHdl^^.prInfo.rPage;
			AddResource(Handle(printRectHdl), rTypePrintRect, kPrintRectID, 
									'Print rectangle');
			end;
		end;

	if (printRectHdl <> nil) then
		begin
		UpdateResFile(fileRefNum);
		osError := ResError;
		if (osError <> noErr) then
			DoError(osError);

		ReleaseResource(Handle(printRectHdl));
		end;
	
	gPrintStyleChanged := false;

	CloseResFile(fileRefNum);
	end;
		{of procedure DoSavePrintableSize}

{  DoGetPrintableSize }

procedure DoGetPrintableSize(myWindowPtr : WindowPtr);

	var
	docRecHdl : DocRecordHandle;
	fileRefNum : integer;
	osError : OSErr;
	printRectHdl : RectHandle;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	fileRefNum := FSpOpenResFile(docRecHdl^^.fileFSSpec, fsRdWrPerm);
	if (fileRefNum < 0) then
		begin
		osError := ResError;
		DoError(osError);
		Exit(DoGetPrintableSize);
		end;

	printRectHdl := RectHandle(Get1Resource(rTypePrintRect, kPrintRectID));
	if (printRectHdl <> nil) then
		begin
		gPrintRect := printRectHdl^^;
		ReleaseResource(Handle(printRectHdl));
		end;

	CloseResFile(fileRefNum);
	end;
		{of procedure DoGetPrintableSize}

{  DoSetWindowState } 

procedure DoSetWindowState(myWindowPtr : WindowPtr; userStateRect, stdStateRect : Rect);

	var
	windowRecPtr : WindowPeek;
	winStateDataPtr : WStateDataPtr;

	begin
	windowRecPtr := WindowPeek(myWindowPtr);
	winStateDataPtr := WStateDataPtr(windowRecPtr^.dataHandle^);
	winStateDataPtr^.userState := userStateRect;
	winStateDataPtr^.stdState := stdStateRect;
	end;
		{of procedure DoSetWindowState}

{  DoSaveWindowPosition }

procedure DoSaveWindowPosition(myWindowPtr : WindowPtr);

	var
	docRecHdl : DocRecordHandle;
	fileRefNum : integer;
	windowRecPtr : WindowPeek;
	winStateDataPtr : WStateDataPtr;
	stdRect, userRect : Rect;
	contentRgnHdl : RgnHandle;
	userRectAndZoomState : WinState;
	winStateHdl : WinStateHandle;
	osError : OSErr;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	fileRefNum := FSpOpenResFile(docRecHdl^^.fileFSSpec, fsRdWrPerm);
	if (fileRefNum < 0) then
	 	begin
		osError := ResError;
		DoError(osError);
		Exit(DoSaveWindowPosition);
		end;

	windowRecPtr := WindowPeek(myWindowPtr);	
	winStateDataPtr := WStateDataPtr(windowRecPtr^.dataHandle^);		
	stdRect := winStateDataPtr^.stdState;
	userRect := winStateDataPtr^.userState;

	contentRgnHdl := windowRecPtr^.contRgn;
	userRectAndZoomState.userStateRect := contentRgnHdl^^.rgnBBox;
	userRectAndZoomState.zoomState := EqualRect(userRectAndZoomState.userStateRect, stdRect);
	if (userRectAndZoomState.zoomState) then
		userRectAndZoomState.userStateRect := userRect;

	winStateHdl := WinStateHandle(Get1Resource(rTypeWinState, kWinStateID));
	if (winStateHdl <> nil) then
		begin	
		winStateHdl^^ := userRectAndZoomState;
		ChangedResource(Handle(winStateHdl));
		osError := ResError;
		if (osError <> noErr) then
			DoError(osError);
		end
	else
		begin
		winStateHdl := WinStateHandle(NewHandle(sizeof(WinState)));
		if (winStateHdl <> nil) then
			begin
			winStateHdl^^ := userRectAndZoomState;
			AddResource(Handle(winStateHdl), rTypeWinState, kWinStateID, 
									'Last window state');
			end;
		end;

	if (winStateHdl <> nil) then
		begin
		UpdateResFile(fileRefNum);
		osError := ResError;
		if (osError <> noErr) then
			DoError(osError);

		ReleaseResource(Handle(winStateHdl));
		end;

	CloseResFile(fileRefNum);
	end;
		{of procedure DoSaveWindowPosition}

{  DoGetandSetWindowPosition }

procedure DoGetandSetWindowPosition(myWindowPtr : WindowPtr);

	var
	userStateRect, stdStateRect, displayRect : Rect;
	docRecHdl : DocRecordHandle;
	fileRefNum : integer;
	winStateHdl : WinStateHandle;
	gotResource : boolean;
	osError : OSErr;

	begin
	userStateRect := qd.screenBits.bounds;	
	SetRect(userStateRect, userStateRect.left + 3, userStateRect.top + 42, 
					userStateRect.right - 40, userStateRect.bottom - 6);

	stdStateRect := qd.screenBits.bounds;	
	SetRect(stdStateRect, stdStateRect.left + 3, stdStateRect.top + 42, 
					stdStateRect.right - 3, stdStateRect.bottom - 6);

	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	fileRefNum := FSpOpenResFile(docRecHdl^^.fileFSSpec, fsRdWrPerm);
	if (fileRefNum < 0) then
		begin
		osError := ResError;
		DoError(osError);
		Exit(DoGetandSetWindowPosition);
		end;

	winStateHdl := WinStateHandle(Get1Resource(rTypeWinState, kWinStateID));
	if (winStateHdl <> nil ) then
		begin
		gotResource := true;
		userStateRect := winStateHdl^^.userStateRect;
		end
	else
		gotResource := false;

	if (gotResource) then
		begin
		if (winStateHdl^^.zoomState) then
			displayRect := stdStateRect
		else
			displayRect := userStateRect;
		end
	else
		begin
		displayRect := userStateRect;
		end;

	MoveWindow(myWindowPtr, displayRect.left, displayRect.top, false);

	GlobalToLocal(displayRect.topLeft);
	GlobalToLocal(displayRect.botRight);
	SizeWindow(myWindowPtr, displayRect.right, displayRect.bottom, true);

	DoSetWindowState(myWindowPtr, userStateRect, stdStateRect);

	ReleaseResource(Handle(winStateHdl));
	CloseResFile(fileRefNum);
	end;
		{of procedure DoGetandSetWindowPosition}

{  DoSavePreferences }

procedure DoSavePreferences;

	var
	appPrefsHdl : AppPrefsHandle;
	existingResHdl : Handle;
	resourceName : string;

	begin
	resourceName := 'Preferences';
	if (gPrefsFileRefNum = -1) then
		Exit(DoSavePreferences);

	appPrefsHdl := AppPrefsHandle(NewHandleClear(sizeof(AppPrefs)));

	HLock(Handle(appPrefsHdl));

	appPrefsHdl^^.soundOn := gSoundOn;
	appPrefsHdl^^.fullScreenOn := gFullScreenOn;
	appPrefsHdl^^.autoScrollOn := gAutoScrollOn;

	UseResFile(gPrefsFileRefNum);

	existingResHdl := Get1Resource(rTypePrefs, kPrefsID);
	if (existingResHdl <> nil) then
		begin
		RemoveResource(existingResHdl);
		if (ResError = noErr) then
			AddResource(Handle(appPrefsHdl), rTypePrefs, kPrefsID, resourceName);
		if (ResError = noErr) then
			WriteResource(Handle(appPrefsHdl));
		end;

	HUnlock(Handle(appPrefsHdl));

	ReleaseResource(Handle(appPrefsHdl));
	UseResFile(gAppResFileRefNum);
	end;
		{of procedure DoSavePreferences}

{  DoCopyResource }

function DoCopyResource(rsrcType : ResType; resID, sourceFileRefNum, 
											destFileRefNum : integer) : OSErr;

	var
	oldResFileRefNum : integer;
	sourceResourceHdl : Handle;
	ignoredType : ResType;
	ignoredID : integer;
	resourceName : string;
	resAttributes : integer;
	osError : OSErr;

	begin
	oldResFileRefNum := CurResFile;
	UseResFile(sourceFileRefNum);

	sourceResourceHdl := Get1Resource(rsrcType, resID);

	if (sourceResourceHdl <> nil) then
		begin
		GetResInfo(sourceResourceHdl, ignoredID, ignoredType, resourceName);
		resAttributes := GetResAttrs(sourceResourceHdl);
		DetachResource(sourceResourceHdl);
		UseResFile(destFileRefNum);
		if (ResError = noErr) then
			AddResource(sourceResourceHdl, rsrcType, resID, resourceName);
		if (ResError = noErr) then
			SetResAttrs(sourceResourceHdl, resAttributes);
		if (ResError = noErr) then
			ChangedResource(sourceResourceHdl);
		if (ResError = noErr) then
			WriteResource(sourceResourceHdl);
		end;

	osError := ResError;

	ReleaseResource(sourceResourceHdl);
	UseResFile(oldResFileRefNum);

	DoCopyResource := osError;
	end;
		{of function DoCopyResource}

{  DoGetPreferences }

procedure DoGetPreferences;

	var
	prefsFileName : string;
	osError : OSErr;
	volRefNum : integer;
	directoryID : longint;
	fileSSpec : FSSpec;
	fileRefNum : integer;
	appPrefsHdl : AppPrefsHandle;

	begin
	GetIndString(prefsFileName, rStringList, iPrefsFileName);

	osError := FindFolder(kOnSystemDisk, kPreferencesFolderType, kDontCreateFolder, 
					volRefNum, directoryID);

	if (osError = noErr) then
		osError := FSMakeFSSpec(volRefNum, directoryID, prefsFileName, fileSSpec);
	if ((osError = noErr) or (osError = fnfErr)) then
		fileRefNum := FSpOpenResFile(fileSSpec, fsCurPerm);

	if (fileRefNum = -1) then
		begin
		FSpCreateResFile(fileSSpec, 'PpPp', 'pref', smSystemScript);
		osError := ResError;

		if (osError = noErr) then
			begin
			fileRefNum := FSpOpenResFile(fileSSpec, fsCurPerm);
			if (fileRefNum <> -1 ) then
				begin
				UseResFile(gAppResFileRefNum);
					
				osError := DoCopyResource(rTypePrefs, kPrefsID, gAppResFileRefNum, 
												fileRefNum);
				if (osError = noErr) then
					osError := DoCopyResource(rTypeAppMiss, kAppMissID, gAppResFileRefNum, 
														fileRefNum);
				if (osError <> noErr) then
					begin
					CloseResFile(fileRefNum);
					osError := FSpDelete(fileSSpec);
					fileRefNum := -1;
					end;
				end;
			end;
		end;

	if (fileRefNum <> -1) then
		begin
		UseResFile(fileRefNum); 

		appPrefsHdl := AppPrefsHandle(Get1Resource(rTypePrefs, kPrefsID));
		if (appPrefsHdl = nil) then
			Exit(DoGetPreferences);

		gSoundOn := appPrefsHdl^^.soundOn;
		gFullScreenOn := appPrefsHdl^^.fullScreenOn;
		gAutoScrollOn := appPrefsHdl^^.autoScrollOn;

		gPrefsFileRefNum := fileRefNum;

		UseResFile(gAppResFileRefNum);
		end;
	end;
		{of procedure DoGetPreferences}

{  DoPrintStyleDialog }

procedure DoPrintStyleDialog;

	begin
	PrOpen;

	if (PrStlDialog(gTPrintHdl)) then
		begin
		gPrintStyleChanged := true;
		gPrintRect := gTPrintHdl^^.prInfo.rPage;
		InvalRect(gWindowPtr^.portRect);
		end;

	PrClose;
	end;
		{of procedure DoPrintStyleDialog}

{  DoPreferencesDialog }

procedure DoPreferencesDialog;

	var
	modalDlgPtr : DialogPtr;
	oldPort : GrafPtr;
	oldPenState : PenState;
	buttonOval, itemHit, itemType, temp : integer;
	itemHdl : Handle;
	itemRect : Rect;

	begin
	modalDlgPtr := GetNewDialog(rModalDialog, nil, WindowPtr(-1));
	if (modalDlgPtr = nil) then
		Exit(DoPreferencesDialog);

	GetDialogItem(modalDlgPtr, iSoundOn, itemType, itemHdl, itemRect);
	if gSoundOn 
		then temp := 1
		else temp := 0;
	SetControlValue(ControlRef(itemHdl), temp);
	GetDialogItem(modalDlgPtr, iFullScreenOn, itemType, itemHdl, itemRect);
	if gFullScreenOn 
		then temp := 1
		else temp := 0;
	SetControlValue(ControlRef(itemHdl), temp);
	GetDialogItem(modalDlgPtr, iAutoScrollOn, itemType, itemHdl, itemRect);
	if gAutoScrollOn 
		then temp := 1
		else temp := 0;
	SetControlValue(ControlRef(itemHdl), temp);

	ShowWindow(modalDlgPtr);

	GetPort(oldPort);
	GetPenState(oldPenState);
	GetDialogItem(modalDlgPtr, 1, itemType, itemHdl, itemRect);
	SetPort(ControlHandle(itemHdl)^^.contrlOwner);
	InsetRect(itemRect, -4, -4);
	PenPat(qd.black);
	PenSize(3, 3);
	buttonOval := trunc((itemRect.bottom - itemRect.top) / 2) + 2;
	FrameRoundRect(itemRect, buttonOval, buttonOval);
	SetPenState(oldPenState);
	SetPort(oldPort);

	repeat
	ModalDialog(nil, itemHit);
	GetDialogItem(modalDlgPtr, itemHit, itemType, itemHdl, itemRect);
	if GetControlValue(ControlRef(itemHdl)) = 0
		then temp := 1
		else temp := 0;
	SetControlValue(ControlRef(itemHdl), temp);
	until ((itemHit = 1) or (itemHit = 2));

	if (itemHit = 1) then
		begin
		GetDialogItem(modalDlgPtr, iSoundOn, itemType, itemHdl, itemRect);
		if GetControlValue(ControlRef(itemHdl)) = 1
			then gSoundOn := true
			else gSoundOn := false;
			
		GetDialogItem(modalDlgPtr, iFullScreenOn, itemType, itemHdl, itemRect);
		if GetControlValue(ControlRef(itemHdl)) = 1
			then gFullScreenOn := true
			else gFullScreenOn := false;
			
		GetDialogItem(modalDlgPtr, iAutoScrollOn, itemType, itemHdl, itemRect);
		if GetControlValue(ControlRef(itemHdl)) = 1
			then gAutoScrollOn := true
			else gAutoScrollOn := false;
		end;

	DisposeDialog(modalDlgPtr);

	if (gWindowPtr <> nil) then
		InvalRect(gWindowPtr^.portRect);

	DoSavePreferences;
	end;
		{of procedure DoPreferencesDialog}

{  DoCloseCommand }

procedure DoCloseCommand;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecordHandle;
	osError : OSErr;

	begin
	osError := 0;
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	DoSaveWindowPosition(myWindowPtr);

	if (gPrintStyleChanged) then
		DoSavePrintableSize(gWindowPtr);

	DisposeHandle(Handle(docRecHdl));
	DisposeWindow(myWindowPtr);
	gWindowOpen := false;
	end;
		{of procedure DoCloseCommand}

{  DoOpenCommand }

procedure DoOpenCommand;

	var
	fileTypes : SFTypeList;
	fileReply : StandardFileReply;
	docRecHdl : DocRecordHandle;
	osError : OSErr;

	begin
	osError := 0;
	fileTypes[0] := 'TEXT';
	
	StandardGetFile(nil, 1, @fileTypes, fileReply);
	if not (fileReply.sfGood) then
		Exit(DoOpenCommand);

	gWindowPtr := GetNewWindow(rNewWindow, nil, WindowPtr(-1));
	if (gWindowPtr = nil) then
		Exit(DoOpenCommand);

	docRecHdl := DocRecordHandle(NewHandle(sizeof(DocRecord)));
	if (docRecHdl = nil) then
		begin
		DisposeWindow(gWindowPtr);
		Exit(DoOpenCommand);
		end;

	gWindowOpen := true; 
	SetPort(gWindowPtr);

	SetWRefCon(gWindowPtr, longint(docRecHdl));
	docRecHdl^^.fileFSSpec := fileReply.sfFile;
	SetWTitle(gWindowPtr, docRecHdl^^.fileFSSpec.name);	

	DoGetandSetWindowPosition(gWindowPtr);
	DoGetPrintableSize(gWindowPtr);

	ShowWindow(gWindowPtr);
	end;
		{of procedure DoOpenCommand}

{  InvalidateScrollBarArea }

procedure InvalidateScrollBarArea(myWindowPtr : WindowPtr);

	var
	tempRect : Rect;

	begin
	SetPort(myWindowPtr);

	tempRect := myWindowPtr^.portRect;
	tempRect.left := tempRect.right - 15;
	InvalRect(tempRect);

	tempRect := myWindowPtr^.portRect;
	tempRect.top := tempRect.bottom - 15;
	InvalRect(tempRect);
	end;
		{of procedure InvalidateScrollBarArea}

{  DoFileMenu }

procedure DoFileMenu(menuItem : integer);

	begin
	case (menuItem) of

		iClose:
			begin
			DoCloseCommand;
			end;

		iOpen:
			begin
			DoOpenCommand;
			end;

		iPageSetup:
			begin
			DoPrintStyleDialog;
			end;

		iQuit:
			begin
			while (FrontWindow <> nil) do
				DoCloseCommand;
			gDone := true;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoFileMenu}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : integer;
	itemName : string;
	daDriverRefNum : integer;
	
	begin
	menuID := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);

	if (menuID = 0) then
		Exit(DoMenuChoice);

	case (menuID) of

		mApple:
			begin
			GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
			daDriverRefNum := OpenDeskAcc(itemName);
			end;

		mFile:
			begin
			DoFileMenu(menuItem);
			end;

		mDemonstration:
			begin
			if (menuItem = iPreferences) then
				DoPreferencesDialog;
			end;
		end;
			{of case statement}

	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}

{  DoAdjustMenus }

procedure DoAdjustMenus;

	var
	menuHdl : MenuHandle;

	begin
	if (gWindowOpen) then
		begin
		menuHdl := GetMenuHandle(mFile);
		DisableItem(menuHdl, iOpen);
		EnableItem(menuHdl, iClose);
		EnableItem(menuHdl, iPageSetup);
		end
	else
		begin
		menuHdl := GetMenuHandle(mFile);
		EnableItem(menuHdl, iOpen);
		DisableItem(menuHdl, iClose);
		DisableItem(menuHdl, iPageSetup);
		end;

	DrawMenuBar;
	end;
		{of procedure DoAdjustMenus}

{  DoUpdateWindow }

procedure DoUpdateWindow(myWindowPtr : WindowPtr);

	var
	str : string;
	
	begin
	SetPort(myWindowPtr);
	
	MoveTo(10, 20);
	DrawString('Current Application Preferences:');
	MoveTo(10, 35);
	DrawString('Sound On: ');
	if (gSoundOn) then 
		DrawString('YES')
	else  
		DrawString('NO');
		
	MoveTo(10, 50);
	DrawString('Full Screen On: ');
	if (gFullScreenOn) then 
		DrawString('YES')
	else  
		DrawString('NO');
		
	MoveTo(10, 65);
	DrawString('AutoScroll On: ');
	if (gAutoScrollOn) then 
		DrawString('YES')
	else  
		DrawString('NO');

	if (gPrintRect.bottom <> 0) then
		begin
		MoveTo(10, 85);
		DrawString('Information from printable area ("PrAr") resource:'); 
		NumToString(longint(gPrintRect.bottom), str);
		MoveTo(10, 100);
		DrawString('Page print area height in screen pixels: ');
		DrawString(str);
		NumToString(longint(gPrintRect.right), str);
		MoveTo(10, 115);
		DrawString('Page print area width in screen pixels:  ');
		DrawString(str);
		end
	else
		begin
		MoveTo(10, 85);
		DrawString('No printable area ("PrAr") resource saved yet'); 
		end;
		
	end;
		{of procedure DoUpdateWindow}

{  DoMouseDown }

procedure DoMouseDown(var eventRec : EventRecord);

	var
	myWindowPtr : WindowPtr;
	partCode : integer;
	growRect : Rect;
	newSize : longint;

	begin
	partCode := FindWindow(eventRec.where, myWindowPtr);
	
	case (partCode) of

		inMenuBar:
			begin
			DoAdjustMenus;
			DoMenuChoice(MenuSelect(eventRec.where));
			end;

		inSysWindow:
			begin
			SystemClick(eventRec, myWindowPtr);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow) then
				SelectWindow(myWindowPtr);
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, eventRec.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, eventRec.where)) then
				DoCloseCommand;
			end;

		inGrow:
			begin
			growRect := qd.screenBits.bounds;
			growRect.top := 145; 
			growRect.left := 345;
			newSize := GrowWindow(myWindowPtr, eventRec.where, growRect);
			if (newSize <> 0) then
				begin
				InvalidateScrollBarArea(myWindowPtr);
				SizeWindow(myWindowPtr, LoWord(newSize), HiWord(newSize), true);
				InvalidateScrollBarArea(myWindowPtr);
				end;
			end;

		inZoomIn, inZoomOut:
			begin
			if (TrackBox(myWindowPtr, eventRec.where, partCode)) then
				begin
				SetPort(myWindowPtr);
				EraseRect(myWindowPtr^.portRect);
				ZoomWindow(myWindowPtr, partCode, false);
				InvalRect(myWindowPtr^.portRect);
				end;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoMouseDown}

{  DoEvents }

procedure DoEvents(var eventRec : EventRecord);

	var
	myWindowPtr : WindowPtr;
	charCode : char;

	begin
	myWindowPtr := WindowPtr(eventRec.message);		

	case (eventRec.what) of

		mouseDown:
			begin
			DoMouseDown(eventRec);
			end;

		keyDown, autoKey:
			begin
			charCode := chr(BAnd(eventRec.message, charCodeMask));
			if (BAnd(eventRec.modifiers, cmdKey) <> 0) then
				begin
				DoAdjustMenus;
				DoMenuChoice(MenuKey(charCode));
				end;
			end;

		updateEvt:
			begin
			BeginUpdate(myWindowPtr);
			EraseRgn(myWindowPtr^.visRgn);
			DoUpdateWindow(myWindowPtr);
			DrawGrowIcon(myWindowPtr);
			EndUpdate(myWindowPtr);
			end;

		osEvt:
			begin
			case (BAnd(BSR(eventRec.message, 24), $000000FF)) of
			
				suspendResumeMessage:
					begin
					gInBackground := (BAnd(eventRec.message, resumeFlag) = 0);
					end;
				end;
					{of inner case statement}
			HiliteMenu(0);
			end;
		end;
			{of outer case statement}
	end;
		{of procedure DoEvents}
	
{  start of main program }

begin

	{  initialize managers }

	DoInitManagers;
	gWindowOpen := false;
	gPrintStyleChanged := false;
	gPrefsFileRefNum := 0;

	{  set current resource file to application resource fork }

	gAppResFileRefNum := CurResFile;

	{  set up menu bar and menus }

	menubarHdl := GetNewMBar(rMenubar);
	if (menubarHdl = nil) then
		DoError(MemError);
	SetMenuBar(menubarHdl);
	DrawMenuBar;

	menuHdl := GetMenuHandle(mApple);
	if (menuHdl = nil) then
		DoError(MemError)
	else
		AppendResMenu(menuHdl, 'DRVR');

	{  create and initialize a TPrint record }

	PrOpen;
	gTPrintHdl := THPrint(NewHandleClear(sizeof(TPrint)));
	PrintDefault(gTPrintHdl);
	PrClose;

	{  read in application preferences }

	DoGetPreferences;

	{  enter event loop }
	
	gDone := false;
	
	while not (gDone) do
		begin
		if (WaitNextEvent(everyEvent, eventRec, kMaxLong, nil)) then
			DoEvents(eventRec);
		end;
end.
	
{  }
